package pl.edu.pjwstk.yaod.atoms;

import java.io.*;

import pl.edu.pjwstk.yaod.system.buffers.*;

/**
 * Klasa reprezentuje atom, czyli strukture danych reprezentujaca
 * obiekt warstwy logicznej.
 * TODO: serializacja atomow za pomoca wlasnego mechanizmu (nie Javy)
 * zapewni kilkukrotne zmniejszenie objetosci zserializowanego atomu.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public abstract class Atom implements Serializable, Bufferable {
  public final static byte ALL_TYPES = -1;

  private String m_name;
  private byte m_type;

  private AtomID m_aid, m_owner, m_next, m_previous;

  private int m_uses;
  private boolean m_modified;

/**
 * Tworzy nowy atom.
 * @param name nazwa atomu.
 * @return nowy atom.
 */
  public Atom(String name) {
    m_name = name;
  }

/**
 * Zwraca identyfikator atomu.
 * @return identyfikator atomu.
 */
  public AtomID getAID() {
    return m_aid;
  }

/**
 * Ustala identyfikator atomu.
 * @param aid nowy identyfikator atomu.
 */
  public void setAID(AtomID aid) {
    m_aid = aid;
  }

/**
 * Ustala atom nadrzedny.
 * @param ptr identyfikator atomu nadrzednego.
 */
  public void setOwner(AtomID ptr) {
    m_owner = ptr;
    setModified(true);
  }

/**
 * Zwraca identyfikator atomu nadrzednego.
 * (nie dotyczy niektorych typow atomow nadrzednych).
 * @return identyfikator atomu nadrzednego.
 */
  public AtomID getOwner() {
    return m_owner;
  }

/**
 * Ustala atom poprzedni w grafie warstwy logicznej.
 * (nie dotyczy niektorych typow atomow nadrzednych).
 * @param ptr identyfikator poprzedniego atomu.
 */

  public void setPrevious(AtomID ptr) {
    m_previous = ptr;
    setModified(true);
  }

/**
 * Pobiera identyfikator poprzedniego atomu w grafie warstwy logicznej
 * (nie dotyczy niektorych typow atomow nadrzednych).
 * @return identyfikator poprzedniego atomu.
 */
  public AtomID getPrevious() {
    return m_previous;
  }

/**
 * Ustala nastepny atom w grafie warstwy logicznej
 * (nie dotyczy niektorych typow atomow nadrzednych).
 * @param ptr identyfikator nastepnego atomu.
 */
  public void setNext(AtomID ptr) {
    m_next = ptr;
    setModified(true);
  }

/**
 * Pobiera nastepny atom w grafie warstwy logicznej
 * (nie dotyczy niektorych typow atomow nadrzednych).
 * @return identyfikator nastepnego atomu.
 */
  public AtomID getNext() {
    return m_next;
  }

/**
 * Zwraca nazwe atomu.
 * @return nazwa atomu.
 */
  public String getName() {
    return m_name;
  }

/**
 * Ustala nowa nazwe atomu.
 * @param newName nowa nazwa atomu.
 */
  public void setName(String newName) {
    m_name = newName;
    setModified(true);
  }

/**
 * Zwraca typ atomu. Obecnie nie dziala - sprawdzanie typu za
 * pomoca instanceof.
 * @return identyfikator typu atomu.
 */
  public int getType() {
    System.out.println("Atom.getType() nie dziala");

    return 0;
  }

  // bufferable

/**
 * Ustala flage mowiaca, czy atom zostal zmodyfikowany
 * i nalezy zapisac go na dysku przy najblizszym punkcie kontrolnym.
 * @param val zmodyfikowany (true), nie (false)
 */
  public void setModified(boolean val) {
    m_modified = val;
  }

/**
 * Sprawdza czy atom jest brudny (wymaga zapisania na dysku).
 * @return flaga oznaczajaca czy atom wymaga zapisu na dysku.
 */
  public boolean isModified() {
    return m_modified;
  }
}
