package pl.edu.pjwstk.yaod.network;

import java.io.*;

import java.net.*;

import java.util.*;

import javax.swing.tree.*;

import pl.edu.pjwstk.yaod.debug.*;
import pl.edu.pjwstk.yaod.network.protocol.*;
import pl.edu.pjwstk.yaod.objects.*;
import pl.edu.pjwstk.yaod.store.*;
import pl.edu.pjwstk.yaod.system.processes.*;
import pl.edu.pjwstk.yaod.util.*;
import pl.edu.pjwstk.yaod.xml.*;
import pl.edu.pjwstk.yaod.yaql.*;

/**
 * Klasa zapewnia mozliwosc komunikacji miedzy klientem i serwerem
 * za pomoca protokolu Y.NET.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class YNETCommunicator implements Communicable {
  protected final static int BUFSIZE = 8192;

  private byte[] readBuffer = new byte[BUFSIZE];

  private Socket m_socket;
  private InputStream m_input;
  private OutputStream m_output;
  private SVRP m_server;

/**
 * Otwiera polaczenie serwera z klientem.
 * @param socket gniazdko klienta, ktory polaczyl sie z serwerem.
 * @exception IOException blad komunikacji
 * @return nowy obiekt zarzadzajacy komunikacja.
 */
  public YNETCommunicator(Socket socket, SVRP server) throws IOException {
    m_socket = socket;
    m_server = server;
    m_output = m_socket.getOutputStream();
    m_input = m_socket.getInputStream();
  }

/**
 * Otwiera polaczenie klienta z serwerem.
 * @param host nazwa hosta, gdzie nasluchuje serwer.
 * @param port nazwa portu, na ktorym nasluchuje serwer.
 * @exception IOException blad komunikacji.
 * @return nowy obiekt zarzadzajacy komunikacja.
 */
  public YNETCommunicator(String host, int port) throws IOException {
    m_socket = new Socket(host, port);
    m_output = m_socket.getOutputStream();
    m_input = m_socket.getInputStream();
  }

/**
 * Rozlacza klienta z serwerem.
 */
  public void disconnect() {
    try {
      m_socket.close();
    }
    catch (IOException e) {
    }
  }

/**
 * Wysyla komunikat zgodny z protokolem Y.NET.
 * @param msg komunikat do przeslania.
 * @exception IOException blad komunikacji.
 */
  public void putMessage(NetMessage msg) throws IOException {
    byte[] rawCmd = new ObjectSerializer().objectSerialize(msg);

    byte[] rawCmdLength = new byte[NewBits.INT_SIZE];
    NewBits.putInt(rawCmdLength, 0, rawCmd.length);
    
    m_output.write(rawCmdLength);
    m_output.write(rawCmd);
  }

/**
 * Odbiera komunikat zgodny z protokolem Y.NET.
 * @exception IOException blad w komunikacji.
 * @return odebrany komunikat.
 */
  public NetMessage getMessage() throws IOException {  
    byte[] rawMsgLength = new byte[NewBits.INT_SIZE];
    m_input.read(rawMsgLength);

    int msgLength = NewBits.getInt(rawMsgLength, 0);

    byte[] rawMsg = new byte[msgLength];
    int readSize = 0;

    while (readSize < msgLength)
      readSize += m_input.read(rawMsg, readSize, msgLength - readSize);

    if (readSize == -1) {
      System.out.println("Polaczenie zamkniete");
      return null;
    }
    else {
      NetMessage replyMsg =
        (NetMessage) new ObjectSerializer().objectDeserialize(rawMsg);
      return replyMsg;
    }
  }

  public void serveRequest() throws IOException {
    ClientRequest request = (ClientRequest) getMessage();
    ServerReply reply = (ServerReply) serveYNetRequest(request);
    putMessage(reply);
  }

/**
 * Obsluguje zadanie klienta przeslane przez siec za pomoca
 * protokolu Y.NET.
 * @param req zadanie klienta.
 * @return odpowiedz serwera na wykonanie zadania.
 */
  private ServerReply serveYNetRequest(ClientRequest req) { 
    if (req instanceof QueryRequest) {
      try {
        QueryRequest qreq = (QueryRequest) req;

        Vector result = m_server.execute(m_server.getInterpreter(), qreq.getQuery());

        DefaultMutableTreeNode ast = m_server.getInterpreter().dumpAST();
        DefaultMutableTreeNode envs = m_server.getInterpreter().dumpENVS();

        QueryReply reply = new QueryReply(result, ast, envs);

        return reply;       
      }
      catch (SBQLException e) {
        ErrorReply reply = new ErrorReply(e.getMessage());
        return reply;
      }
    }
    else if (req instanceof DumpRequest) {
      DumpRequest dreq = (DumpRequest) req;

      try {
        DefaultMutableTreeNode dump = new AtomExtractor().dumpStore(m_server, (short) 0);
        return new DumpReply(dump);
      }
      catch (Exception e) {
        return new ErrorReply(e.getMessage());
      }
    }
    else if (req instanceof XMLLoadRequest) {
      XMLLoadRequest xmlreq = (XMLLoadRequest) req;

      XMLLoader xreader = new XMLLoader(m_server.getObjectManager());

      try {
        xreader.loadDocument(xmlreq.getDocument(),
                 (ObjectID) ((Wrappable) m_server.getStores().nextElement()).getEntry());

        m_server.getInterpreter().initializeENVS();
        return new ServerReply("Document loaded successfully.");
      }
      catch (Exception e) {
        e.printStackTrace();
        return new ErrorReply(e.getMessage());
      }
    }
    else {
      ErrorReply reply = new ErrorReply("Unknown client request.");

      return reply;
    }
  }
}
