package pl.edu.pjwstk.yaod.objects;

import pl.edu.pjwstk.yaod.atoms.*;
import pl.edu.pjwstk.yaod.store.*;
import pl.edu.pjwstk.yaod.system.processes.*;

/**
 * Tworzy i odczytuje obiekty.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class ObjectManager  {
  private SVRP m_server;

/**
 * Tworzy nowego menedzera obiektow.
 * @param sp proces uslugowy, ktory zapewnial bedzie obsluge czynnosci
 * zaimplementowanych w menedzerze obiektow.
 * @return nowy menedzer.
 */

  public ObjectManager(SVRP sp) {
    m_server = sp;
  }

/**
 * Zwraca obiekt identyfikowany podanym oid.
 * @param ptr oid szukanego obiektu.
 * @exception NoSuchObjectException obiekt o podanym identyfikatorze nie istnieje.
 * @return znaleziony obiekt.
 */
  public YObject getObject(ObjectID ptr) throws NoSuchObjectException {
    if (ptr == null) return null;

    try {  
      Wrappable store = m_server.getStoreByID(ptr.getStoreID());
      Atom atm  = store.retrieveAtom((AtomID) ptr);

      if (atm instanceof ProcedureAtom)
        return new YProcedure(m_server, (ProcedureAtom) atm);
      else
      if (atm instanceof JavaStringAtom)
        return new YJavaString(m_server, (JavaStringAtom)atm);
      else
      if (atm instanceof JavaIntegerAtom)
        return new YJavaInteger(m_server, (JavaIntegerAtom)atm);
      else
      if (atm instanceof JavaDoubleAtom)
        return new YJavaDouble(m_server, (JavaDoubleAtom)atm);
      else
      if (atm instanceof SpiderAtom)
        return new YSpider(m_server, (SpiderAtom) atm);
      else
      if (atm instanceof StringAtom)
        return new YString(m_server, (StringAtom) atm);
      else
      if (atm instanceof IntegerAtom)
        return new YInteger(m_server, (IntegerAtom) atm);
      else
      if (atm instanceof DoubleAtom)
        return new YDouble(m_server, (DoubleAtom) atm);
      else
      if (atm instanceof RingAtom)
        return new YRing(m_server, (RingAtom)atm);
      else
      if (atm instanceof ReferenceAtom)
        return new YReference(m_server, (ReferenceAtom) atm);
      else
      if (atm instanceof BackwardAtom)
        return new YBackward(m_server, (BackwardAtom) atm);
      else
      if (atm instanceof BinaryAtom)
        return new YBinary(m_server, (BinaryAtom) atm);
      else {
        pl.edu.pjwstk.yaod.util.Errors.printError(1, "getObject()");
        throw new NoSuchObjectException();
      }
    }
    catch (NoSuchAtomException e) {
      throw new NoSuchObjectException();
    }
  }

/**
 * Tworzy nowy obiekt YString (nie jest przylaczony do bazy). 
 * @param name nazwa obiektu.
 * @param value wartosc obiektu.
 * @return nowy obiekt YString.
 */
  public YString createString(String name, String value) {
    return new YString(m_server, name, value);
  }

/**
 * Tworzy nowy obiekt YSpider (nie jest przylaczony do bazy). 
 * @param name nazwa obiektu.
 * @return nowy obiekt YSpider.
 */
  public YSpider createSpider(String name) {
    return new YSpider(m_server, name);
  }

/**
 * Tworzy nowy obiekt YRing (nie jest przylaczony do bazy). 
 * @param name nazwa obiektu.
 * @return nowy obiekt YRing.
 */
  public YRing createRing(String name) {
    return new YRing(m_server, name);
  }

/**
 * Tworzy nowy obiekt YReference (nie jest przylaczony do bazy). 
 * @param name nazwa obiektu.
 * @return nowy obiekt YReference.
 */
  public YReference createReference(String name, ObjectID target) {
    return new YReference(m_server, name, target);
  }

/**
 * Tworzy nowy obiekt YInteger (nie jest przylaczony do bazy). 
 * @param name nazwa obiektu.
 * @param val wartosc obiektu.
 * @return nowy obiekt YInteger.
 */
  public YInteger createInteger(String name, int val) {
    return new YInteger(m_server, name, val);
  }

/**
 * Tworzy nowy obiekt YDouble (nie jest przylaczony do bazy). 
 * @param name nazwa obiektu.
 * @param val wartosc obiektu.
 * @return nowy obiekt YDouble.
 */
  public YDouble createDouble(String name, double val) {
    return new YDouble(m_server, name, val);
  }

/**
 * Tworzy nowy obiekt YProcedure (nie jest przylaczony do bazy). 
 * @param name nazwa procedury.
 * @return nowy obiekt YProcedure.
 */
  public YProcedure createProcedure(String name) {
    return new YProcedure(m_server, name);
  }

/**
 * Tworzy nowy obiekt YBinary (nie jest przylaczony do bazy). 
 * @param name nazwa obiektu.
 * @param value wartosc obiektu.
 * @return nowy obiekt YBinary.
 */
  public YBinary createBinary(String name, byte[] value) {
    return new YBinary(m_server, name, value);
  }

/**
 * Tworzy nowy obiekt YJavaString (nie jest przylaczony do bazy).
 * Dotyczy tylko skladu obiektow Javy.
 * @param name nazwa obiektu.
 * @param obj obiekt, ktorego atrybut bedzie widziany w skladzie.
 * @param atrybut obiektu obj, ktorego wartosc bedzie widziana w skladzie.
 * @return nowy obiekt YJavaString.
 */
  public YJavaString createJavaString(String name, Object obj, String attr) throws NoSuchFieldException {
    return new YJavaString(m_server, name, obj, attr);
  }

/**
 * Tworzy nowy obiekt YJavaInteger (nie jest przylaczony do bazy).
 * Dotyczy tylko skladu obiektow Javy.
 * @param name nazwa obiektu.
 * @param obj obiekt, ktorego atrybut bedzie widziany w skladzie.
 * @param atrybut obiektu obj, ktorego wartosc bedzie widziana w skladzie.
 * @return nowy obiekt YJavaString.
 */
  public YJavaInteger createJavaInteger(String name, Object obj, String attr) throws NoSuchFieldException {
    return new YJavaInteger(m_server, name, obj, attr);
  }

/**
 * Tworzy nowy obiekt YJavaDouble (nie jest przylaczony do bazy).
 * Dotyczy tylko skladu obiektow Javy.
 * @param name nazwa obiektu.
 * @param obj obiekt, ktorego atrybut bedzie widziany w skladzie.
 * @param atrybut obiektu obj, ktorego wartosc bedzie widziana w skladzie.
 * @return nowy obiekt YJavaDobule.
 */
  public YJavaDouble createJavaDouble(String name, Object obj, String attr) throws NoSuchFieldException {
    return new YJavaDouble(m_server, name, obj, attr);
  }

}
