package pl.edu.pjwstk.yaod.objects;

import pl.edu.pjwstk.yaod.atoms.*;
import pl.edu.pjwstk.yaod.store.defaultstore.blocks.*;
import pl.edu.pjwstk.yaod.system.processes.*;
import pl.edu.pjwstk.yaod.util.*;

/**
 * Klasa, ktorej zadaniem jest dodanie zachowania do statycznych struktur,
 * jakimi sa atomy. Kazdy z obiektow YObject zawiera referencj do
 * reprezentowanego przez niego atomu. Podczas gdy w systemie moze istniec
 * tylko jeden obiekt typu Atom na rekord, obiektow typu YObject moze byc nieskonczona
 * ilosc (rozroznienie to ma znaczenie przede wszystkim dla wspolbieznosci).
 * Czesc metod realizowana jest poprzez delegacje (np. pobieranie nazwy),
 * a czesc dodaje nowa funkcjonalnosc (np. zarzadzanie referencjami zwrotnymi).
 * Kazda zmiana w obiekcie YObject jest natychmiast nanoszona w atomie.
 * Modul zarzadzania wspolbieznoscia powinien w przyszlosci zapewnic
 * blokowanie atomu przez rownolegle piszace/czytajace instancje klasy YObject.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public abstract class YObject {
  private SVRP m_server;
  private Atom m_atom;
  private ObjectManager m_obman;

/**
 * Konstruktor.
 * @param sp proces usugowy.
 * @param atm atom, ktorym zarzadza dany obiekt.
 * @return utworzony obiekt.
 */

  YObject(SVRP sp, Atom atm) {
    m_server = sp;
    m_atom = atm;
    m_obman = getServer().getObjectManager();
  }

/**
 * Zwraca atom, ktory reprezentuje dany obiekt.
 * @return atom obiektu.
 */

  protected Atom getAtom() {
    return m_atom;
  }

/**
 * Zwraca obiekt procesu usugowego, ktry przypisano danemu obiektowi.
 * @return proces usugowy.
 */

  protected SVRP getServer() {
    return m_server;
  }

/**
 * Zwraca nazwe atomu, ktory reprezentuje dany obiekt (lub inaczej:
 * zwraca nazwe obiektu).
 * @return nazwa atomu (obiektu).
 */

  public String getName() {
    return m_atom.getName();
  }

/**
 * Ustala nazwe obiektu.
 * @param name nazwa obiektu.
 */

  public void setName(String name) {
    m_atom.setName(name);
  }

/**
 * Zwraca identyfikator obiektu (identyfikator atomu).
 * @return identyfikator obiektu.
 */

  public ObjectID getOID() {
    return (ObjectID) m_atom.getAID();
  }

/**
 * Zwraca obiekt nadrzedny.
 * @return identyfikator obiektu nadrzednego.
 */

  public ObjectID getOwner() {
    if (m_atom.getOwner() != null)
      return (ObjectID) m_atom.getOwner();

    if (m_atom.getNext() != null)
      return null;

    Errors.printError(2, "Zaimplementuj getOwner()");

    return null;
  }

/**
 * Zwraca nastepny obiekt na tym samym poziomie drzewa obiektow.
 * @return identyfikator nastepnego obiektu.
 */

  public ObjectID getNextSibling() {
    try {
      if (m_atom.getNext() != null)
        return (ObjectID) m_atom.getNext();
      else {
        ObjectID ownerID = getOwner();
        YComplexObject owner = (YComplexObject) m_obman.getObject(ownerID);

        return owner != null ? owner.getNextMember(getOID()) : null;
      }
    }
    catch (NoSuchObjectException e) {
      Errors.printError(1, "Nonexistent owner (getNextSibling()).");
    }

    return null;
  }

/**
 * Zwraca poprzedni obiekt na tym samym poziomie drzewa obiektow.
 * @return identyfikator poprzedniego obiektu.
 */

  public ObjectID getPreviousSibling() {
    try {
      if (m_atom.getPrevious() != null)
        return (ObjectID) m_atom.getPrevious();
      else {
        ObjectID ownerID = getOwner();
        YComplexObject owner = (YComplexObject) m_obman.getObject(ownerID);

        return owner != null ? owner.getPreviousMember(getOID()) : null;
      }
    }
    catch (NoSuchObjectException e) {
      Errors.printError(1, "Nonexistent owner (getPreviousSibling()).");
    }

    return null;
  }

/**
 * Podlacza obiekt do obiektu nadrzednego o podanym identyfikatorze.
 * @param ownerID identyfikator obiektu nadrzednego.
 * @return NoSuchObjectException brak obiektu o podanym identyfikatorze.
 * @return BlockFreeSpaceException brak wolnego miejsca.
 */

  public void connect(ObjectID ownerID)
      throws NoSuchObjectException, BlockFreeSpaceException {
    YObject obj = m_obman.getObject(ownerID);
    YComplexObject root = (YComplexObject) obj;

    if (root == null)
      throw new NoSuchObjectException();

    root.connectMember(this);
  }

/**
 * Podlacza obiekt do obiektu nadrzednego o podanym identyfikatorze
 * w taki sposob, by byl on umieszczony za pewnym jego podobiektem.
 * @param ownerID identyfikator obiektu nadrzednego.
 * @param afterID identtyfikator obiektu po ktorym podpiac obiekt.
 * @return NoSuchObjectException brak obiektu o podanym identyfikatorze.
 * @return BlockFreeSpaceException brak wolnego miejsca.
 */

  void connectAfter(ObjectID ownerID, ObjectID afterID)
      throws NoSuchObjectException, BlockFreeSpaceException {
    YComplexObject root = (YComplexObject) m_obman.getObject(ownerID);

    if (root == null)
      throw new NoSuchObjectException();

    root.connectMemberAfter(this, afterID);
  }

/**
 * Podlacza obiekt do obiektu nadrzednego o podanym identyfikatorze
 * w taki sposob, by byl on umieszczony przed pewnym jego podobiektem.
 * @param ownerID identyfikator obiektu nadrzednego.
 * @param afterID identyfikator obiektu przed ktorym podpiac obiekt.
 * @return NoSuchObjectException brak obiektu o podanym identyfikatorze.
 * @return BlockFreeSpaceException brak wolnego miejsca.
 */

  void connectBefore(ObjectID ownerID, ObjectID afterID)
      throws NoSuchObjectException, BlockFreeSpaceException {
    YComplexObject root = (YComplexObject) m_obman.getObject(ownerID);

    if (root == null)
      throw new NoSuchObjectException();

    root.connectMemberBefore(this, afterID);
  }

/**
 * Odlacza obiekt od biezacego obiektu nadrzednego i podlacza
 * go do innego obiektu.
 * @param newRootID identyfikator nowego obiektu nadrzednego.
 * @exception NoSuchObjectException brak jakiegos obiektu wskazanego
 * jego identyfikatorem.
 * TODO: pozostale atomy zlozone
 */
  public void reconnect(ObjectID newRootID) throws NoSuchObjectException {
    YComplexObject currRoot = (YComplexObject) m_obman.getObject(getOwner());
    YComplexObject newRoot = (YComplexObject) m_obman.getObject(newRootID);
    YBackward backward = getBackward();

    ((YSpider) currRoot).unregisterMember(getOID());
    ((YSpider) newRoot).registerMember(getOID());

    getAtom().setOwner((AtomID) newRootID);

    if (backward != null)
      backward.reconnectAfter(newRootID, getOID());
  }

/**
 * Odlacza obiekt od biezacego obiektu nadrzednego i podlacza
 * go do innego obiektu.
 * @param newRootID identyfikator nowego obiektu nadrzednego.
 * @param afterID identyfikator obiektu podrzednego po ktorym
 * obiekt ma zostac umieszczony.
 * @exception NoSuchObjectException brak jakiegos obiektu wskazanego
 * jego identyfikatorem.
 */

  public void reconnectAfter(ObjectID newRootID, ObjectID afterID) throws NoSuchObjectException {
    YComplexObject currRoot = (YComplexObject) m_obman.getObject(getOwner());
    YComplexObject newRoot = (YComplexObject) m_obman.getObject(newRootID);

    ((YSpider) currRoot).unregisterMember(getOID());
    ((YSpider) newRoot).registerMemberAfter(getOID(), afterID);

    getAtom().setOwner((AtomID) newRootID);
  }

/**
 * Odlacza obiekt od obiektu nadrzednego. Najpierw kasuje wszystkie
 * referencje do danego obiektu, a pozniej kasuje wszystkie obiekty
 * podrzedne oraz obiekt je zawierajacy.
 * @exception NoSuchObjectException brak w skladzie jakiegos obiektu.
 */

  public void disconnect() throws NoSuchObjectException {
    disconnectBackward();

    YComplexObject root = (YComplexObject) m_obman.getObject(getOwner());

    if (root != null) {
      if (this instanceof YComplexObject) {
        ObjectID objid;
        while ( (objid = ((YSpider) this).getFirstSubordinate()) != null) {
          YObject obj = m_obman.getObject(objid);
          try {
            obj.disconnect();
          }
          catch (NoSuchObjectException e) {
          }
        }
      }
    }

    root.disconnectMember(getOID());
  }

/**
 * Kasuje obiekt referencji zwrotnych danego obiektu.
 */
  private void disconnectBackward() throws NoSuchObjectException {
    YBackward backward = getBackward();

    if (backward != null)
      backward.disconnect();
  }

/**
 * Zwraca obiekt referencji zwrotnych przyporzadkowany danemu obiektowi.
 * @return obiekt referencji zwrotnych.
 */

  public YBackward getBackward() {
    try {
      ObjectID siblingID = getNextSibling();
      YObject sibling = m_obman.getObject(siblingID);

      return sibling instanceof YBackward ? (YBackward) sibling : null;
    }
    catch (NoSuchObjectException e) {
      Errors.printError(1, "Nonexistent backward references object (getBackward())");
    }

    return null;
  }

}
