package pl.edu.pjwstk.yaod.objects;

import pl.edu.pjwstk.yaod.atoms.*;
import pl.edu.pjwstk.yaod.store.defaultstore.blocks.*;
import pl.edu.pjwstk.yaod.system.processes.*;

/**
 * Klasa, ktorej zadaniem jest dodanie zachowania do statycznych struktur,
 * jakimi sa atomy referencyjne. Atomy referencyjne modeluja referencje -
 * obiekty, ktorych celem jest wskazywanie innych obiektow. Klasa YReference
 * np. powoduje, ze podpiecie do skladu referencji, spowoduje rownoczesne
 * utworzenie referencji zwrotnej w obiekcie wskazywanym. Po usunieciu
 * referencji, referencja zwrotna jest usuwana.
 * 
 * @author M. Lentner
 * @version 1.0
 */

public class YReference extends YObject {
/**
 * Konstruktor tworzacy nowy obiekt referencji. Utworzony obiekt
 * musi byc pozniej podpiety do skladu za pomoca metody connect().
 * @param sp proces usugowy, ktory umozliwia obiektowi wykonywanie akcji.
 * @param name nazwa obiektu referencji
 * @param value identyfikator wskazywanego obiektu.
 * @return nowy obiekt referencji
 */
  YReference(SVRP sp, String name, ObjectID value) {
    super(sp, new ReferenceAtom(name, (AtomID) value));
  }

/**
 * Konstruktor tworzacy nowy obiekt referencji na podstawie atomu
 * istniejacego juz w systemie.
 * @param sp proces usugowy zarzdzajcy danym obiektem.
 * @return nowy obiekt referencji.
 */
  YReference(SVRP sp, ReferenceAtom atom) {
    super(sp, atom);
  }

/**
 * Zwraca warto referencji, tzn. identyfikator wskazywanego obiektu.
 * @return warto referencji.
 */
  public ObjectID getValue() {
    return (ObjectID) ((ReferenceAtom) getAtom()).getValue();
  }

/**
 * Ustawia wartosc referencji, tzn. identyfikator wskazywanego obiektu.
 * @param val nowa wartosc referencji.
 * TODO: modyfikacja referencji zwrotnych !!!
 */
  public void setValue(ObjectID val) {
    ((ReferenceAtom) getAtom()).setValue((AtomID) val);
  }

/**
 * Odlacza referencje od skladu (kasuje obiekt). Oprocz usuniecia
 * atomu, powoduje rowniez skasowanie referencji zwrotnej wskazywanego obiektu.
 * @exception XxxxxxException if ...
 * @return comments
 */
  public void disconnect() throws NoSuchObjectException {
    YObject refObject = (YComplexObject) getServer().getObjectManager().getObject(getValue());
    if (refObject == null)
      throw new NoSuchObjectException();

    YBackward backward = refObject.getBackward();
    if (backward == null)
      throw new NoSuchObjectException();

    backward.deleteBackwardRef(getOID());

    super.disconnect();
  }

/**
 * Podlacza obiekt referencji do podanego obiektu nadrzednego.
 * @param ownerID identyfikator obiektu nadrzednego.
 * @exception NoSuchObjectException nieistniejacy obiekt nadrzedny.
 */
  public void connect(ObjectID ownerID)
  throws NoSuchObjectException, BlockFreeSpaceException {
    YObject refObject = (YObject) getServer().getObjectManager().getObject(getValue());
    YBackward backward = refObject.getBackward();

    super.connect(ownerID);

    if (backward != null)
      backward.addBackwardRef(getOID());
    else {
      backward = new YBackward(getServer());
      backward.addBackwardRef(getOID());
      backward.connectAfter(refObject.getOwner(), refObject.getOID());
    }
  }
}
