package pl.edu.pjwstk.yaod.objects;

import java.util.*;

import pl.edu.pjwstk.yaod.atoms.*;
import pl.edu.pjwstk.yaod.store.*;
import pl.edu.pjwstk.yaod.store.defaultstore.blocks.*;
import pl.edu.pjwstk.yaod.system.processes.*;

/**
 * Klasa, ktorej zadaniem jest dodanie zachowania do statycznych struktur,
 * jakimi sa atomy typu spider. Spider jest atomem umozliwiajacym modelowanie
 * zlozonych struktur danych - jego zawartoscia sa referencje do innych
 * obiektow. Kazdy YSpider posiada referencje do atomu, ktorego zawartoscia
 * zarzadza. Podczas gdy w systemie moze istniec tylko jeden obiekt SpiderAtom
 * reprezentujacy te sama jednostke danych, obiektow typu YSpider moze byc
 * nieskonczona ilosc (rozroznienie to ma znaczenie przede wszystkim dla
 * wspolbieznosci). Czesc metod realizowana jest poprzez delegacje
 * (np. pobieranie nazwy), a czesc dodaje nowa funkcjonalnosc
 * (np. zarzadzanie referencjami zwrotnymi). Poniewaz spider tworzy,
 * odczytuje i usuwa obiekty, dlatego musi komunikowac sie z procesem
 * uslugowym, ktory wykonuje dla niego te operacje.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class YSpider extends YComplexObject {

/**
 * Konstruktor, ktorego zadaniem jest tworzenie w systemie nowych spiderow.
 * @param sp proces uslugowy, przyporzadkowany dla danego obiektu.
 * @param name nazwa tworzonego obiektu.
 * @return nowy spider.
 */

  YSpider(SVRP sp, String name) {
    super(sp, new SpiderAtom(name));
  }

/**
 * Konstruktor, ktorego zadaniem jest utworzenie obiektu YSpider,
 * reprezentujacego istniejacy juz w systemie atom.
 * @param sp proces uslugowy, przyporzadkowany dla danego obiektu.
 * @return nowy spider.
 */

  YSpider(SVRP sp, SpiderAtom spider) {
    super(sp, spider);
  }

/**
 * Zwraca identyfikator pierwszego obiektu podrzednego.
 * @return identyfikator pierwszego obiektu podrzednego.
 */

  public ObjectID getFirstSubordinate() {
    return (ObjectID) ((SpiderAtom) getAtom()).getFirstMember();
  }

/**
 * Zwraca identyfikatory wszystkich obiektow lezacych na tym
 * samym poziomie w hierarchii obiektow.
 * @return obiekt zawierajacy identyfikatory.
 */

  public Enumeration getSubordinates() {
    return new Enumeration() {
      private ObjectID curr = null;

      private ObjectID nextSubordinate() {
        ObjectID subord;
        try {
          if (curr == null)
            subord = getFirstSubordinate();
          else
            subord = getNextSubordinate(curr);
        }
        catch (NoSuchObjectException e) {
          pl.edu.pjwstk.yaod.util.Errors.printError(1, "getSubordinates()");
          subord = null;
        }
        return subord;
      }

      public boolean hasMoreElements() {
        ObjectID tmp = nextSubordinate();

        return tmp == null ? false : true;
      }

      public Object nextElement() {
        ObjectID tmp = nextSubordinate();
        if (tmp == null)
          throw new NoSuchElementException();
        curr = tmp;

        return curr;
      }
    };
  }

/**
 * Zwraca identyfikator nastepnego obiektu, ,,rowiesnika'' w hierarchii
 * (na tym samym poziomie wzgledem obiektow podrzednych podanego
 * obiektu nadrzednego).
 * @param oid identyfikator poprzednika.
 * @exception 
 * @return comments
 */

  public ObjectID getNextSubordinate(ObjectID oid) throws NoSuchObjectException {
    YObject prev = getServer().getObjectManager().getObject(oid);
    YObject next = getServer().getObjectManager().getObject(prev.getNextSibling());

    if (next == null)
      return null;

    if (next instanceof YBackward)
      return getNextSubordinate(next.getOID());

    return next.getOID();
  }

/**
 * Podpina nowy obiekt podrzedny, utrwalajac go w skladzie
 * oraz zapisujac jego identyfikator.
 * @param member podpinany obiekt.
 * @exception BlockFreeSpaceException brak wolnego miejsca.
 */

  void connectMember(YObject member) throws BlockFreeSpaceException {
    Wrappable store = getServer().getStoreByID(getOID().getStoreID());

    Atom memberAtm = member.getAtom();
    memberAtm.setOwner((AtomID) getOID());
    store.createAtom(memberAtm);

    ((SpiderAtom) getAtom()).addMember(memberAtm.getAID());
  }

/**
 * Podpina nowy obiekt podrzedny jako nastepnik istniejacego obiektu.
 * @param member obiekt majacy zostac podpiety do skladu.
 * @exception BlockFreeSpaceException brak wolnego miejsca.
 * @return comments
 */

  void connectMemberAfter(YObject member, ObjectID base)
      throws BlockFreeSpaceException {
    Wrappable store = getServer().getStoreByID(getOID().getStoreID());

    Atom memberAtm = member.getAtom();
    memberAtm.setOwner((AtomID) getOID());
    store.createAtom(memberAtm);

    ((SpiderAtom) getAtom()).connectMemberAfter(memberAtm.getAID(), (AtomID) base);
  }

/**
 * Podpina nowy obiekt podrzedny jako poprzednik istniejacego obiektu.
 * Podpiecie oznacza utworzenie obiektu w skladzie i zarejestrowanie go
 * jako obiekt podrzedny.
 * @param member podpinany obiekt.
 * @exception BlockFreeSpaceException brak wolnego miejsca.
 * @param base identyfikator obiektu przed ktorym umiescic nowy obiekt podrzedny.
 */

  void connectMemberBefore(YObject member, ObjectID base)
      throws BlockFreeSpaceException {
    Wrappable store = getServer().getStoreByID(getOID().getStoreID());

    Atom memberAtm = member.getAtom();
    memberAtm.setOwner((AtomID) getOID());
    store.createAtom(memberAtm);

    ((SpiderAtom) getAtom()).connectMemberBefore(memberAtm.getAID(), (AtomID) base);
  }

/**
 * Rejestruje obiekt podrzedny w taki sposob, aby zostal umieszczony
 * po innym obiekcie.
 * @param memberPtr identyfikator rejestrowanego obiektu.
 * @param basePtr identyfikator obiektu po ktorym umiescic nowy obiekt.
 */
 
  void registerMemberAfter(ObjectID memberPtr, ObjectID basePtr) {
    ((SpiderAtom) getAtom()).connectMemberAfter((AtomID) memberPtr, (AtomID) basePtr);
  }

/**
 * Rejestruje obiekt podrzedny.
 * @param ptr identyfikator rejestrowanego obiektu.
 */

  void registerMember(ObjectID ptr) {
    ((SpiderAtom) getAtom()).addMember((AtomID) ptr);
  }

/**
 * Odrejestrowywuje obiekt podrzedny.
 * @param ptr identyfikator odrejestrowywanego obiektu.
 */

  void unregisterMember(ObjectID ptr) {
    ((SpiderAtom) getAtom()).removeMember((AtomID) ptr);
  }

/**
 * Rozlacza obiekt podrzedny, odrejestrowywujac jego identyfikator
 * oraz kasujac atom ze skladu.
 * @param delPtr identyfikator usuwanego obiektu.
 * @exception NoSuchObjectException proba usuniecia nieistniejacego
 * obiektu podrzednego.
 */

  void disconnectMember(ObjectID delPtr) throws NoSuchObjectException {
    unregisterMember(delPtr);

    try {
      Wrappable store = getServer().getStoreByID(delPtr.getStoreID());
      store.deleteAtom((AtomID) delPtr);
    }
    catch (NoSuchAtomException e) {
      throw new NoSuchObjectException();
    }
  }

/**
 * Zwraca wszystkie obiekty podrzedne.
 * @return obiekt zawierajacy identyfikatory wszystkich obiektow
 * podrzednych.
 */

  public Enumeration getMembers() {
    return ((SpiderAtom) getAtom()).getMembers();
  }

/**
 * Zwraca nastepny obiekt podrzedny w stosunku do podanego.
 * @param startID identyfikator obiektu bazowego.
 * @return oid nastepnego obiektu.
 */

  public ObjectID getNextMember(ObjectID startID) {
    ObjectID prevID = null;

    for (Enumeration e = getMembers(); e.hasMoreElements(); ) {
      ObjectID memberID = (ObjectID) e.nextElement();
      if (prevID != null && prevID.equals(startID))
        return memberID;
    
      prevID = memberID;
    }

    return null;
  }

/**
 * Zwraca identyfikator poprzedniego obiektu w grafie obiektow.
 * TODO: do zaimplementowania
 * @param member oid bazowego obiektu.
 * @return oid poprzedniego obiektu.
 */

  public ObjectID getPreviousMember(ObjectID member) {
    return null;
  }

/**
 * Zwraca pierwszy obiekt podrzedny o podanym typie.
 * TODO: do zaimplementowania.
 * @param type typ obiektu.
 * @param startPtr identyfikator obiektu, od ktorego zaczac poszukiwania.
 * @return oid znalezionego obiektu.
 */

  public ObjectID findMemberByType(byte type, ObjectID startPtr) {
    return null;
  }

/**
 * Zwraca pierwszy obiekt podrzedny o podanej nazwie.
 * @param name nazwa szukanego obiektu.
 * @param startPtr identyfikator obiektu od ktorego zaczac poszukiwania.
 * @return identyfikator znalezionego obiektu.
 */

  public ObjectID findMemberByName(String name, ObjectID startPtr) {
    return findMemberByNameAndType(name, Atom.ALL_TYPES, startPtr);
  }

/**
 * Zwraca pierwszy obiekt podrzedny o podanej nazwie i typie.
 * @param name nazwa obiektu.
 * @param type typ obiektu (obecnie nie jest obslugiwany).
 * @param strartPtr identyfikator obiektu, od ktorego rozpoczac przeszukiwanie.
 * @return identyfikator znalezionego obiektu.
 */

  public ObjectID findMemberByNameAndType(String name, byte type, ObjectID startPtr) {
    try {
      Wrappable store = getServer().getStoreByID(getOID().getStoreID());

      boolean start = startPtr == null ? true : false ;

      for (Enumeration e = getMembers(); e.hasMoreElements(); ) {
        ObjectID currID = (ObjectID) e.nextElement();
        if (start) {
          Atom currAtm = store.retrieveAtom((AtomID) currID);
          if (currAtm.getName().equals(name)) // && (currAtm.getType() == type || type == Atom.ALL_TYPES))
            return currID;
        }
        if (start == false && startPtr.equals(currID))
          start = true;
      }
    }
    catch (NoSuchAtomException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "Nonexistent member atom (findMemberByNameAndType()).");
    }
    return null;
  }

}
