package pl.edu.pjwstk.yaod.store.defaultstore.blocks;

import pl.edu.pjwstk.yaod.util.*;

/**
 * Klasa reprezentuje pewien wycinek bloku o zadanym poczatku i koncu,
 * do ktorego mozna sie odwolywac za pomoca indeksu zaczynajacego
 * sie od liczby 0. Naruszenie granic sekcji powoduje wywolanie wyjatku.
 * Ulatwia to poruszanie sie po sekcjach glownie w bloku danych, ale nie tylko.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class BlockSection {
  private BlockSection m_parentSection;
  private byte[] m_data;

  private short m_start;
  private short m_size;

/**
 * Tworzy nowa sekcje na bazie istniejacej juz sekcji.
 * @param parentSection sekcja bazowa.
 * @param start punkt w sekcji bazowej, ktory ma stanowic poczatek nowej sekcji.
 * @param size rozmiar nowej sekcji.
 * @exception naruszono granice sekcji bazowej.
 * @return nowa sekcja.
 */
  public BlockSection(BlockSection parentSection, short start, short size) throws SectionBoundsException {
    if (start < parentSection.getStart() || size < 0 || start + size - 1 > parentSection.getEnd())
      throw new SectionBoundsException();
  
    m_parentSection = parentSection;
    m_start = start;
    m_size = size;
  }

/**
 * Tworzy nowa sekcje (pierwsza) na bazie tablicy.
 * @param data tablica majaca zostac podzielona na sekcje.
 * @return nowa sekcja.
 */
  public BlockSection(byte[] data) {
    m_data = data;
    m_start = 0;
    m_size = (short) data.length;
  }

/**
 * Sprawdza czy sekcja jest sekcja bazowa (jest pierwsza sekcja).
 * @return sekcja jest sekcja bazowa (true) lub nie (false).
 */
  public boolean isSuperSection() {
    return m_parentSection == null ? true : false;
  }

/**
 * Zwraca rozmiar sekcji.
 * @return rozmiar sekcji.
 */
  public short getSize() {
    return m_size;
  }

/**
 * Ustala poczatek sekcji wzgledem sekcji nadrzednej.
 * @param start poczatek sekcji.
 * @exception SectionBoundsException granice sekcji naruszone.
 * @return comments
 */
  public void setStart(short start) throws SectionBoundsException {    
    if (start < m_parentSection.getStart() || start > m_parentSection.getEnd())
      throw new SectionBoundsException();

    if (start > m_start + m_size)
      m_size = 0;

    m_start = start;
  }

/**
 * Zwraca poczatek sekcji wzgledem sekcji nadrzednej.
 * @return poczatek sekcji.
 */
  public short getStart() {
    return m_start;
  }

/**
 * Zwraca punkt konca danej sekcji wzgledem sekcji nadrzednej.
 * @param paramName comments
 * @exception SectionBoundsException granice sekcji naruszone.
 * @return comments
 */
  public short getEnd() {
    return (short) (getSize() == 0 ? m_start : m_start + getSize() - 1);
  }

/**
 * Ustala wartosc sekcji.
 * @param size nowy rozmiar sekcji.
 * @exception SectionBoundsException granice sekcji naruszone.
 */
  public void setSize(short size) throws SectionBoundsException {
    if (size < 0 || m_start + size - 1 > m_parentSection.getEnd())
      throw new SectionBoundsException();
    m_size = size;
  }

/**
 * Zwraca zawartosc sekcji.
 * @return zawartosc sekcji.
 */
  public byte[] getBytes() {
    try {
      if (isSuperSection())
        return m_data;
      else {
        byte[] ret = get((short) 0, getSize());
        return ret;
      }
    }
    catch (SectionBoundsException e) {
      System.out.println("Nie mozna zserializowac sekcji");
    }
    return null;
  }

/**
 * Umieszcza wartosc tablicy w okreslonym punkcie sekcji.
 * @param buf wartosc umieszczana w sekcji.
 * @param offset gdzie umiescic wartosc.
 * @exception SectionBoundsException parametry naruszaja sekcje.
 */
  public void put(byte[] buf, int offset) throws SectionBoundsException {
    short absOffset = (short) (offset + m_start);

    if (buf.length + offset > getSize() || offset < 0)
      throw new SectionBoundsException();
    
    if (isSuperSection()) 
      System.arraycopy(buf, 0, m_data, offset, buf.length);
    else
      m_parentSection.put(buf, absOffset);
  }

/**
 * Zwraca czesc sekcji.
 * @param offset poczatek fragmentu.
 * @param length dlugosc fragmentu.
 * @exception SectionBoundsException parametry naruszaja sekcje.
 * @return tablica zawierajaca fragment sekcji.
 */
  public byte[] get(short offset, short length) throws SectionBoundsException {
    short absOffset = (short) (offset + m_start);

    if (length + offset > getSize() || offset < 0)
      throw new SectionBoundsException();

    if (isSuperSection()) {
      byte[] buf = new byte[length];
      System.arraycopy(m_data, absOffset, buf, 0, length);
      return buf;
    }
    else 
      return m_parentSection.get(absOffset, length);
  }

/**
 * Kopiuje fragment sekcji w nowe miejsce.
 * @param start poczatek fragmentu.
 * @param val o ile bajtow przeniesc.
 * @param len ile bajtow przeniesc.
 */
  public void move(short start, short val, short len) {
    if (isSuperSection())
      System.arraycopy(m_data, start, m_data, start + val, len);
    else
      m_parentSection.move((short) (m_start + start), val, len);
  }

/**
 * Zwraca wartosc short umieszczona w okreslonym punkcie sekcji.
 * @param offset punkt w ktorym odczytac wartosc.
 * @exception SectionBoundsException parametry naruszaja granice sekcji.
 */
  public short getShort(short offset) throws SectionBoundsException {
    if (offset + NewBits.SHORT_SIZE > m_size || offset < 0)
      throw new SectionBoundsException();
  
    if (isSuperSection())
      return NewBits.getShort(m_data, offset);
    else
      return m_parentSection.getShort((short) (m_start + offset));
  }

/**
 * Umieszcza wartosc byte w okreslonym punkcie sekcji.
 * @param offset punkt w ktorym umiescic wartosc.
 * @param value wartosc ktora umiescic.
 * @exception SectionBoundsException parametry naruszaja granice sekcji.
 */
  public byte getByte(short offset) throws SectionBoundsException {
    if (offset + NewBits.BYTE_SIZE > m_size || offset < 0)
      throw new SectionBoundsException();

    if (isSuperSection())
      return m_data[offset];
    else
      return m_parentSection.getByte((short) (m_start + offset));    
  }

/**
 * Umieszcza wartosc short w okreslonym punkcie sekcji.
 * @param offset punkt w ktorym umiescic wartosc.
 * @param value wartosc ktora umiescic.
 * @exception SectionBoundsException parametry naruszaja granice sekcji.
 */
  public void putShort(short offset, short value) throws SectionBoundsException {
    if (offset + NewBits.SHORT_SIZE > m_size || offset < 0)
      throw new SectionBoundsException();

    if (isSuperSection())
      NewBits.putShort(m_data, offset, value);
    else
      m_parentSection.putShort((short) (m_start + offset), value);
  }

}
