package pl.edu.pjwstk.yaod.store.defaultstore.blocks;

import pl.edu.pjwstk.yaod.store.*;
import pl.edu.pjwstk.yaod.util.*;

/**
 * Klasa zapewnia logike zwiazana z wewnetrznym funkcjonowaniem blokow danych.
 * Blok danych jest struktura umozliwiajaca rejestrowanie w niej rekordow.
 * Rekord jest zserializowanym atomem. Blok danych sklada sie z sekcji:
 * <ul>
 *  <li>naglowka bloku (stay rozmiar).<br>
 *   <ul>
 *    <li> numer pliku (2 bajty) </li>
 *    <li> numer bloku (2 bajty) </li>
 *    <li> typ bloku (1 bajt), zawsze =1 </li>
 *   </ul>
 *  </li>
 *  <li>naglowka bloku danych (staly rozmiar).<br>
 *   <ul> 
 *    <li> poczatek (2 bajty) i dlugosc (2 bajty) sekcji slotow</li>
 *    <li> poczatek (2 bajty) i dlugosc (2 bajty) sekcji pustej</li>
 *    <li> poczatek (2 bajty) i dlugosc (2 bajty) sekcji danych</li>
 *   </ul>
 *  </li>
 *  <li>slotow (zmienny rozmiar, 2 bajty na kazdy slot)</li>
 *  <li>pustej (zmienny rozmiar).</li>
 *  <li>danych (zmienny rozmiar, rekordy roznej dlugosci).</li>
 * </ul>
 * Sekcja slotow i sekcja danych przyrastaja kosztem sekcji pustej - sekcja
 * slotow od gory w dol, sekcja danych od dolu do gory.
 * 
 * Struktura rekordu jest nastepujaca:
 * <ul>
 *  <li> dlugosc danych (15 bitow) </li>
 *  <li> flaga "lancuchowatosci" (1 bit, 0 - rekord prosty, 1 - lancuchowany) </li>
 *  <li> dane (o dlugosci podanej w pierwszej czesci rekordu)</li>
 *  <li> opcjonalny wskaznik do nastepnego rekordu (8 bajtow = plik, blok, slot),
 *        wystepujacy tylko wowczas, gdy rekord jest lancuchowany.
 *        Modul wywolujacy musi zapewnic obsluge tego wskaznika
 *        we wlasnym zakresie.</li>
 * </ul>
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class DataBlock extends AbstractBlock {
  public final static short SLOT_LENGTH = NewBits.SHORT_SIZE;
  public final static short RECORD_HEADER_LENGTH = NewBits.SHORT_SIZE;
  public final static short NEXT_RECORD_LENGTH = NewBits.SHORT_SIZE * 3;
  public final static short DATABLOCK_HEADER_LENGTH = 12;

  private BlockSection m_rawBlock;
  private BlockSection m_bhSection, m_dbSection, m_slSection, m_esSection, m_dtSection;

/**
 * Tworzy pusty blok danych podanej wielkosci.
 * @param size wielkosc tworzonego bloku.
 * @return utworzony blok.
 * @exception BlockFormatException blad niespodziewany (prawdopodobnie zbyt
 *                                 maly rozmiar bloku)
 */

  public DataBlock(short size, BlockID blockID) throws BlockFormatException {
    setBlockID(blockID);
    try {
      m_rawBlock = new BlockSection(new byte[size]);
      initEmptyBlock(size);
    }
    catch (SectionBoundsException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "DataBlock()");
      throw new BlockFormatException();
    }
  }

/**
 * Tworzy blok danych na podstawie podanej tablicy, zawierajacej jego
 * zserializowana zawartosc.
 * @param rawBlock zserializowany blok.
 * @return utworzony blok.
 * @exception BlockFormatException podnoszony, jesli podana tablica nie
 *                                 zawiera wlasciwie sformatowanego bloku danych.
 * @exception Exception podnoszony w przypadku awarii nieznanego
 *                      pochodzenia.
 */

  public DataBlock(byte[] rawBlock, BlockID blockID) throws BlockFormatException {
    setBlockID(blockID);

    m_rawBlock = new BlockSection(rawBlock);

    try {
      initEmptyBlock(rawBlock.length);

      m_slSection.setStart(m_dbSection.getShort((short) (NewBits.SHORT_SIZE * 0)));
      m_slSection.setSize(m_dbSection.getShort((short) (NewBits.SHORT_SIZE * 1)));

      m_esSection.setStart(m_dbSection.getShort((short) (NewBits.SHORT_SIZE * 2)));
      m_esSection.setSize(m_dbSection.getShort((short) (NewBits.SHORT_SIZE * 3)));

      m_dtSection.setStart(m_dbSection.getShort((short) (NewBits.SHORT_SIZE * 4)));
      m_dtSection.setSize(m_dbSection.getShort((short) (NewBits.SHORT_SIZE * 5)));

      if (m_slSection.getStart() > 100)
        System.out.println("JEST!");
    }
    catch (SectionBoundsException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "DataBlock()");
      throw new BlockFormatException();
    }
  }

/**
 * Serializuje blok (w celu pozniejszego zapisania go np. na dysku).
 * @return zserializowany blok.
 * @exception podnoszony w razie awarii nieznanego pochodzenia.
 */

  public byte[] toByteArray() {
    byte[] raw = null;

    try {
      m_dbSection.putShort((short) (NewBits.SHORT_SIZE * 0), m_slSection.getStart());
      m_dbSection.putShort((short) (NewBits.SHORT_SIZE * 1), m_slSection.getSize());

      m_dbSection.putShort((short) (NewBits.SHORT_SIZE * 2), m_esSection.getStart());
      m_dbSection.putShort((short) (NewBits.SHORT_SIZE * 3), m_esSection.getSize());

      m_dbSection.putShort((short) (NewBits.SHORT_SIZE * 4), m_dtSection.getStart());
      m_dbSection.putShort((short) (NewBits.SHORT_SIZE * 5), m_dtSection.getSize());  

      raw = m_rawBlock.getBytes();
    }
    catch (SectionBoundsException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "toByteArray()");
    }

    return raw;
  }

/**
 * Inicjalizuje blok podanej wielkosci, ustanawiajac poczatkowe
 * rozmiary poszczegolnych sekcji.
 * @return zserializowany blok.
 */

  private void initEmptyBlock(int size) throws SectionBoundsException {
    short blockHeaderLength = (short) (((BlockID) m_blockID).sizeOf() + 1);

    m_bhSection = new BlockSection(m_rawBlock, (short) 0, blockHeaderLength);
    m_dbSection = new BlockSection(m_rawBlock, (short) (m_bhSection.getEnd() + 1), DATABLOCK_HEADER_LENGTH);
    m_slSection = new BlockSection(m_rawBlock, (short) (m_dbSection.getEnd() + 1), (short) 0);
    m_esSection = new BlockSection(m_rawBlock, (short) (m_slSection.getEnd() + 1), (short) (size - 1 - m_slSection.getEnd() - 1));
    m_dtSection = new BlockSection(m_rawBlock, (short) (m_esSection.getEnd() + 1), (short) 0);
  }

/**
 * Zwraca wielkosc bloku (suma wszystkich sekcji).
 * @return wielkosc bloku.
 */

  public short getSize() {
    return m_rawBlock.getSize();
  }

/**
 * Zwraca wielkosc dostepnej wolnej przestrzeni w ramach bloku,
 * jako rozmiar sekcji pustej.
 * @return wielkosc wolnej przestrzeni.
 */

  public short getFreeSpaceSize() {
    return m_esSection.getSize();
  }

/**
 * Zwraca wielkosc zapisanej przestrzeni w ramach bloku, jako rozmiar
 * sekcji danych.
 * @return wielkosc sekcji danych.
 */

  public short getDataSize() {
    return m_dtSection.getSize();
  }

// -----------------------
// sekcje

/**
 * Pomniejsza sekcje slotow, zwiekszajac sekcje pusta (od gory).
 * @param size o ile bajtow pomniejszyc sekcje slotow.
 */

  private void shrinkSlotSection(int size) throws SectionBoundsException {
    if (m_slSection.getSize() < size || size < 0)
      throw new SectionBoundsException();
    m_esSection.setStart((short)(m_esSection.getStart() - size));
    m_esSection.setSize((short)(m_esSection.getSize() + size));
    m_slSection.setSize((short)(m_slSection.getSize() - size));
  }

/**
 * Powieksza sekcje slotow, pomniejszajac sekcje pusta.
 * @param size o ile bajtow powiekszyc sekcje slotow.
 */

  private void expandSlotSection(int size) throws SectionBoundsException {
    if (m_esSection.getSize() < size || size < 0)
      throw new SectionBoundsException();
    m_esSection.setStart((short)(m_esSection.getStart() + size));
    m_esSection.setSize((short)(m_esSection.getSize() - size));
    m_slSection.setSize((short)(m_slSection.getSize() + size));
  }

/**
 * Powieksza sekcje danych, zmniejszajac sekcje pusta (od dolu).
 * @param size o ile bajtow powiekszyc sekcje danych.
 */

  private void expandDataSection(int size) throws SectionBoundsException {
    if (m_esSection.getSize() < size || size < 0)
      throw new SectionBoundsException();
    m_esSection.setSize((short)(m_esSection.getSize() - size));
    m_dtSection.setStart((short)(m_dtSection.getStart() - size));
    m_dtSection.setSize((short)(m_dtSection.getSize() + size));
  }

/**
 * Pomniejsza sekcje danych, zwiekszajac sekcje pusta (od dolu).
 * @param size  o ile bajtow powiekszyc sekcje danych.
 */

  private void shrinkDataSection(int size) throws SectionBoundsException {
    if (m_dtSection.getSize() < size || size < 0)
      throw new SectionBoundsException();
    m_esSection.setSize((short)(m_esSection.getSize() + size));
    m_dtSection.setStart((short)(m_dtSection.getStart() + size));
    m_dtSection.setSize((short)(m_dtSection.getSize() - size));
  }

// -----------------------
// rekordy

/**
 * Tworzy rekord. Powieksza sekcje danych, tworzy slot i ustawia naglowek.
 * @param record tresc rekordu jako ciag bajtow.
 * @param type czy rekord posiada dalszy ciag w innym bloku.
 * @return numer slotu.
 * @exception NoFreeSpaceException za malo wolnego miejsca na stworzenie rekordu.
 * @exception 
 */
  
  public short createRecord(byte[] record, boolean chained)
      throws BlockFreeSpaceException {
    if (getFreeSpaceSize() < record.length + RECORD_HEADER_LENGTH)
      throw new BlockFreeSpaceException();

    short slot = 0;
    try {
      expandDataSection(record.length + RECORD_HEADER_LENGTH);
      slot = createSlot(m_dtSection.getStart());

      short header = (short) ((record.length << 1) | (chained ? 1 : 0));

      m_dtSection.putShort((short) 0, header);
      m_dtSection.put(record, RECORD_HEADER_LENGTH);
    }
    catch (SectionBoundsException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "createRecord()");
      throw new BlockFreeSpaceException();
    }

    return slot;
  }

/**
 * Zwraca rekord identyfikowany podanym slotem.
 * @param slot numer slotu przypisanego danemu rekordowi.
 * @return tresc rekordu.
 * @exception NoSuchRecordException rekord o podanym slocie nie istnieje.
 */

  public byte[] getRecord(short slot) throws NoSuchSlotException {
    short recOffset = getRecordOffset(slot);
    byte[] record;
    try {
      short recLength = (short) (m_rawBlock.getShort(recOffset) >> 1);
      record = m_rawBlock.get((short)(recOffset + RECORD_HEADER_LENGTH), recLength);
    }
    catch (SectionBoundsException e) {
      throw new NoSuchSlotException();
    }
    
    return record;
  }

/**
 * Aktualizuje rekord identyfikowany podanym slotem. Operacja ta polega
 * na usunieciu danych (i wypelnieniu powstalej ewentualnie dziury +
 * aktualizacja slotow na nowe wartosci) oraz wprowadzeniu nowych danych +
 * aktualizacja wartosci slotu modyfikowanego rekordu. Metoda nie akceptuje
 * rekordow wiekszych niz moze zapisac w bloku - ustalenie odpowiedniej
 * wielkosci musi zapewnic modul wywolujacy.
 * @param slot numer slotu przypisanego danemu rekordowi.
 * @param record nowa tresc rekordu.
 * @param chained czy rekord ma dalszy ciag w innym bloku.
 * @exception NoFreeSpaceException nie ma miejsca na zaktualizowanie danych.
 * @exception NoSuchSlotException blad nieznanego pochodzenia.
 */

  public void updateRecord(short slot, byte[] record, boolean chained)
      throws BlockFreeSpaceException, NoSuchSlotException {
    try {
      short recOffset = getRecordOffset(slot);

      short header = (short)(m_rawBlock.getShort(recOffset));
      short recLength = (short) (header >> 1);

      if (getFreeSpaceSize() < record.length - recLength + RECORD_HEADER_LENGTH)
        throw new BlockFreeSpaceException();

      deleteRecordData(slot);

      header = (short)((record.length << 1) | (chained ? 1 : 0));

      expandDataSection(record.length + RECORD_HEADER_LENGTH);
      m_dtSection.putShort((short) 0, header);
      m_dtSection.put(record, RECORD_HEADER_LENGTH);

      setSlotValue(slot, m_dtSection.getStart());
    }
    catch (SectionBoundsException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "updateRecord()");
      throw new BlockFreeSpaceException();
    }
  }

/**
 * Usuwa rekord identyfikowany podanym slotem.
 * @param slot numer slotu przypisanego danemu rekordowi.
 * @exception NoSuchRecordException rekord zwiazany ze slotem nie istnieje.
 */

  public void deleteRecord(short slot) throws NoSuchSlotException {
    try {
      deleteRecordData(slot);
      deleteSlot(slot);
    }
    catch (SectionBoundsException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "deleteRecord()");
      throw new NoSuchSlotException();
    }
    catch (NoSuchSlotException e) {
      throw new NoSuchSlotException();
    }
  }

/**
 * Usuwa tylko dane rekordu, bez kasowania slotu (potrzebne przy aktualizacji).
 * @param slot numer slotu przypisanego danemu rekordowi.
 * @exception NoSuchRecordException podany rekord nie istnieje.
 * @exception SectionBoundsException naruszona zostala ktoras sekcja.
 */

  private void deleteRecordData(short slot)
      throws NoSuchSlotException, SectionBoundsException  {
    short recOffset = getRecordOffset(slot);
    short recLength = (short)((m_rawBlock.getShort(recOffset) >> 1) +
                              RECORD_HEADER_LENGTH);

    // jesli rekord pierwszy w sekcji danych, to przesun sekcje danych.
    // w przeciwnym razie przesun poczatek sekcji na dziure
    if (recOffset - m_dtSection.getStart() > 0)
      m_rawBlock.move(m_dtSection.getStart(), recLength, 
                      (short)(recOffset - m_dtSection.getStart()));

    shrinkDataSection(recLength);

    try {
      // wylacz slot
      setSlotValue(slot, (short) 0);
    }
    catch (NoSuchSlotException e) {
      throw new NoSuchSlotException();
    }

    try {
      // zmodyfikuj sloty
      for (short s = 0; s < countAllSlots(); s++) {
        short slotValue = getSlotValue(s);
        if (recOffset > slotValue && slotValue != 0)
          setSlotValue(s, (short)(slotValue + recLength));
      }
    }
    catch (NoSuchSlotException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "deleteRecordData()");
      throw new NoSuchSlotException();
    }
  }

/**
 * Sprawdza, czy dany rekord jest lancuchowany, czyli czy zawiera dalszy ciag
 * w innym bloku. Jesli rekord jest lancuchowany, wowczas bezposrednio za nim
 * wystepuje identyfikator kolejnego rekordu.
 * @param slot numer slotu przypisanego danemu rekordowi.
 * @exception NoSuchRecordException podany rekord bazowy nie istnieje.
 */

  public boolean isChainedRecord(short slot) throws NoSuchSlotException {
    boolean chained;
    try {
      short recOffset = getRecordOffset(slot);
      short header = m_rawBlock.getShort(recOffset);
      chained = (header & 1) == 1 ? true : false;
    }
    catch (SectionBoundsException e) {
      Errors.printError(1, "isChainedRecord()");
      throw new NoSuchSlotException();
    }
    return chained;
  }

/**
 * Ustawia wskaznik kolejnego rekordu dla danego rekordu lancuchowanego.
 * @param slot numer slotu przypisanego danemu rekordowi.
 * @param ptr wskaznik do nastepnego rekordu.
 * @exception NoSuchRecordException podany rekord bazowy nie istnieje.
 * @exception RecordNotChained rekord nie jest lancuchowany.
 */

  public void setNextPart(short slot, byte[] rawPartPtr)
      throws NoSuchSlotException, RecordNotChainedException {
    try {
      short recOffset = getRecordOffset(slot);
      short header = m_rawBlock.getShort(recOffset);
      if ((header & 1) == 0)
        throw new RecordNotChainedException();

      short recLength = (short)( header >> 1);
      short nextPartOffset = (short) (recOffset + recLength +
                                      RECORD_HEADER_LENGTH - NEXT_RECORD_LENGTH);

      m_rawBlock.put(rawPartPtr, nextPartOffset);
    }
    catch (SectionBoundsException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "setNextPart()");
      throw new NoSuchSlotException();
    }
  }

/**
 * Zwraca wskaznik kolejnego rekordu dla danego rekordu lancuchowanego.
 * @param slot numer slotu przypisanego danemu rekordowi.
 * @return wskaznik do kolejnego rekordu.
 * @exception NoSuchRecordException podany rekord bazowy nie istnieje.
 */

  public byte[] getNextPart(short slot) throws NoSuchSlotException {
    try {
      short recOffset = getRecordOffset(slot);
      short header = (short)(m_rawBlock.getShort(recOffset));
      if ((header & 1) == 0)
        return null;

      short recLength = (short)(header >> 1);
      short nextPartOffset = (short) (recOffset + recLength +
                                      RECORD_HEADER_LENGTH - NEXT_RECORD_LENGTH);

      byte[] rawPart = m_rawBlock.get(nextPartOffset, NEXT_RECORD_LENGTH);
      return rawPart;
    }
    catch (SectionBoundsException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "getNextPart()");
      throw new NoSuchSlotException();
    }
  }

/**
 * Zwraca dlugosc rekordu.
 * @param slot numer slotu przypisanego danemu rekordowi.
 * @return dlugosc rekordu.
 * @exception NoSuchRecordException rekord z podanym slotem nie istnieje.
 */

  public short getRecordLength(short slot) throws NoSuchSlotException {
    short length;
    try {
      short recOffset = getRecordOffset(slot);
      length = (short)((m_rawBlock.getShort(recOffset) >> 1));
    }
    catch (SectionBoundsException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "getRecordLength()");
      throw new NoSuchSlotException();
    }
    return length;
  }

/**
 * Zwraca rzeczywista dlugosc, jaka przyjelyby dane, gdyby 
 * zostaly zapisane w bloku jako rekord.
 * @param rec dane majace utworzyc rekord.
 * @return dlugosc potencjalnego rekordu.
 */

 public static int getRawRecordLength(byte[] rec) {
   return rec.length + pl.edu.pjwstk.yaod.store.defaultstore.blocks.DataBlock.RECORD_HEADER_LENGTH +
                       pl.edu.pjwstk.yaod.store.defaultstore.blocks.DataBlock.SLOT_LENGTH;
 }

// -----------------------
// sloty

/**
 * Zwraca numer pierwszego wolnego slotu. Slot wolny to taki, ktory byl uzywany,
 * ale w wyniku skasowania rekordu, juz nie jest w uzyciu. Wartoscia slotu
 * wolnego jest liczba 0.
 * @return numer pierwszego wolnego slotu.
 * @exception NoSuchSlotException brak wolnych slotow.
 */

  private short getFirstEmptySlot() throws NoSuchSlotException {
    for (short slot = 0; slot < countAllSlots(); slot++) {
      int currSlotVal = getSlotValue(slot);
      if (currSlotVal == 0)
        return slot;  
    }
    throw new NoSuchSlotException();
  }

/**
 * Tworzy nowy slot.
 * @param slotValue wartosc slotu, czyli offset (wzgledem
 *                  poczatku bloku, czyli wartosci 0) rekordu.
 * @return numer utworzonego slotu.
 * @exception NoFreeSpaceException nie mozna wstawic slotu ze wzgledu na brak
 *                                 wolego miejsca w bloku.
 */

  private short createSlot(short slotValue) throws BlockFreeSpaceException {
    short slot;

    try {
      try {
        slot = getFirstEmptySlot();
        setSlotValue(slot, slotValue);
      }
      catch (NoSuchSlotException e) {
        expandSlotSection(SLOT_LENGTH);

        slot = (short) (countAllSlots() - 1);
        setSlotValue(slot, slotValue);
      }
    }
    catch (NoSuchSlotException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "createSlot()");
      throw new BlockFreeSpaceException();
    }
    catch (SectionBoundsException e) {
      throw new BlockFreeSpaceException();
    }

    return slot;
  }

/**
 * Usuwa slot. Najpierw go zeruje, a pozniej sprawdza czy sloty na
 * konecu sekcji slotow sa nieuzywane. Jesli sa takie sloty, to wowczas
 * sekcja slotow jest zmniejszana
 * @param slot numer slotu, ktory nalezy usunac.
 * @exception NoSuchSlotException podany slot nie istnieje.
 */

  private void deleteSlot(short slot) throws NoSuchSlotException {  
    setSlotValue(slot, (short) 0);

    try {
      for (short i = (short) (countAllSlots() - 1); i >= 0; i--) {
        short slotValue = getSlotValue(i);
        if (slotValue == 0)
          shrinkSlotSection(SLOT_LENGTH);
        else
          break;
      }
    }
    catch (SectionBoundsException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "deleteSlot()");
    }
  }

/**
 * Zwraca ilosc wszystkich slotow w bloku (pustych i niepustych).
 * @return ilosc slotow.
 */

  private short countAllSlots() {
    return (short) (m_slSection.getSize() / SLOT_LENGTH);
  }

/**
 * Zwraca ilosc wszystkich niepustych slotow w bloku. Slot jest
 * niepusty (nieuzywany), jesli jest rozny od 0.
 * @return ilosc slotow.
 */

  private int countRecords() {
    int slots = 0;

    try {
      for (short i = 0; i < countAllSlots(); i++) {
        if (getSlotValue(i) != 0) slots++;
      }
    }
    catch (NoSuchSlotException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "countRecords()");
    }

    return slots;
  }

/**
 * Zwraca offset rekordu w pliku.
 * @param slot numer slotu przyporzadkowanego rekordowi.
 * @return offset rekordu.
 * @exception NoSuchRecordException rekord identyfikowany przez podany slot
 *                                  nie istnieje.
 */

  private short getRecordOffset(short slot) throws NoSuchSlotException {
    short value;
    try {
      value = getSlotValue(slot);
      if (value == 0)
        throw new NoSuchRecordException();
      else
        return value;
    }
    catch (NoSuchSlotException e) {
      throw new NoSuchSlotException();
    }
  }

/**
 * Zwraca wartosc slotu. Jesli wartoscia slotu jest 0, wowczas slot
 * jest nieuzywany.
 * @param slot numer slotu.
 * @return wartosc slotu.
 * @exception NoSuchSlotException podany slot nie istnieje.
 */

  private short getSlotValue(short slot) throws NoSuchSlotException {
    try {
      return m_slSection.getShort((short) (slot * SLOT_LENGTH));
    }
    catch (SectionBoundsException e) {
      throw new NoSuchSlotException();
    }
  }

/**
 * Ustawia wartosc slotu na podana wartosc.
 * @param slot numer slotu.
 * @param value nowa wartosc slotu.
 * @exception NoSuchSlotException podany slot nie istnieje.
 */

  private void setSlotValue(short slot, short value) throws NoSuchSlotException {
    try {
      m_slSection.putShort((short) (slot * SLOT_LENGTH), value);
    }
    catch (SectionBoundsException e) {
      throw new NoSuchSlotException();
    }
  }

}
