package pl.edu.pjwstk.yaod.store.defaultstore.blocks;

import pl.edu.pjwstk.yaod.store.*;
import pl.edu.pjwstk.yaod.util.*;

/**
 * Klasa zapewnia logike zwiazana z wewnetrzynm funkcjonowaniem blokow wolnej
 * przestrzeni. Blok wolnej przestrzeni jest struktura umozliwiajaca
 * opis poziomu zajetosci poszczegolnych blokow w ramach pliku danych.
 * Kazdy plik danych posiada przynajmniej jeden blok wolnej przestrzeni.
 * Blok taki moze zawierac odwolania do kolejnych blokow danych,
 * przydzielanych w razie potrzeby. Wolna przestrzen reprezentowana jest
 * przez ciag wartosci short, w taki sposob, ze 2 bajty przydzielone sa jednemu
 * blokowi. Blok wolnej przestrzeni sklada sie z sekcji:
 * <ul>
 *  <li>naglowka bloku (stay rozmiar).<br>
 *   <ul>
 *    <li> numer pliku (2 bajty) </li>
 *    <li> numer bloku (2 bajty) </li>
 *    <li> typ bloku (1 bajt), zawsze =1 </li>
 *   </ul>
 *  </li>
 *  <li>opcjonalnego numeru bloku w pliku, zawierajacego dane na temat
 *      kolejnych blokow (2 bajty).</li>
 *  <li>danych (reszta bloku)</li>
 * </ul>
 */

public class FreeSpaceBlock extends AbstractBlock {
  private BlockSection m_rawBlock, m_bhSection, m_dtSection;

/**
 * Tworzy nowy blok wolnej przestrzeni na podstawie tablicy bajtow,
 * zawierajacej jego wersje zserializowana.
 * @param data zserializowany blok.
 * @return nowy blok wolnej przestrzeni.
 * @exception BlockFormatException dane nie zawieraja poprawnie sformowanego
 *                                 bloku wolnej przestrzeni.
 */

  public FreeSpaceBlock(byte[] rawBlock, BlockID blockID) throws BlockFormatException {
    try {
      m_blockID = blockID;
      m_rawBlock = new BlockSection(rawBlock);
      initEmptyBlock((short) rawBlock.length);
    }
    catch (SectionBoundsException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "FreeSpaceBlock()");
      throw new BlockFormatException();
    }
  }

/**
 * Tworzy nowy pusty blok wolnej przestrzeni.
 * @param size rozmiar bloku.
 * @param blockID identyfikator bloku.
 * @exception BlockFormatException nie mozna sformatowac bloku.
 * @return comments
 */
  public FreeSpaceBlock(short size, BlockID blockID) throws BlockFormatException {
    try {
      m_blockID = blockID;
      m_rawBlock = new BlockSection(new byte[size]);
      initEmptyBlock(size);
    }
    catch (SectionBoundsException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "FreeSpaceBlock()");
      throw new BlockFormatException();
    }
  }

/**
 * Serializuje blok wolnej przestrzeni.
 * @return tablica bajtow, zawierajaca zserializowany blok
 *         (gotowy np. do zapisu na dysku).
 */

  public byte[] toByteArray() {
    return m_rawBlock.getBytes();
  }

/**
 * Inicjalizuje poszczegolne sekcje nowego bloku wolnej przestrzeni.
 * @param size rozmiar tworzonego bloku wolnej przestrzeni.
 * @exception SectionBoundsException naruszenie ktorejs sekcji (prawdopodobnie
 *                                   czytanie poza blokiem).
 */

  private void initEmptyBlock(short size) throws SectionBoundsException {
    short blockHeaderLength = (short) (((BlockID) m_blockID).sizeOf() + 1);
  
    m_bhSection = new BlockSection(m_rawBlock, (short) 0, blockHeaderLength);
    m_dtSection = new BlockSection(m_rawBlock, (short) (m_bhSection.getEnd() + 1), (short) (size - m_bhSection.getEnd() - 1));
  }

/**
 * Zwraca wielkosc bloku (suma wszystkich sekcji).
 * @return wielkosc bloku.
 */

  public short getSize() {
    return m_rawBlock.getSize();
  }

/**
 * Zwraca ilosc blokow, jakie moze reprezentowac dany blok wolnej przestrzeni.
 * @return ilosc blokow.
 */

  public short blocksHolded() {
    return (short)(m_dtSection.getSize() / NewBits.SHORT_SIZE);
  }

/**
 * Rejestruje informacje na temat wolnej przestrzeni bloku o podanym numerze.
 * @param block numer bloku.
 * @param freeSpace zakodowana wolna przestrzen.
 * @exception NoSuchBlockException podano zly numer bloku.
 */

  public void setBlocksFreeSpace(short block, short freeSpace)
      throws NoSuchBlockException {
    try {
      byte[] rawValue = new byte[NewBits.SHORT_SIZE];
      NewBits.putShort(rawValue, 0, freeSpace);
      m_dtSection.put(rawValue, block * NewBits.SHORT_SIZE);
    }
    catch (SectionBoundsException e) {
      throw new NoSuchBlockException();
    }
  }

/**
 * Pobiera informacje na temat wolnej przestrzeni bloku o podanym
 * identyfikatorze.
 * @return zakodowana wolna przestrzen.
 * @exception NoSuchBlockException podany numer bloku nie jest reprezentowany
 *            w danym bloku wolnej przestrzeni.
 */

  public short getBlocksFreeSpace(short block) throws NoSuchBlockException {
    short freeSpace;
    try {
      freeSpace = m_dtSection.getShort((short)(block * NewBits.SHORT_SIZE));
    }
    catch (SectionBoundsException e) {
      throw new NoSuchBlockException();
    }
    return freeSpace;
  }
}
