package pl.edu.pjwstk.yaod.store.defaultstore.persistentstore;

import java.io.*;

import java.util.*;

import pl.edu.pjwstk.yaod.store.defaultstore.*;
import pl.edu.pjwstk.yaod.store.defaultstore.blocks.*;

/**
 * Klasa reprezentuje menedzera plikow.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class FileManager {
  private Hashtable m_dataFiles;
  private PersistentStore m_store;

/**
 * Tworzy nowy obiekt menedzera plikow.
 * @param store sklad dla ktorego menedzer ma zostac utworzony.
 * @return nowy menedzer plikow.
 */
  public FileManager(PersistentStore store) {
    m_dataFiles = new Hashtable();
    m_store = store;

    // zarejestrowanie plikow danych
    for (Enumeration e = m_store.getStoreDescriptor().getDatafiles(); e.hasMoreElements(); ) {
      DatafileDescriptor dfd = (DatafileDescriptor)e.nextElement();
      m_dataFiles.put(new Integer(dfd.getFileID()), new Datafile(dfd));
    }
  }

/**
 * Zwraca wszystkie pliki danych, na jakich operuje menedzer plikow.
 * @return obiekt umozliwiajacy dostep do wszystkich plikow danych.
 */
  public Enumeration getDataFiles() {
    return m_dataFiles.elements();
  }

/**
 * Zwraca plik danych o podanym identyfikatorze.
 * @param id identyfikator pliku danych.
 * @return plik danych o podanym identyfikatorze.
 */
  public Datafile getDataFile(int id) {
    return (Datafile)m_dataFiles.get(new Integer(id));
  }

/**
 * Tworzy plik danych o wybranej lokalizacji i rozmiarze.
 * @param path gdzie utworzyc plik.
 * @param fileSize wielkosc pliku.
 * @exception Exception jesli nie mozna utworzyc pliku (np. blad dyskowy).
 * @return identyfikator utworzonego pliku.
 */
  public int createDatafile(String path, int fileSize) throws Exception {
    short id = getNewDatafileId();
    short blockSize = m_store.getStoreDescriptor().getBlockSize();

    DatafileDescriptor info = new DatafileDescriptor(id, path, fileSize, blockSize);
    Datafile file = new Datafile(info);

    formatDatafile(file);

    // pliki danych
    m_dataFiles.put(new Integer(id), file);
    m_store.getStoreDescriptor().registerDatafile(info);

    return id;
  }

/**
 * Usuwa plik danych, odrejestrowywujac go z menedzera oraz kasujac z dysku.
 * @param id identyfikator pliku danych.
 * @exception IOException jesli nie mozna skasowac pliku.
 */
  public void deleteDatafile(int id) throws IOException {
    Datafile file = getDataFile(id);

    m_store.getStoreDescriptor().unregisterDatafile(file.getDatafileDescriptor());
    m_dataFiles.remove(new Integer(id));

    String path = file.getDatafileDescriptor().getPath();

    file.close();
    File delFile = new File(path);
    delFile.delete();
  }

/**
 * Otwiera wszystkie pliki danych.
 * @exception FileNotFoundException jesli jakis plik danych nie istnieje.
 */
  public void openAllDatafiles() throws FileNotFoundException {
    for (Enumeration e = m_dataFiles.elements(); e.hasMoreElements(); ) {
      ((Datafile)e.nextElement()).open("rw");
    }    
  }

/**
 * Zamyka wszystkie pliki danych.
 * @exception IOException jesli nie mozna zamknac ktoregos pliku.
 */
  public void closeAllDatafiles() throws IOException {
    for (Enumeration e = m_dataFiles.elements(); e.hasMoreElements(); ) {
      ((Datafile)e.nextElement()).close();
    }    
  }

/**
 * Wyszukuje kolejny wolny identyfikator, ktory mozna by nadac
 * plikowi danych.
 * @return wolny identyfikator.
 */
  private short getNewDatafileId() {
    for (short i = 0; i < m_dataFiles.size(); i++) {
      if (m_dataFiles.get(new Integer(i)) == null)
        return i;
    }
    return (short) m_dataFiles.size();
  }

/**
 * Formatuje wybrany plik danych, tzn. zapisuje w nim puste bloki danych
 * oraz blok wolnej przestrzeni.
 * @param file plik danych do sformatowania.
 * @exception Exception jesli nie mozna sformatowac pliku danych.
 */
  public void formatDatafile(Datafile file) throws Exception {
    System.out.println("*** Formatting datafile " + file.getDatafileDescriptor().getPath());

    DatafileDescriptor fdesc = file.getDatafileDescriptor();
    int fileSize = fdesc.getFileSize();
    short blockSize = m_store.getBlockSize();
  
    file.open("rw");

    PersistentBlockManager blockman = new PersistentBlockManager(m_store);

    // zapisanie sformatowany blokow danych
    PersistentBlockID dtBlockID =
      new PersistentBlockID(m_store.getStoreID(), fdesc.getFileID(), (short) 0);
    DataBlock dtBlock = new DataBlock(blockSize, dtBlockID);

    for (short i = PersistentBlockManager.FIRST_DATA_BLOCK; i < fileSize / blockSize; i++) {
      dtBlockID.setBlockID(i);
      blockman.writeBlock(dtBlock);
    }

    // zapisanie sformatowanego bloku wolnej przestrzeni
    PersistentBlockID fsBlockID =
      new PersistentBlockID(m_store.getStoreID(),
                            fdesc.getFileID(),
                            PersistentBlockManager.FIRST_FREE_SPACE_BLOCK);
    FreeSpaceBlock fsBlock = new FreeSpaceBlock(blockSize, fsBlockID);

    fsBlock.setBlocksFreeSpace((short) 0, (short) 0);
    for (short i = PersistentBlockManager.FIRST_DATA_BLOCK; i < fsBlock.blocksHolded(); i++) {
      fsBlock.setBlocksFreeSpace(i, dtBlock.getFreeSpaceSize());
    }
    blockman.writeBlock(fsBlock);
  }

/**
 * Formatuje wszystkie pliki danych.
 * @exception Exception jesli nie mozna sfortmatowac jakiegos pliku
 * (np. blad dyskowy).
 */
  public void formatDatafiles() throws Exception {
    for (Enumeration e = getDataFiles(); e.hasMoreElements(); ) {
      Datafile file = (Datafile) e.nextElement();
      formatDatafile(file);
    }
  }

}
