package pl.edu.pjwstk.yaod.store.defaultstore.persistentstore;

import pl.edu.pjwstk.yaod.store.defaultstore.blocks.*;
import pl.edu.pjwstk.yaod.util.*;

/**
 * Klasa reprezentuje identyfikator bloku. Dla skladu trwalych obiektow
 * ma on postac <plik, blok>.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class PersistentBlockID extends BlockID {
  private short m_file, m_block;
  private final static int RAW_STRUCT_LENGTH = NewBits.SHORT_SIZE * 2;

/**
 * Tworzy nowy identyfikator bloku.
 * @param file id pliku, w ktorym blok jest zapisany.
 * @param block numer bloku od poczatku pliku (dokladny poczatek block * rozmiar bloku).
 * @return nowy identyfikator.
 */
  public PersistentBlockID(short file, short block) {
    super((short) 0);
    m_file = file;
    m_block = block;
  }

/**
 * Tworzy nowy identyfikator bloku.
 * @param store identyfikator skladu do ktorego nalezy blok.
 * @param file id pliku, w ktorym blok jest zapisany.
 * @param block numer bloku od poczatku pliku (dokladny poczatek block * rozmiar bloku).
 * @return nowy identyfikator.
 */
  public PersistentBlockID(short store, short file, short block) {
    super(store);
    m_file = file;
    m_block = block;
  }

/**
 * Zwraca identyfikator pliku danych.
 * @return identyfikator pliku.
 */
  public short getFileID() {
    return m_file;
  }

/**
 * Ustala identyfikator pliku danych.
 * @param identyfikator pliku danych.
 */
  public void setFileID(short file) {
    m_file = file;
  }

/**
 * Ustala numer bloku w pliku danych.
 * @param block identyfikator bloku.
 */
  public void setBlockID(short block) {
    m_block = block;
  }

/**
 * Zwraca numer bloku w pliku danych.
 * @return numer bloku.
 */
  public short getBlockID() {
    return m_block;
  }

/**
 * Zwraca klucz haszowania, wykorzystywany w buforze blokow
 * (jest obiektem Hashtable).
 * @return klucz haszowania.
 */
  public int hashCode() {
    return getStoreID() * 10000 + getFileID() * 100 + getBlockID();
  }

/**
 * Sprawdza czy dwa identyfikatory wskazuja na ten sam blok.
 * @param par porownywany identyfiaktor.
 * @return oba identyfikatory wskazuja na ten sam blok (true) lub nie (false).
 */
  public boolean equals(Object par) {
    if (par instanceof PersistentBlockID == false)
      return false;

    PersistentBlockID cmp = (PersistentBlockID) par;

    return cmp.getFileID() == getFileID() &&
           cmp.getBlockID() == getBlockID() &&
           cmp.getStoreID() == getStoreID();
  }

/**
 * Zwraca dlugosc zserializowanego identyfikatora bloku.
 * @return dlugosc zserializowanego identyfikatora bloku.
 */
  public short sizeOf() {
    return RAW_STRUCT_LENGTH;
  }

/**
 * Serializuje identyfikator bloku.
 * @return zserializowany identfikator bloku.
 */
  public byte[] toByteArray() {
    byte[] rawID = new byte[RAW_STRUCT_LENGTH];

    NewBits.putShort(rawID, 0, getFileID());
    NewBits.putShort(rawID, NewBits.SHORT_SIZE, getBlockID());

    return rawID;
  }
}
