package pl.edu.pjwstk.yaod.store.defaultstore.persistentstore;

import java.io.*;

import java.util.*;

import pl.edu.pjwstk.yaod.store.*;
import pl.edu.pjwstk.yaod.store.defaultstore.blocks.*;
import pl.edu.pjwstk.yaod.system.buffers.*;
import pl.edu.pjwstk.yaod.util.*;

/**
 * Klasa reprezentuje menedzer blokow, ktorego zadaniem jest wczytywanie
 * i zapisywanie blokow w skladzie trwalych obiektow. Menedzerem blokow
 * posluguje sie menedzer rekordow, ktory tworzony jest dla kazdego
 * procesu uslugowego.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class PersistentBlockManager {
  private final static byte DATA_BLOCK = 1;
  private final static byte FREESPACE_BLOCK = 2;

  public final static short FIRST_FREE_SPACE_BLOCK = 0;
  public final static short FIRST_DATA_BLOCK = 1;

  private PersistentStore m_store;

/**
 * Tworzy nowy obiekt menedzera blokow.
 * @param store dla jakiego skladu pracowac ma ten menedzer.
 * @return nowy menedzer.
 */
  public PersistentBlockManager(PersistentStore store) {
    m_store = store;
  }

/**
 * Zwraca rozmiar bloku, jaki wykorzystuje dany sklad.
 * @return rozmiar bloku.
 */
  public short getBlockSize() {
    return m_store.getBlockSize();
  }

/**
 * Zapisuje blok na dysku.
 * @param block jaki blok zapisac.
 * @exception IOException jesli nie mozna zapisac bloku.
 */
  public void writeBlock(AbstractBlock block) throws IOException {
    FileManager fm = m_store.getFileManager();
    PersistentBlockID blockID = (PersistentBlockID) block.getBlockID();

    Datafile file = fm.getDataFile(blockID.getFileID());

    byte[] rawBlock = block.toByteArray();

    NewBits.putShort(rawBlock, 0, blockID.getFileID());
    NewBits.putShort(rawBlock, NewBits.SHORT_SIZE, blockID.getBlockID());

    if (block instanceof DataBlock)
      rawBlock[NewBits.SHORT_SIZE * 2] = DATA_BLOCK;
    else if (block instanceof FreeSpaceBlock)
      rawBlock[NewBits.SHORT_SIZE * 2] = FREESPACE_BLOCK;

    file.write(rawBlock, blockID.getBlockID() * file.getDatafileDescriptor().getBlockSize());
  }

/**
 * Aktualizuje informacje o wolnej przestrzeni w podanym bloku.
 * Informacja ta zapisywana jest w bloku wolnej przestrzeni nalezacym
 * do pliku w ktorym zapisany jest podany blok.
 * @param block blok danych ktorego informacja o wolnej przestrzeni
 * ma zostac zaktualizowana.
 * @exception Exception jesli wystapil blad uniemozliwiajacy aktualizacje.
 */
  public void updateFreeSpace(DataBlock block) throws Exception {
    PersistentBlockID dtBlockID = (PersistentBlockID) block.getBlockID();
    PersistentBlockID fsBlockID =
      new PersistentBlockID(dtBlockID.getFileID(), (short) FIRST_FREE_SPACE_BLOCK);
    FreeSpaceBlock fsBlock = readFreeSpaceBlock(fsBlockID);
    fsBlock.setBlocksFreeSpace(dtBlockID.getBlockID(), block.getFreeSpaceSize());
  }

/**
 * Wczytuje blok wolnej przestrzeni o podanym identyfikatorze.
 * @param ptr identyfikator bloku wolnej przestrzeni.
 * @exception IOException nie mozna wczytac bloku z powodu bledu i/o.
 * @exception BlockFormatException wczytany blok jest niezgodny z formatem
 * bloku wolnej przestrzeni (nie mozna zdeserializowac).
 * @return wczytany blok wolnej przestrzeni.
 */
  public FreeSpaceBlock readFreeSpaceBlock(AbstractBlockID ptr)
      throws IOException, BlockFormatException {
    PersistentBlockID blockID = (PersistentBlockID) ptr;

    BlockBuffer blockBuffer = m_store.getDatabaseManager().getBlockBuffer();
    FreeSpaceBlock block = (FreeSpaceBlock) blockBuffer.get(ptr);

    if (block == null) {
      PersistentBlockID fsBlockID =
        new PersistentBlockID(blockID.getFileID(), blockID.getBlockID());
      byte[] rawBlock = readDiskBlock(fsBlockID);

      short hFile = NewBits.getShort(rawBlock, 0);
      short hBlock = NewBits.getShort(rawBlock, NewBits.SHORT_SIZE);
      byte hType = rawBlock[NewBits.SHORT_SIZE * 2];

      if (hType != FREESPACE_BLOCK)
        throw new BlockFormatException();

      PersistentBlockID hBlockID =
        new PersistentBlockID(m_store.getStoreID(), hFile, hBlock);

      block = new FreeSpaceBlock(rawBlock, hBlockID);
      
      blockBuffer.put(block);
    }

    return block; 
  }

/**
 * Wyszukuje blok danych zawierajacy wymagana ilosc wolnego miejsca.
 * @param reqSpace wymagana ilosc wolnego miejsca.
 * @exception Exception jesli nie mozna znalezc takiego bloku z powodu awarii.
 * @return blok danych zawierajacy wymagana ilosc wolnego miejsca.
 */
  public DataBlock findDataBlockByFreeSpace(int reqSpace) throws Exception {
    FileManager fm = m_store.getFileManager();

    for (Enumeration e = fm.getDataFiles(); e.hasMoreElements(); ) {
      Datafile file = (Datafile) e.nextElement();

      PersistentBlockID fsbPtr =
        new PersistentBlockID((short) file.getDatafileDescriptor().getFileID(),
                                      FIRST_FREE_SPACE_BLOCK);
      FreeSpaceBlock fsb = readFreeSpaceBlock(fsbPtr);

      for (short i = FIRST_DATA_BLOCK; i < file.holdsBlocks(); i++) {
        int freeSpace = fsb.getBlocksFreeSpace(i);
        if (reqSpace <= freeSpace) {
          PersistentBlockID blockPtr =
            new PersistentBlockID((short) file.getDatafileDescriptor().getFileID(), i);
          DataBlock block = readDataBlock(blockPtr);
          if (reqSpace > block.getFreeSpaceSize()) {
            updateFreeSpace(block);
            return findDataBlockByFreeSpace(reqSpace);
          }
          else
            return block;
        }
      }
    }
    throw new BlockFreeSpaceException();
  }

/**
 * Wczytuje blok danych o podanym identyfikatorze.
 * @param identyfikator bloku do wczytania.
 * @exception IOException nie mozna wczytac bloku.
 * @exception BlockFormatException zserializowana wersja bloku nie odpowiada
 * formatowi bloku (nie mozna zdeserializowac).
 * @return wczytany blok danych.
 */
  public DataBlock readDataBlock(AbstractBlockID ptr)
      throws IOException, BlockFormatException {
    BlockBuffer blockBuffer = m_store.getDatabaseManager().getBlockBuffer();
    DataBlock block = (DataBlock) blockBuffer.get(ptr);

    if (block == null) {
      byte[] rawBlock = readDiskBlock((PersistentBlockID) ptr);

      short hFile = NewBits.getShort(rawBlock, 0);
      short hBlock = NewBits.getShort(rawBlock, NewBits.SHORT_SIZE);
      byte hType = rawBlock[NewBits.SHORT_SIZE * 2];

      if (hType != DATA_BLOCK)
        throw new BlockFormatException();

      PersistentBlockID hBlockID =
        new PersistentBlockID(m_store.getStoreID(), hFile, hBlock);

      block = new DataBlock(rawBlock, hBlockID);
      blockBuffer.put(block);
    }

    return block;
  }

/**
 * Wczytuje blok o podanym identyfikatorze.
 * @param ptr identyfikator bloku danych.
 * @exception IOException nie mozna wczytac pliku danych.
 * @return wczytany blok danych w postaci zserializowanej.
 */
  private byte[] readDiskBlock(PersistentBlockID ptr) throws IOException {
    FileManager fm = m_store.getFileManager();
    Datafile file = fm.getDataFile(ptr.getFileID());
    int blockSize = file.getDatafileDescriptor().getBlockSize();

    byte[] rawBlock = new byte[blockSize];
    file.read(rawBlock, ptr.getBlockID() * blockSize);

    return rawBlock;
  }
}
