package pl.edu.pjwstk.yaod.store.defaultstore.persistentstore;

import java.io.*;

import java.util.*;

import pl.edu.pjwstk.yaod.store.*;
import pl.edu.pjwstk.yaod.store.defaultstore.blocks.*;
import pl.edu.pjwstk.yaod.util.*;

/**
 * Klasa zapewnia niskopoziomowa funkcjonalnosc domyslnego skladu trwalych
 * obiektow. Jej podstawowym zadaniem jest tworzenie rekordow w ramach
 * blokow danych oraz wczytywanie blokow danych z dysku lub bufora blokow.
 * Instancja tej klasy jest czescia procesu uslugowego, a jej metody
 * wywolywane sa przez oslone (wrapper), udostepniajaca uslugi tworzenia atomow
 * dla menadzera atomow.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class PersistentRecordManager {
  private final static short FIRST_FREE_SPACE_BLOCK = 1;
  private final static short FIRST_DATA_BLOCK = 2;

  private PersistentBlockManager m_blockman;

  public PersistentRecordManager(PersistentBlockManager bm) {
    m_blockman = bm;
  }

/**
 * Tworzy nowy rekord. Jesli rekord jest zbyt duzy by zmiescic go w jednym
 * bloku, wowczas dzieli go na fragmenty i zapisuje jako rekord lancuchowany
 * w formie kilku rekordow prostych. Kazdy rekord lancuchowany posiada
 * na koncu 8 bajtow wskaznika do nastepnej czesci. Jesli rekord nie jest
 * lancuchowany (w calosci miesci sie w jednym bloku), wowczas nie posiada
 * takiego wskaznika. Ostatnia czesc rekordu lancuchowanego rowniez go nie
 * posiada. Informacja o tym, czy dany rekord prosty ma wiecej fragmentow
 * zapisywana jest przez podniesienie specjalnej flagi (wiecej o tym
 * w DataBlock.createRecord()).
 * @param data dane do zapisania.
 * @exception NoFreeSpaceException nie mozna utworzyc bloku ze wzgledu na
 * brak wolnego miejsca.
 * @return wskaznik do utworzonego rekordu.
 */

  public PersistentRecordID createRecord(byte[] data) throws Exception {
    PersistentRecordID newRecordPtr;
    DataBlock dumbBlock =
      new DataBlock(m_blockman.getBlockSize(), new PersistentBlockID((short) 0, (short) 0));
  
    int reqSpace = DataBlock.getRawRecordLength(data);

    short maxFreeSpaceLength = dumbBlock.getFreeSpaceSize();
    short maxRecordLength = (short)(maxFreeSpaceLength - DataBlock.RECORD_HEADER_LENGTH -
                                                         DataBlock.SLOT_LENGTH -
                                                         DataBlock.NEXT_RECORD_LENGTH);

    // czy rekord nie zmiesci sie w jednym bloku?
    if (reqSpace > maxRecordLength) {
      int start = 0;
      int left = data.length;

      byte[] partData;
      boolean chained;
      
      Vector parts = new Vector();

      // podzial na czesci i przydzial miejsca na nie w blokach
      while (left > 0) {
        if (left > maxRecordLength) {
          partData = new byte[maxRecordLength + DataBlock.NEXT_RECORD_LENGTH];
          chained = true;
        }
        else {
          partData = new byte[left];
          chained = false;
        }

        System.arraycopy(data, start, partData, 0, partData.length);
        
        DataBlock block =
          m_blockman.findDataBlockByFreeSpace(DataBlock.getRawRecordLength(partData));

        short partSlot = block.createRecord(partData, chained);
        m_blockman.updateFreeSpace(block);

        AbstractBlockID blockID = (AbstractBlockID) block.getBlockID();
        PersistentRecordID recordID = new PersistentRecordID(blockID, partSlot);
        parts.addElement(recordID);

        left -= partData.length;
        start += partData.length;
      }

      if (parts.size() < 1) {
        pl.edu.pjwstk.yaod.util.Errors.printError(1, "createRecord()");
        throw new Exception();
      }

      // ustawienie wskaznikow kolejnych czesci
      PersistentRecordID currPart, nextPart;
      for (int i = 0; i < parts.size() - 1; i++) {
        currPart = (PersistentRecordID)parts.elementAt(i);
        nextPart = (PersistentRecordID)parts.elementAt(i + 1);

        DataBlock block = m_blockman.readDataBlock(currPart.getBlockID());
        block.setNextPart(currPart.getSlotID(), nextPart.toByteArray());
      }

      newRecordPtr = (PersistentRecordID)parts.firstElement();
    }
    else {
      DataBlock block = m_blockman.findDataBlockByFreeSpace(reqSpace);
      short slot = block.createRecord(data, false);
      
      m_blockman.updateFreeSpace(block);

      AbstractBlockID blockID = (AbstractBlockID) block.getBlockID();
      newRecordPtr = new PersistentRecordID(blockID, slot);
    }

    return newRecordPtr;
  }

/**
 * Aktualizuje rekord o podanym wskazniku, zastepujac go nowymi danymi.
 * Zastosowano uproszczona wersje aktualizacji: jesli rekord jest
 * lancuchowany i ma 1..n czesci, wowczas najpierw kasuje czesci 2..n,
 * nastepnie tworzy nowy rekord, a czesc 1 modyfikuje w taki sposob,
 * by nie zawierala zadnych danych, oprocz wskaznika na poczatek nowego rekordu.
 * @param recordPtr aktualizowany rekord.
 * @param data dane, jakie maja zastapic aktualizowany rekord.
 * @exception NoSuchRecordException brak takiego rekordu.
 */

  public void updateRecord(PersistentRecordID recordPtr, byte[] data) throws NoSuchRecordException {
    try {
      DataBlock block = m_blockman.readDataBlock(recordPtr.getBlockID());

      // jesli rekord ma wiecej czesci, to skasuj je
      if (block.isChainedRecord(recordPtr.getSlotID())) {
        PersistentRecordID nextPart =
          new PersistentRecordID(block.getNextPart(recordPtr.getSlotID()));
        deleteRecord(nextPart);
      }

      int exRecLength = block.getRecordLength(recordPtr.getSlotID());
      int ftRecLength =  DataBlock.getRawRecordLength(data);

      // czy podzielic na czesci?
      if (block.getFreeSpaceSize() + exRecLength < ftRecLength) {
        byte[] tmp = new byte[DataBlock.NEXT_RECORD_LENGTH];
        block.updateRecord(recordPtr.getSlotID(), tmp, true);

        m_blockman.updateFreeSpace(block);

        PersistentRecordID newPtr = createRecord(data);

        block.setNextPart(recordPtr.getSlotID(), newPtr.toByteArray());
      }
      else
        block.updateRecord(recordPtr.getSlotID(), data, false); 
    }
    catch (Exception e) {
      e.printStackTrace();
      throw new NoSuchRecordException();
    }
  }

/**
 * Usuwa rekord identyfikowany przez podany wskaznik rekordu. Jesli rekord
 * jest lancuchowany, to usuwa wszystkie jego czesci.
 * @param recordPtr wskaznik do usuwanego rekordu.
 * @exception NoSuchRecordException rekord wskazywany przez recordPtr nie istnieje.
 */

  public void deleteRecord(PersistentRecordID recordPtr) throws NoSuchRecordException, Exception {
    DataBlock block = m_blockman.readDataBlock(recordPtr.getBlockID());

    if (block.isChainedRecord(recordPtr.getSlotID())) {
      PersistentRecordID nextPart;

      do {
        nextPart = new PersistentRecordID(block.getNextPart(recordPtr.getSlotID()));

        block.deleteRecord(recordPtr.getSlotID());
        m_blockman.updateFreeSpace(block);

        if (nextPart != null) {
          recordPtr = nextPart;
          block = m_blockman.readDataBlock(recordPtr.getBlockID());
        }
        
      } while (nextPart != null);
    }
    else {
      block.deleteRecord(recordPtr.getSlotID());
      m_blockman.updateFreeSpace(block);
    }
  }

/**
 * Zwraca rekord identyfikowany podanym wskaznikiem. Jesli jest to rekord
 * lancuchowany (zapisany w kilku czesciach umieszczonych w osobnych blokach),
 * wowczas czesci te skladane sa w jedna calosc.
 * @param recordPtr wskaznik do rekordu.
 * @exception NoSuchRecordException   brak takiego rekordu.
 * @return dane rekordu w formie ciagu bajtow.
 */

  public byte[] getRecord(PersistentRecordID recordPtr) throws NoSuchRecordException {
    byte[] data = null;

    try {
      DataBlock block = m_blockman.readDataBlock(recordPtr.getBlockID());

      if (block.isChainedRecord(recordPtr.getSlotID())) {
        while (true) {
          byte[] buf = block.getRecord(recordPtr.getSlotID());
          byte[] rawNextPart = block.getNextPart(recordPtr.getSlotID());

          if (rawNextPart != null) {
            recordPtr = new PersistentRecordID(rawNextPart);
            data = NewBits.concat(data, buf, buf.length - DataBlock.NEXT_RECORD_LENGTH);
            block = m_blockman.readDataBlock(recordPtr.getBlockID());
          }
          else {
            data = NewBits.concat(data, buf, buf.length);
            break;
          }
        }
      }
      else
        data = block.getRecord(recordPtr.getSlotID());
    }
    catch (NoSuchSlotException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "Slot error");
      throw new NoSuchRecordException();
    }
    catch (IOException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "I/O error.");
      throw new NoSuchRecordException();
    }
    catch (BlockFormatException e) {
      pl.edu.pjwstk.yaod.util.Errors.printError(1, "Faulty block format.");
      throw new NoSuchRecordException();
    }

    return data;
  }

}
