package pl.edu.pjwstk.yaod.store.defaultstore.persistentstore;

import java.io.*;

import pl.edu.pjwstk.yaod.atoms.*;
import pl.edu.pjwstk.yaod.store.*;
import pl.edu.pjwstk.yaod.system.*;
import pl.edu.pjwstk.yaod.util.*;

/**
 * Klasa reprezentuje sklad trwalych obiektow.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class PersistentStore extends AbstractStore {
  private PersistentStoreDescriptor m_descriptor;
  private FileManager m_fileman;
  private DatabaseManager m_dbman;
  private String m_ctlFilePath;

/**
 * Tworzy nowy obiekt skladu trwalych obiektow.
 * @param id identyfikator skladu.
 * @param ctlPath sciezka dostepu do pliku kontrolnego skladu.
 * @param manager menedzer bazy danych zarzadzajacy tym skladem.
 * @exception jesli nie mozna utworzyc skladu (np. brak pliku kontrolnego).
 * @return nowy obiekt skladu.
 */
  public PersistentStore(short id, String ctlPath, DatabaseManager manager) throws Exception {
    super(id);

    m_ctlFilePath = ctlPath;
    readControlFile();

    m_dbman = manager;
    
    m_fileman = new FileManager(this);
  }

/**
 * Wczytuje plik kontrolny skladu.
 * @exception Exception jesli nie mozna wczytac pliku kontrolnego.
 */
  public void readControlFile() throws Exception {
    m_descriptor = (PersistentStoreDescriptor) new ObjectSerializer().readObject(m_ctlFilePath);
  }

/**
 * Zapisuje plik kontrolny skladu na dysku.
 * @exception jesli nie mozna zapisac pliku na dysku.
 */
  public void writeControlFile() throws Exception {
    new ObjectSerializer().writeObject(m_descriptor, m_ctlFilePath);
  }

/**
 * Inicjalizuje sklad, formatujac wszystkie pliki danych
 * i tworzac atom wejsciowy.
 * @exception jesli nie mozna zainicjalizowac bazy (np. brak plikow).
 */
  public void initialize() throws Exception {
    System.out.println("*** Initializing persistent store ...");
    m_fileman.formatDatafiles();

    PersistentStoreWrapper wrapper = (PersistentStoreWrapper) createClient();

    SpiderAtom entry = new SpiderAtom("entry");
    wrapper.createAtom(entry);

    setEntry(entry.getAID());
  }

/**
 * Otwiera sklad otwierajac wszystkie pliki danych.
 * @exception FileNotFoundException brak jakiegos pliku danych.
 */
  public void open() throws FileNotFoundException {
    m_fileman.openAllDatafiles();
  }

/**
 * Zamyka sklad zamykajac wszystkie pliki danych.
 * @exception Exception jesli nie mozna zamknac plikow danych.
 */
  public void close() throws IOException {
    m_fileman.closeAllDatafiles();
  }

/**
 * Zwraca menedzera plikow.
 * @return obiekt menedzera plikow.
 */
  public FileManager getFileManager() {
    return m_fileman;
  }

/**
 * Zwraca menedzera bazy danych zarzadzajacego tym skladem.
 * @return menedzer bazy danych.
 */
  public DatabaseManager getDatabaseManager() {
    return m_dbman;
  }

/**
 * Tworzy oslone skladu.
 * @return obiekt oslony.
 */
  public Wrappable createClient() {
    return new PersistentStoreWrapper(this);
  }

/**
 * Ustala identyfikator atomu wejsciowego.
 * @param entry identyfikator atomu wejsciowego.
 */
  public void setEntry(AtomID entry) {
    m_descriptor.setEntry(entry);
    try {
      writeControlFile();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

/**
 * Zwraca identyfikator atomu wejsciowego do skladu.
 * @return obiekt atomu wejsciowego.
 */
  public AtomID getEntry() {
    return m_descriptor.getEntry();
  }

/**
 * Zwraca deskryptor skladu.
 * @return deksryptor skladu.
 */
  public PersistentStoreDescriptor getStoreDescriptor() {
    return m_descriptor;
  }

/**
 * Zwraca wielkosc bloku, jakim posluguje sie sklad.
 * @return wielkosc bloku.
 */
  public short getBlockSize() {
    return m_descriptor.getBlockSize();
  }
}
