package pl.edu.pjwstk.yaod.store.defaultstore.persistentstore;

import pl.edu.pjwstk.yaod.atoms.*;
import pl.edu.pjwstk.yaod.store.*;
import pl.edu.pjwstk.yaod.store.defaultstore.blocks.*;
import pl.edu.pjwstk.yaod.system.buffers.*;

/**
 * Klasa stanowi oslone (wrapper) nad skladem trwalych obiektow.
 * Warstwom wyzszym udostepnia interfejs CRUD atomow, umozliwiajac przykrycie
 * zlozonych zagadnien zwiazanych z funkcjonowaniem blokow, rekordow itp.
 * Oprocz funkcji charakterystycznych dla kazdego skladu (interfejs Wrappable),
 * zapewnia rowniez funkcje do zarzadzania tylko i wylacznie domyslnym
 * skladem trwalych obiektow. Funkcje te maja charakter administracyjny
 * i charakterystyczne sa tylko i wylacznie dla danego rodzaju skladu,
 * jednak musza byc udostepnione na zewnatrz.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class PersistentStoreWrapper implements Wrappable {
  private PersistentStore m_store;
  private PersistentRecordManager m_recman;

/**
 * Konstruktor tworzacy nowy obiekt oslony dla domyslnego skladu trwalych
 * obiektow.
 * @param store obiekt domyslnego skladu trwalych obiektow, dla ktorego
 * tworzona jest oslona.
 * @return nowy obiekt oslony.
 */

  public PersistentStoreWrapper(PersistentStore store) {
    m_recman = new PersistentRecordManager(new PersistentBlockManager(store));
    m_store = store;
  }

/**
 * Zwraca identyfikator obiektu wejsciowego do skladu.
 * @return identyfikator obiektu wejsciowego.
 */

  public AtomID getEntry() {
    return m_store.getEntry();
  }

/**
 * Odczytuje ze skladu rekord wskazywany danym identyfikatorem,
 * a nastepnie tworzy na jego podstawie atom. Najpierw sprawdzany jest
 * bufor atomow, a dopiero pozniej analizowana jest zawartosc dysku.
 * @param ptr identyfikator atomu, ktory ma zostac wczytany.
 * @exception NoSuchAtomException brak takiego atomu.
 * @return odczytany atom.
 */

  public Atom retrieveAtom(AtomID ptr) throws NoSuchAtomException {
    try {
    
      AtomBuffer atomBuffer = m_store.getDatabaseManager().getAtomBuffer();
      Atom atm = atomBuffer.get(ptr);
      if (atm != null) return atm;

      PersistentAtomID atmPtr = (PersistentAtomID)ptr;
      byte[] rawRecord = m_recman.getRecord(atmPtr);

      AtomSerializer ser = new AtomSerializer();
      atm = ser.atomDeserialize(rawRecord);
      atm.setAID(atmPtr);

      atomBuffer.put(atm);

      return atm;
    }
    catch (NoSuchRecordException e) {
      throw new NoSuchAtomException();
    }
    catch (Exception e) {
      throw new NoSuchAtomException();
    }
  }

/**
 * Kasuje atom o podanym identyfikatorze.
 * @param ptr identyfikator atomu, ktory ma zostac wczytany.
 * @exception NoSuchAtomException brak takiego atomu.
 */

  public void deleteAtom(AtomID ptr) throws NoSuchAtomException {
    try {
      AtomBuffer atomBuffer = m_store.getDatabaseManager().getAtomBuffer();
      atomBuffer.remove(ptr);
      m_recman.deleteRecord((PersistentAtomID)ptr);
    }
    catch (Exception e) {
      throw new NoSuchAtomException();
    }
  }

/**
 * Aktualizuje podany atom.
 * @param atm atom utworzony wczesniej w bazie.
 * @exception BlockFreeSpaceException za malo wolnego miejsca.
 */

  public void updateAtom(Atom atm) throws BlockFreeSpaceException {
    try {
      AtomSerializer ser = new AtomSerializer();
      byte[] rawAtm = ser.atomSerialize(atm);
  
      PersistentAtomID atmPtr = (PersistentAtomID)atm.getAID();

      m_recman.updateRecord(atmPtr, rawAtm);
    }
    catch (Exception e) {
      e.printStackTrace();
      
      throw new BlockFreeSpaceException();
    }
  }

/**
 * Zapisuje w skladzie podany atom.
 * @param atm atom, ktory ma zostac umieszczony w skladzie.
 * @exception BlockFreeSpaceException za malo wolnego miejsca.
 */

  public void createAtom(Atom atm) throws BlockFreeSpaceException {
    try {
      AtomSerializer ser = new AtomSerializer();
      byte[] rawAtm = ser.atomSerialize(atm);
    
      PersistentRecordID recPtr = m_recman.createRecord(rawAtm);
      PersistentBlockID blockID = (PersistentBlockID) recPtr.getBlockID();
      
      atm.setAID(new PersistentAtomID(m_store.getStoreID(),
                                          blockID.getFileID(),
                                          blockID.getBlockID(),
                                          recPtr.getSlotID()));

      AtomBuffer atomBuffer = m_store.getDatabaseManager().getAtomBuffer();
      atomBuffer.put(atm);
    }
    catch (Exception e) {
      e.printStackTrace();
      throw new BlockFreeSpaceException();
    }
  }
}
