package pl.edu.pjwstk.yaod.system;

import java.net.*;

import java.util.*;

import pl.edu.pjwstk.yaod.store.*;
import pl.edu.pjwstk.yaod.system.buffers.*;
import pl.edu.pjwstk.yaod.system.processes.*;
import pl.edu.pjwstk.yaod.network.*;

/**
 * Serce systemu. Tworzy procesy i bufory, otwiera i zamyka sklady.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class DatabaseManager {
  private Database m_database;
  private Hashtable m_stores;
  private AtomBuffer m_atomBuffer;
  private BlockBuffer m_blockBuffer;

  private DBWR m_dbwr;
  private LSNR m_lsnr_ynet;
  private LSNR m_lsnr_http;

/**
 * Tworzy nowego menedzera danych.
 * @param db baza danych, ktora ma byc zarzadzana przez menedzera.
 * @exception Exception gdy cos poszlo nie tak (np. nie mozna otworzyc skladu).
 * @return nowy menedzer danych.
 */
  public DatabaseManager(Database db) throws Exception {
    m_database = db;
    db.initDBManager(this);
  }

/**
 * Pozwala na wprowadzenie do menedzera listy obiektow reprezentujacych sklady.
 * Wywolywana przez Database podczas inicjalizacji menedzera.
 * @param stores tablica zawierajaca obiekty reprezentujace sklady obiektow.
 */
  public void setStores(Hashtable stores) {
    m_stores = stores;
  }

/**
 * Zwraca obiekt bazy danych, ktora jest zarzadzana przez danego menedzera.
 * @return obiekt bazy danych.
 */
  public Database getDatabase() {
    return m_database;
  }

/**
 * Zwraca liste skladow, ktorymi zarzadza menedzer, a ktore sa skladowymi
 * bazy danych.
 * @return lista skladow (obiektow AbstractStore).
 */
  public Enumeration getStores() {
    return m_stores.elements();
  }

/**
 * Zwraca sklad identyfikowany okreslonym numerem.
 * @param Wartosc Short zawierajaca numer skladu.
 * @return sklad o podanym identyfikatorze.
 */
  public AbstractStore getStoreById(Object key) {
    return (AbstractStore) m_stores.get(key);
  }

/**
 * Zwraca obiekt bufora blokow.
 * @return bufor blokow.
 */
  public BlockBuffer getBlockBuffer() {
    return m_blockBuffer;
  }

/**
 * Zwraca bufor atomow.
 * @return bufor atomow.
 */
  public AtomBuffer getAtomBuffer() {
    return m_atomBuffer;
  }

/**
 * Inicjalizuje baze danych. W przypadku skladu trwalych obiektow
 * jest to np. sformatowanie plikow danych.
 * @exception Exception jesli cos poszlo nie tak (np. brak pliku danych).
 */
  public void initialize() throws Exception {
    for (Enumeration e = m_stores.elements(); e.hasMoreElements(); ) {
      AbstractStore store = (AbstractStore)e.nextElement();
      store.initialize();
    }
  }

/**
 * Uruchamia menedzera. Otwiera wszystkie sklady, uruchamia procesy,
 * alokuje pamiec na bufory.
 * @exception Exception jesli nie mozna uruchomic menedzera.
 */
  public void start() throws Exception {
    for (Enumeration e = m_stores.elements(); e.hasMoreElements(); ) {
      AbstractStore store = (AbstractStore)e.nextElement();
      store.open();
    }

    m_lsnr_http = new LSNR(this, SVRP.HTTP, 82);
    m_lsnr_ynet = new LSNR(this, SVRP.YNET, 1521);
    m_dbwr = new DBWR(this);

    m_atomBuffer = new AtomBuffer(m_dbwr);
    m_blockBuffer = new BlockBuffer(m_dbwr);

    m_dbwr.startDBWR();
    m_lsnr_http.startLSNR();
    m_lsnr_ynet.startLSNR();
  }

/**
 * Zatrzymuje menedzera. Przerywa dzialanie procesow, zamyka sklady.
 */
  public void stop() {
    try {
      m_lsnr_ynet.stopLSNR();
      m_lsnr_http.stopLSNR();
      m_dbwr.stopDBWR();

      System.out.println("*** Waiting for DBWR & LSNRs ...");
      while (m_dbwr.isAlive() || m_lsnr_http.isAlive() || m_lsnr_ynet.isAlive()) { }

      for (Enumeration e = m_stores.elements(); e.hasMoreElements(); ) {      
        AbstractStore store = (AbstractStore) e.nextElement();
        store.close();
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

/**
 * Tworzy i uruchamia nowy proces uslugowy. Wywolywana przez LSNR przy nadejsciu
 * nowego polaczenia od klienta.
 * @param socket gniazdko sluzace do komunikacji z klientem.
 * @exception IOException jesli problem z komunikacja.
 */
  public void createServerProcess(Socket socket, int protocol) throws java.io.IOException {
    SVRP proc = new SVRP(this, socket, protocol);
    proc.start();
  }
}
