package pl.edu.pjwstk.yaod.system.processes;

import java.util.*;

import pl.edu.pjwstk.yaod.atoms.*;
import pl.edu.pjwstk.yaod.store.*;
import pl.edu.pjwstk.yaod.store.defaultstore.persistentstore.*;
import pl.edu.pjwstk.yaod.system.*;
import pl.edu.pjwstk.yaod.system.buffers.*;

/**
 * Klasa procesu DBWR (od database writer). Proces DBWR zajmuje sie
 * przenoszeniem zawartosci bufora atomow do bufora stron,
 * a bufora blokow na dysk (jesli sklad obsluguje dostep blokowy).
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class DBWR extends Thread {
  private boolean m_work;
  private DatabaseManager m_dbman;
  private Hashtable m_storeWrappers;

/**
 * Tworzy nowy obiekt DBWR. M.in. tworzy sobie wrappery
 * do skladow.
 * @param manager menedzer bazy danych, do ktorego nalezy dbwr.
 */
  public DBWR(DatabaseManager manager) {
    super("dbwr_thread");
    m_work = true;
    m_dbman = manager;

    m_storeWrappers = new Hashtable();
    for (Enumeration e = m_dbman.getStores(); e.hasMoreElements(); ) {
      AbstractStore store = (AbstractStore) e.nextElement();
      m_storeWrappers.put(new Integer(store.getStoreID()), store.createClient());
    }
  }

/**
 * Zwraca oslone skladu o podanym identyfikatorze.
 * @param id identyfikator skladu.
 * @return oslona skladu.
 */
  public Wrappable getStoreByID(int id) {
    return (Wrappable)m_storeWrappers.get(new Integer(id));
  }

/**
 * Uruchamia proces (watek) dbwr.
 */
  public void startDBWR() {
    super.start();
  }

/**
 * Zatrzymuje proces (watek) dbwr.
 */
  public void stopDBWR() {
    m_work = false;
  }

/**
 * Ustawia wartosc flagi, ktora nakazuje watkowi prace lub zatrzymanie sie.
 * @param status pracuj (true), zatrzymaj sie (false)
 */
  public void setWork(boolean status) {
    m_work  = status;
  }

/**
 * Wykonuje punkt kontrolny, czyli zapisuje z buforow wszystkie atomy do blokow,
 * a bloki na nosnik (jesli jest obsluga blokow). Jest to jedyny sposob
 * zapisu na trwaly nosnik, efektywny tylko gdy istnieje obsluga transakcji.
 */
  public void doCheckpoint() {
    // zapisz atomy
    AtomBuffer atomBuf = m_dbman.getAtomBuffer();
    Object[] atoms = atomBuf.elements();

    int wrat = 0;
    for (int i = 0; i < atoms.length; i++) {
      Atom atm = (Atom) atoms[i];
      if (atm.isModified() == false)
        continue;

      Wrappable store = getStoreByID(atm.getAID().getStoreID());
      try {
        store.updateAtom(atm);
        wrat++;
      }
      catch (Exception e) {
        e.printStackTrace();
      }

      atm.setModified(false);
    }

    // zapisz bloki
    BlockBuffer blockBuf = m_dbman.getBlockBuffer();
    Object[] blocks = blockBuf.elements();

    for (int i = 0; i < blocks.length; i++) {
      AbstractBlock block = (AbstractBlock) blocks[i];

      PersistentStore store =
        (PersistentStore) m_dbman.getStoreById(new Short(block.getBlockID().getStoreID()));

      PersistentBlockManager blockman = new PersistentBlockManager(store);

      try {
        blockman.writeBlock(block);
      }
      catch (Exception e) {
        e.printStackTrace();
      }

      blockBuf.remove(block.getBlockID());
    }

    System.out.println("*** DBWR checkpoint status: " + wrat + " atoms, " + blocks.length + " blocks");
  }

/**
 * Petla watku. Jesli watek ma pracowac, to usypiany jest na trzy sekundy,
 * po czym wykonuje punkt kontrolny, znowu jest usypiany, itd.
 */
  public void run() {
    System.out.println("*** DBWR started");
    try {
      while (m_work) {
        doCheckpoint();
        sleep(3000);
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    doCheckpoint();
    System.out.println("*** DBWR stopped");
  }
}
