package pl.edu.pjwstk.yaod.system.processes;

import java.io.*;

import java.net.*;

import java.util.*;

import javax.swing.tree.*;

import pl.edu.pjwstk.yaod.debug.*;
import pl.edu.pjwstk.yaod.network.*;
import pl.edu.pjwstk.yaod.network.protocol.*;
import pl.edu.pjwstk.yaod.objects.*;
import pl.edu.pjwstk.yaod.store.*;
import pl.edu.pjwstk.yaod.system.*;
import pl.edu.pjwstk.yaod.xml.*;
import pl.edu.pjwstk.yaod.yaql.*;

/**
 * Obiekty tej klasy reprezentuja klienta po stronie serwera.
 * Jeden obiekt klasy ServerProcess tworzony dla jednego nadchodzcego
 * polaczenia od klienta.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */

public class SVRP extends Thread {
  private DatabaseManager m_dbman;

  private Socket m_socket;

  private ObjectManager m_obman;
  private Hashtable m_wrappers;
  private Interpreter m_qproc;

  private Communicable m_comm;

  public final static int YNET = 1;
  public final static int HTTP = 2;

/**
 * Tworzy nowy proces uslugowy: tworzy komunikator, pobiera liste oslon
 * oslon, inicjalizuje menedzera obiektow i parser yaql.
 * @param manager menedzer bazy danych, ktory uruchomil ten proces.
 * @param socket gniazdko reprezentujace polaczenie z klientem.
 * @exception IOException nie mozna stworzyc komunikatora.
 * @return nowy obiekt procesu uslugowego.
 */
  public SVRP(DatabaseManager manager, Socket socket, int protocol) throws IOException {
    super("SVRP");

    m_dbman = manager;
    m_socket = socket;

    switch (protocol) {
      case YNET : m_comm = new YNETCommunicator(socket, this); break;
      case HTTP : m_comm = new HTTPCommunicator(socket, this); break;
    }

    // tworzymy oslone dla kazdego skladu i rejestrujemy go procesie uslugowym
    m_wrappers = new Hashtable();
    for (Enumeration e = m_dbman.getStores(); e.hasMoreElements(); ) {
      AbstractStore store = (AbstractStore) e.nextElement();
      m_wrappers.put(new Integer(store.getStoreID()), store.createClient());
    }

    // zainicjalizowanie menedzera obiektow oraz parsera yaql
    m_obman = new ObjectManager(this);
    m_qproc = new Interpreter(this);
    m_qproc.initializeENVS();
  }

/**
 * Zwraca obiekt reprezentujacy interpreter wykorzystywany
 * przez ten proces uslugowy.
 * @return obiekt interpretera.
 */
  public Interpreter getInterpreter() {
    return m_qproc;
  }

/**
 * Zwraca oslone skladu o podanym identyfikatorze.
 * @param id identyfikator skladu.
 * @return oslona skladu.
 */
  public Wrappable getStoreByID(int id) {
    return (Wrappable)m_wrappers.get(new Integer(id));
  }

/**
 * Zwraca oslony wszystkich skladow.
 * @return oslony wszystkich skladow.
 */
  public Enumeration getStores() {
    return m_wrappers.elements();
  }

/**
 * Zwraca menedzera obiektow wykorzystywanego przez ten proces.
 * @return obiekt menedzera obiektow.
 */
  public ObjectManager getObjectManager() {
    return m_obman;
  }


/**
 * Uruchamia watek realizujacy zadania procesu uslugowego:
 * nasluchiwanie na zadania klientow, wykonywanie i wysylanie odpowiedzi. 
 */
  public void run() {
    boolean m_work = true;

    System.out.println("SVRP started");
    while (m_work) {
      try {
        m_comm.serveRequest();        
      }
      catch (SocketException e) {
        m_work = false;
      }
      catch (Exception e) {
        e.printStackTrace();
        m_work = false;
      }
    }
    System.out.println("SVRP stopped");
  }

/**
 * Wykonuje instrukcje yaql. Tylko jedna sesja moze wykonac w tym samym
 * czasie jakas instrukcje (prosta blokada gruboziarnista).
 * @param qproc obiekt interpretera, ktory ma wykonac dany program yaql.
 * @exception SBQLException blad wykonania programu.
 * @return wynik wykonania programu.
 */
  public synchronized static Vector execute(Interpreter qproc, String cmd) throws SBQLException {  
    return qproc.start(cmd);
  }

}
