package pl.edu.pjwstk.yaod.xml;

import java.io.*;

import java.util.*;

import oracle.xml.parser.v2.*;

import org.w3c.dom.*;

import pl.edu.pjwstk.yaod.atoms.*;
import pl.edu.pjwstk.yaod.util.*;
import pl.edu.pjwstk.yaod.objects.*;

/**
 * Laduje dokument XML do bazy danych, wykorzystujac interfejs DOM.
 * Znaczniki zamienia na obiekty zlozone, zawartosc na obiekty TEXT,
 * a atrybuty umieszcza w obiekcie zloznym ATTRIBUTES. Referencje
 * obsluguje w nastepujacy sposob: gdy widzi element ze atrybutem
 * pointer="true", wowczas jego zawartosc traktuje jako identyfikator.
 * Wartosc identyfikatora musi byc w innym elemencie (dowolnie zagniezdzonym)
 * i oznaczona za pomoca atrybutu id.
 * 
 * @author M. Lentner, raist@ertel.com.pl
 * @version 1.0
 */
public class XMLLoader {
  private ObjectManager m_obman;
  private Document m_doc;

/**
 * Tworzy nowy obiekt loadera.
 * @param obman menedzer danych, ktory bedzie tworzyl i odczytywal obiekty.
 * @return nowy obiekt loadera.
 */
  public XMLLoader(ObjectManager obman) {
    m_obman = obman;
  }

/**
 * Wczytuje dokument do bazy danych.
 * @param strDoc dokument w formie stringu.
 * @param root pod jaki element podpiac dokument.
 * @exception Exception jesli nie mozna zaladowac dokumentu.
 */
  public void loadDocument(String strDoc, ObjectID root) throws Exception {
    DOMParser parser = new DOMParser();
    parser.parse(new StringReader(strDoc));

    m_doc = parser.getDocument();

    processNode((XMLNode) m_doc.getDocumentElement(), root);
  }

/**
 * Rekurencyjnie przeszukuje podelementy podanego elementu w celu znalezienia
 * takiego, ktorego atrybut id rowny jest podanej wartosci.
 * @param node wezel korzenia przeszukiwania.
 * @param value wartosc atrybutu id, ktora musi zostac spelniona.
 * @return znaleziony wezel lub null jesli takiego nie ma.
 */
  private Node findReferencedElement(Node root, String value) {
    NodeList nodes = root.getChildNodes();

    for (int i = 0; i < nodes.getLength(); i++) {
      Node node = nodes.item(i);
      NamedNodeMap attributes = node.getAttributes();

      for (int j = 0; attributes != null && j < attributes.getLength(); j++) {
        Node attribute = attributes.item(j);
        if (attribute.getNodeName().equals("id") &&
            attribute.getNodeValue().equals(value)) { return node; }
      }
      
      Node found = findReferencedElement(node, value);
      if (found != null) return found;
    }

    return null;
  }

/**
 * Obsluguje kolejny wezel DOM. Dla wezlow potomnych wykonuje sie rekurencyjnie.
 * @param node wezel do przetworzenia.
 * @exception Exception jesli wezel nei moze byc obsluzony wskutek bledu.
 */
  private void processNode(XMLNode node, ObjectID parentID) throws Exception {
    NodeList nodes = node.getChildNodes();

    String nodeName = node.getNodeName();
    String nodeValue = node.getNodeValue();

    switch (node.getNodeType()) {
      case Node.DOCUMENT_NODE:
        YSpider ydoc = m_obman.createSpider(nodeName);
        ydoc.connect(parentID);

        node.setProperty("oid", ydoc.getOID());

        processNode( (XMLNode) ((Document) node).getDocumentElement(), ydoc.getOID());

        break;

      case Node.ELEMENT_NODE:
        boolean isReference = false;

        NamedNodeMap children = node.getAttributes();

        YObject yelement;
        
        for (int i = 0; children != null && i < children.getLength(); i++) {
          XMLNode child = (XMLNode) children.item(i);
          
          if (child.getNodeName().equals("pointer") && child.getNodeValue().equals("true")) {
            XMLNode referencedNode =
              (XMLNode) findReferencedElement(m_doc.getDocumentElement(), node.getFirstChild().getNodeValue());

            if (referencedNode == null)
              throw new Exception("Bad XML reference.\nElement with attribute id=" + node.getFirstChild().getNodeValue() + " does not exist.");

            ObjectID referencedID = (ObjectID) referencedNode.getProperty("oid");
            if (referencedID == null) {
              // to wymaga glebszego zastanowienia
              yelement = m_obman.createReference(nodeName, parentID);
              yelement.connect(parentID);
              node.setProperty("oid", yelement.getOID());

              referencedNode.setProperty("reference", yelement.getOID());
            }
            else {
              yelement = m_obman.createReference(nodeName, referencedID);
              yelement.connect(parentID);
              node.setProperty("oid", yelement.getOID());
            }
            isReference = true;

            break;
          }
        }

        if (isReference) break;

        YSpider yobject = m_obman.createSpider(nodeName);
        yobject.connect(parentID);

        node.setProperty("oid", yobject.getOID());

        ObjectID referenceID = (ObjectID) node.getProperty("reference");
        if (referenceID != null) {
          YReference ref = (YReference) m_obman.getObject(referenceID);
          ref.setValue(yobject.getOID());
        }

        if (children.getLength() > 0) {
          YSpider yattrs = m_obman.createSpider("ATTRIBUTES");
          yattrs.connect(yobject.getOID());
          
          for (int i = 0; children != null && i < children.getLength(); i++) {
            XMLNode attr = (XMLNode) children.item(i);

            YString yattr = m_obman.createString(attr.getNodeName(), attr.getNodeValue());
            yattr.connect(yattrs.getOID());
            
            attr.setProperty("oid", yattr.getOID());
          }
        }

        for (int i = 0; i < nodes.getLength(); i++) {
          processNode((XMLNode) nodes.item(i), yobject.getOID());
        }
        break;

      case Node.TEXT_NODE:
        YString ystring = m_obman.createString("TEXT", nodeValue);
        ystring.connect(parentID);

        node.setProperty("oid", ystring.getOID());

        break;
    }
  }
}
