package pl.edu.pjwstk.yaod.yaql;

import java.io.*;

import java.util.*;

import javax.swing.tree.*;

import pl.edu.pjwstk.yaod.objects.*;
import pl.edu.pjwstk.yaod.system.*;
import pl.edu.pjwstk.yaod.system.processes.*;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.*;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.*;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.statements.*;
import pl.edu.pjwstk.yaod.yaql.engine.*;
import pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes.*;
import pl.edu.pjwstk.yaod.yaql.parser.*;
import pl.edu.pjwstk.yaod.yaql.visitors.*;
import pl.edu.pjwstk.yaod.network.protocol.queryresult.*;

/**
 * The <code>Interpreter</code> class represents the main code responsible
 * for program interpretation. It consists of environmental stack,
 * query result stack, abstract syntax tree and a link to the data store.
 * @see EnvironmentalStack
 * @see QueryResultStack
 * @see AbstractSyntaxTree
 * @see SVRP
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class Interpreter {
  private EnvironmentalStack ENVS;
  private QueryResultStack QRES = new QueryResultStack();
  private AbstractSyntaxTree AST = null;
  private SVRP SP;
  // By default the results are shown nowhere.
  // change it to new PrintWriter(System.out, true) for printing it on the 
  // standard output
  private PrintWriter PW = new Interpreter.EmptyWriter();

/**
 * Creates a new interpreter with specified <code>sp</code>, which is a 
 * server process of the data store. It also initilizes properly the 
 * environmental stack.
 * @param sp a link to data store (server process).
 * @see SVRP
 * @see EnvironmentalStack
 */
  public Interpreter(SVRP sp) {
    SP = sp;
    ENVS = new EnvironmentalStack(SP.getObjectManager());
  }

/**
 * Returns the environmental stack.
 * @return the object representing the environmental stack.
 * @see EnvironmentalStack
 */
  public EnvironmentalStack getENVS() {
    return ENVS;
  }

/**
 * Returns the query result stack.
 * @return the object representing the query result stack.
 * @see QueryResultStack
 */
  public QueryResultStack getQRES() { 
    return QRES;
  }

/**
 * Returns the abstract syntax tree.
 * @return the object representing the abstract syntax tree.
 * @see AbstractSyntaxTree
 */
  public AbstractSyntaxTree getAST() {
    return AST;
  }

/**
 * Returns the server process.
 * @return the object representing the server process.
 * @see SVRP
 */
  public SVRP getSP() {
    return SP;
  }

/**
 * Returns the print writer which is an object responsible for displaying 
 * results.
 * @return the object representing the print writer.
 */
  public PrintWriter getPW() {
    return PW;
  }

/**
 * Sets the print writer which is an object responsible for displaying 
 * results.
 * @param PW the object representing the print writer.
 */
  public void setPW(PrintWriter PW) {
    this.PW = PW;
  }

/**
 * Returns the entry of the object store.
 * @see ObjectID
 * @see SVRP
 * @return entry of the object store.
 */
  public ObjectID getEntry() {
    return (ObjectID) SP.getStoreByID(0).getEntry();
  }

/**
 * The main method responsible for program interpretation. First it takes
 * the string query and converts it into stream ({@link StringReader}).
 * Then the lexing/parsing process takes the part, which yields in the 
 * abstract syntax tree form of the source program. Then the evaluation
 * process starts and at the end, the results of such evaluation, are 
 * returned as an vector object ({@link Vector}).
 * @param str a string representation of a query/program.
 * @return the vector object with the results of a query/program.
 * @exception SBQLException if interpretation (lexing/parsing/evaluating) 
 * process fails.
 */
  public Vector start(String str) throws SBQLException {
    Vector results = new Vector();
    StringReader sr = new StringReader(str);
    initializeENVS();
    try {
      AST = (AbstractSyntaxTree)new Parser(new Lexer(sr)).parse().value;
//      AST.show();
      AST.accept(new EvalVisitor(PW, QRES, ENVS, SP, results));

    } catch (SBQLException e) {
      throw e;
    } catch (Exception e) {
      throw new SBQLException(e.getMessage());
    }
/*
    } catch (LexerException e) { // LEXER ERROR
      PW.println(e.getMessage());
    } catch (ParserException e) { // PARSER ERROR
      PW.println(e.getMessage());
    } catch (EvalException e) { // EVALUATION ERROR
      PW.println(e.getMessage());
    } catch (Exception e) { // OTHER ERROR
      PW.println("!!! Unknown Interpreter Error !!!");
      e.printStackTrace();
    }
*/
//    ENVS.show();
   
    Vector tmp = new Vector();
    for (int i=0; i<results.size(); i++) {
      tmp.add(  convertToProtocolResult((QueryResult)results.elementAt(i))    );
    }
    return tmp;
  }

/**
 * The method takes the query/program result as an argument and converts it
 * into the form which is suitable for serialization (used for passing the
 * results through network).
 * @param r the possible result of the program interpretation.
 * @return the converted result.
 * @see QueryResult
 * @see Result
 */
  public Result convertToProtocolResult(QueryResult r) {
    Result res = null;
    if (r instanceof SingleValue) {
      if (r instanceof IdentifierValue) {
        res = new IdentifierResult(  ((IdentifierValue)r).getId() );
      } else if (r instanceof BinderValue) {
        res = new BinderResult( ((BinderValue)r).getName(), convertToProtocolResult(((BinderValue)r).getValue()) );
      } else if (r instanceof AtomicValue) {
        if (r instanceof BooleanValue) {
          res = new BooleanResult(  ((BooleanValue)r).getValue() );
        } else if (r instanceof IntegerValue) {
          res = new IntegerResult(  ((IntegerValue)r).getValue() );
        } else if (r instanceof DoubleValue) {
          res = new DoubleResult(  ((DoubleValue)r).getValue() );
        } else if (r instanceof StringValue) {
          res = new StringResult(  ((StringValue)r).getValue() );
        }
      }
    } else if (r instanceof ComplexValue) {
      if (r instanceof BagValue) {
        res = new BagResult();
      } else if (r instanceof StructValue) {
        res = new StructResult();
      } else if (r instanceof SequenceValue) {
        res = new SequenceResult();
      }
      for (int i=0; i<((ComplexValue)r).getSize(); i++)
        ((ComplexResult)res).addResult(  convertToProtocolResult(((ComplexValue)r).getResultAt(i))  );
    }
    return res;
  }

/**
 * The method responsible for initializing environmental stack. The initializing
 * process puts the identifiers of root objects from data store at the lowest 
 * section (activation record) of the environmental stack. The identifiers 
 * are formed into binders.
 * @see EnvironmentalStack
 * @see ActivationRecord
 * @see IdentifierValue
 * @see BinderValue
 */
  public void initializeENVS() {
    ENVS.deleteAll();      
    ActivationRecord ar = new ActivationRecord();
    try {
      ObjectID entryID = this.getEntry();
      YComplexObject entry = (YComplexObject) SP.getObjectManager().getObject(entryID);
      for (Enumeration e = entry.getSubordinates(); e.hasMoreElements(); ) {
        ObjectID mbrID = (ObjectID) e.nextElement();
        YObject mbr = SP.getObjectManager().getObject(mbrID);
        IdentifierValue iv = new IdentifierValue(mbrID, SP.getObjectManager());
        ar.addBinder(new BinderValue(mbr.getName(), iv));
      }
    } catch (NoSuchObjectException e) { e.printStackTrace(); }
    ENVS.push(ar);
  }

/**
 * Returns the structure of environmental stack suitable for displaying it
 * on graphic panel in tree form.
 * @see EnvironmentalStack
 * @return environmental stack structure suitable form displaying it on graphic
 * panel.
 */
  public DefaultMutableTreeNode dumpENVS() {
    DefaultMutableTreeNode n = new DefaultMutableTreeNode("Environmental Stack");
    dumpENVS(n);
    return n;
  }

/**
 * The <code>dumpENVS</code> is the recursive method which prepares the 
 * environmental stack structure for being displayed on graphic interface.
 * @param parentNode is the node representing the parent of the actual node,
 * in the tree hierarchy.
 */
  public void dumpENVS(DefaultMutableTreeNode parentNode) {
    for (int i=ENVS.getSize()-1; i>=0; i--) {
      ActivationRecord ar = ENVS.getElementAt(i);
      DefaultMutableTreeNode envsNode = new DefaultMutableTreeNode("Activation Record " + i + ": [" + ar.getSize() + " elements]");
      parentNode.add(envsNode);
      for (int j=0; j<ar.getSize(); j++) {
        BinderValue bv = ar.getBinderAt(j);
        if (bv.getValue() instanceof IdentifierValue)
          envsNode.add(new DefaultMutableTreeNode(bv.getName()+" ["+ ((IdentifierValue)bv.getValue()).getId().toString() +"]"));
        else
          envsNode.add(new DefaultMutableTreeNode(bv.getName()+" [not identifier]"));
      }
    }
  }

/**
 * Returns the structure of abstract syntax tree suitable for displaying it
 * on graphic panel in tree form.
 * @see AbstractSyntaxTree
 * @return abstract syntax tree structure suitable form displaying it on graphic
 * panel.
 */
  public DefaultMutableTreeNode dumpAST() {
    DefaultMutableTreeNode n = new DefaultMutableTreeNode("Abstract Syntax Tree");
    dumpAST(getAST(), n);
    return n;
  }

/**
 * The <code>dumpAST</code> is the recursive method which prepares the abstract
 * syntax tree structure for being displayed on graphic interface.
 * @param AST is the abstract syntax tree node representing the actual node.
 * @param parentNode is the node representing the parent of the actual node,
 * in the tree hierarchy.
 */
  public void dumpAST(AbstractSyntaxTree AST, DefaultMutableTreeNode parentNode) {
    if (AST == null) return;
    DefaultMutableTreeNode node = new DefaultMutableTreeNode(AST.getSpellingAndPosition());
    parentNode.add(node);
    if (AST instanceof Expression) {
      if (AST instanceof SingleExpression) {
      } else if (AST instanceof UnaryExpression) {
        dumpAST(((UnaryExpression)AST).r, node);
      } else if (AST instanceof BinaryExpression) {
        dumpAST(((BinaryExpression)AST).l, node);
        dumpAST(((BinaryExpression)AST).r, node);
      } else if (AST instanceof TernaryExpression) {
        dumpAST(((TernaryExpression)AST).l, node);
        dumpAST(((TernaryExpression)AST).m, node);
        dumpAST(((TernaryExpression)AST).r, node);
      }
    } else if (AST instanceof Statement) {
      if (AST instanceof BlockStatement) {
        dumpAST(((BlockStatement)AST).s, node);
      } else if (AST instanceof DeleteStatement) {
        dumpAST(((DeleteStatement)AST).e, node);
      } else if (AST instanceof DoWhileStatement) {
        dumpAST(((DoWhileStatement)AST).e, node);
        dumpAST(((DoWhileStatement)AST).s, node);
      } else if (AST instanceof EmptyBlockStatement) {
      } else if (AST instanceof EmptyStatement) {
      } else if (AST instanceof ExpressionStatement) {
        dumpAST(((ExpressionStatement)AST).e, node);
      } else if (AST instanceof ForEachStatement) {
        dumpAST(((ForEachStatement)AST).e, node);
        dumpAST(((ForEachStatement)AST).s, node);
      } else if (AST instanceof IfElseStatement) {
        dumpAST(((IfElseStatement)AST).e, node);
        dumpAST(((IfElseStatement)AST).s1, node);
        dumpAST(((IfElseStatement)AST).s2, node);
      } else if (AST instanceof IfStatement) {
        dumpAST(((IfStatement)AST).e, node);
        dumpAST(((IfStatement)AST).s, node);
      } else if (AST instanceof InsertIntoStatement) {
        dumpAST(((InsertIntoStatement)AST).e1, node);
        dumpAST(((InsertIntoStatement)AST).e2, node);
      } else if (AST instanceof UpdateToStatement) {
        dumpAST(((UpdateToStatement)AST).e1, node);
        dumpAST(((UpdateToStatement)AST).e2, node);
      } else if (AST instanceof PrintStatement) {
        dumpAST(((PrintStatement)AST).e, node);
      } else if (AST instanceof RenameToStatement) {
        dumpAST(((RenameToStatement)AST).e, node);
      } else if (AST instanceof ShowStatement) {
        dumpAST(((ShowStatement)AST).e, node);
      } else if (AST instanceof WhileStatement) {
        dumpAST(((WhileStatement)AST).e, node);
        dumpAST(((WhileStatement)AST).s, node);
      } else if (AST instanceof XmlPrintStatement) {
        dumpAST(((XmlPrintStatement)AST).e, node);
      } else if (AST instanceof EmptyProcedureDeclarationStatement) {
        dumpAST(((EmptyProcedureDeclarationStatement)AST).s, node);
      } else if (AST instanceof ProcedureDeclarationStatement) {
        dumpAST(((ProcedureDeclarationStatement)AST).p, node);
        dumpAST(((ProcedureDeclarationStatement)AST).s, node);
      }
    } else if (AST instanceof StatementList) {
      if (AST instanceof PairStatementList) {
        dumpAST(((PairStatementList)AST).head, node);
        dumpAST(((PairStatementList)AST).tail, node);
      } else if (AST instanceof LastStatementList) {
        dumpAST(((LastStatementList)AST).tail, node);
      }
    } else if (AST instanceof FormalParametersList) {
      if (AST instanceof PairFormalParametersList) {
        dumpAST(((PairFormalParametersList)AST).tail, node);
      } else if (AST instanceof LastFormalParametersList) {
      }
    }
  }

/**
 * Returns the structure of object store suitable for displaying it
 * on graphic interface in the tree form.
 * @return object store structure suitable form displaying it on graphic
 * panel.
 */
  public DefaultMutableTreeNode dumpDB() {
    DefaultMutableTreeNode n = new DefaultMutableTreeNode("Database");
    try {
      dumpDB(this.getEntry(), n);
    } catch (NoSuchObjectException e) { e.printStackTrace(); }
    return n;
  }

/**
 * The <code>dumpDB</code> is the recursive method which prepares the object 
 * store structure for being displayed on graphic interface.
 * @param rootID is the identifier representing the actual object.
 * @param parentNode is the node representing the parent of the actual node,
 * in the tree hierarchy.
 * @exception NoSuchObjectException if the identifier points object which doesn't
 * exist.
 */
  private void dumpDB(ObjectID rootID, DefaultMutableTreeNode parentNode) 
                                                  throws NoSuchObjectException {
    YComplexObject root = 
                  (YComplexObject) getSP().getObjectManager().getObject(rootID);
    DefaultMutableTreeNode rootNode = 
              new DefaultMutableTreeNode("<" + root.getOID().toString() + ", " + 
                                         root.getName() + ", ...>");
    parentNode.add(rootNode);    

    for (Enumeration e = root.getSubordinates(); e.hasMoreElements(); ) {
      ObjectID currID = (ObjectID) e.nextElement();
      YObject curr = (YObject) getSP().getObjectManager().getObject(currID);

      String tmp = "<" + curr.getOID().toString() + ", " + curr.getName() + ", ";
      if (curr instanceof YComplexObject)
        dumpDB(currID, rootNode);
      else if (curr instanceof YSimpleObject) {
        if (curr instanceof YString)
          rootNode.add(new DefaultMutableTreeNode(tmp + "\"" +
                                  ((YSimpleObject)curr).stringValue() + "\">"));
        else
          rootNode.add(new DefaultMutableTreeNode(tmp +
                                    ((YSimpleObject)curr).stringValue() + ">"));
      }
      else if (curr instanceof YReference)
        rootNode.add(new DefaultMutableTreeNode(tmp + 
                               ((YReference)curr).getValue().toString() + ">"));
      else
        rootNode.add(new DefaultMutableTreeNode(tmp));
    }
  }

/**
 * The <code>EmptyWriter</code> is an auxilary class responsible for displaying
 * results. Its goal is to print results nowhere. The main print writer
 * from the <code>Interpreter</code> class is set to this class by default.
 */
  class EmptyWriter extends PrintWriter {
    EmptyWriter() { super(System.out); }
    public void println(String s) { }
    public void print(String s) { }
  }
}

