package pl.edu.pjwstk.yaod.yaql.engine;

import java.io.*;

import java.util.*;

import pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes.*;

/**
 * The <code>ActivationRecord</code> class represents a section on the 
 * environmental stack. Each section consists of binders.
 * @see EnvironmentalStack
 * @see BinderValue
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version 1.0  
 */
public class ActivationRecord {
  public Vector binders = new Vector();

/**
 * Creates a new empty activation record.
 */
  public ActivationRecord() { 
  }

/**
 * Creates a new activation record with specified binder.
 * @see BinderValue
 * @param binder the binder to be put into activation record.
 */
  public ActivationRecord(BinderValue binder) {
    binders.add(binder);
  }

/**
 * Returns the i-th element from the activation record.
 * @see BinderValue
 * @param i the index of element to be returned.
 * @return the i-th element of activation record.
 */
  public BinderValue getBinderAt(int i) {
    return (BinderValue)binders.get(i);
  }

/**
 * Returns the number of binders in the activation record.
 * @return the number of binders in the activation record.
 */
  public int getSize() { 
    return binders.size();
  }

/**
 * Adds a new binder to the actual activation record.
 * @see BinderValue
 * @param binder the binder to be added.
 */
  public void addBinder(BinderValue binder) {
    binders.add(binder);
  }

/**
 * Adds a new binder (constructed from specified <code>name</code> 
 * and <code>res</code>) to the actual activation record.
 * @see BinderValue
 * @param name the name of the binder to be added.
 * @param res the value of the binder to be added.
 */
  public void addBinder(String name, QueryResult res) {
  	binders.add(new BinderValue(name, res)); 
  }

/**
 * Adds a new binders (achieved from other activation record) to the actual 
 * activation record.
 * @param ar activation record whose binders will be added to the acutal 
 * activation record.
 */
  public void addBinders(ActivationRecord ar) {
    for (int i=0; i<ar.getSize(); i++)
      binders.add(ar.getBinderAt(i));
  }

/**
 * Deletes the i-th element from the activation record.
 * @see QueryResult
 * @param i the index of binder to be deleted.
 * @return the i-th binder of activation record.
 */
  public QueryResult deleteBinderAt(int i) { 
  	return (QueryResult)binders.remove(i);
  }

/**
 * Deletes a specified binder (<code>b</code>) from activation record.
 * @see BinderValue
 * @param b binder which is to be deleted.
 * @exception TypeMismatchException if comparison of elements fails.
 */
  public void deleteBinder(BinderValue b) throws TypeMismatchException {
    for (int i=getSize()-1; i>=0; i--) { 
      BinderValue tmp = getBinderAt(i);
      if (b.operatorCOMPARE(tmp).booleanValue() == true)
        deleteBinderAt(i);
    }    
  }

/**
 * Sets a specified binder (<code>b</code>) a new name (<code>name</code>).
 * @see BinderValue
 * @param b binder which name is to be changed.
 * @param name a new name for the binder
 * @exception TypeMismatchException if comparison of elements fails.
 */
  public void renameBinder(BinderValue b, String name) 
                                                  throws TypeMismatchException {
    for (int i=getSize()-1; i>=0; i--) { 
      BinderValue tmp = getBinderAt(i);
      if (tmp.operatorCOMPARE(b).booleanValue() == true)
        tmp.setName(name);
    }    
  }

/**
 * Tests whether or not the actual activation record possess the binder with 
 * the given <code>name</code>.
 * @see BinderValue
 * @param name the name of the binders to be found.
 * @return <code>true</code> if such binder exists, <code>false</code> otherwise.
 */
  public boolean containsBinderHavingName(String name) {
    for (int i=0; i<getSize(); i++) {
      BinderValue tmp = getBinderAt(i);
      if (tmp.getName().equals(name)) return true;
    }
    return false;
  }

/**
 * Returns a set ({@link BagValue}) of binder values whose name is equal to the
 * <code>name</code> parameter.
 * @see BagValue
 * @see BinderValue
 * @param name the name of the binders to be found.
 * @return a set of binder values.
 */
  public BagValue getBinderValuesHavingName(String name) {
    BagValue res = new BagValue();
    for (int i=0; i<getSize(); i++) {
      BinderValue tmp = getBinderAt(i);
      if (tmp.getName().equals(name))
        res.addResult(tmp.getValue());
    }
    return res;
  }

/**
 * Shows all the elements of activation record.
 */
  public void show() {
    System.out.println("(Binder's = " + getSize() + "):");
    for (int i=0; i<getSize(); i++) {
      BinderValue tmp = getBinderAt(i);
      System.out.println("    binder " + i + ": ");
      tmp.show("     ", new PrintWriter(System.out, true));
    }
  }
}
