package pl.edu.pjwstk.yaod.yaql.engine;

import java.util.*;

import pl.edu.pjwstk.yaod.objects.*;
import pl.edu.pjwstk.yaod.system.*;
import pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes.*;

/**
 * The <code>EnvironmentalStack</code> class represents the environmental stack.
 * Its sections (activation records) consist of binders. The environmental stack
 * has a link to the object store.
 * @see ActivationRecord
 * @see BinderValue
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class EnvironmentalStack {
  private Vector records;
  private ObjectManager om;

/**
 * Creates a new environmental stack, with specified <code>om</code> which is 
 * an object manager (a link to data store objects).
 * @param om object manager - a link to data store objects.
 * @see ObjectManager
 */
  public EnvironmentalStack(ObjectManager om) {
  	records = new Vector();
    this.om = om;
  }

/**
 * Tests whether or not the stack is empty.
 * @return <code>true</code> if stack is empty, <code>false</code> otherwise.
 */
  public boolean empty() {
    return records.isEmpty();
  }

/**
 * Puts new element onto query result stack.
 * @see ActivationRecord
 */
  public void push(ActivationRecord ar) { 
  	records.add(ar); 
  }

/**
 * Returns (and removes) the last element from the environmental stack.
 * @see ActivationRecord
 * @exception EmptyStackException if stack is empty.
 * @return the last section of environmental stack.
 */
  public ActivationRecord pop() throws EmptyStackException {
    if (empty())
 	    throw new EmptyStackException("EnvironmentalStack.pop()");
    return (ActivationRecord)records.remove(getSize() - 1);
  }

/**
 * Returns (without removal) the last element from the environmental stack.
 * @see ActivationRecord
 * @exception EmptyStackException if stack is empty.
 * @return the last section of environmental stack.
 */
  public ActivationRecord top() throws EmptyStackException {
    if (empty())
 	    throw new EmptyStackException("EnvironmentalStack.top()");
    return (ActivationRecord)records.lastElement();
  }

/**
 * Returns the number of elements in the environmental stack.
 * @return the number of elements in the environmental stack.
 */
  public int getSize() { 
  	return records.size(); 
  }

/**
 * Returns (without removal) the i-th element from the environmental stack.
 * @see ActivationRecord
 * @param i the index of element to be returned.
 * @return the i-th element of environmental stack.
 */
  public ActivationRecord getElementAt(int i) { 
  	return (ActivationRecord)records.get(i); 
  }

/**
 * Returns (without removal) the first element from the environmental stack.
 * @see ActivationRecord
 * @return the first section of environmental stack.
 */
  public ActivationRecord getFirstElement() { 
  	return getElementAt(0); 
  }

/**
 * Returns (without removal) the last element from the environmental stack.
 * @see ActivationRecord
 * @return the last section of environmental stack.
 */
  public ActivationRecord getLastElement() { 
  	return getElementAt(getSize()-1); 
  }

/**
 * Shows all the elements of environmental stack.
 */
  public void show() {
    System.out.println("ENVS objects (ActivationRecord's = " + getSize() + "):");
    for (int i=0; i<getSize(); i++) {
      ActivationRecord tmp = getElementAt(i);
      System.out.print("  section " + i + ": ");
      tmp.show();
    }
    System.out.println("ENVS END\n");
  }

/**
 * Deletes all elements on environmental stack.
 */
  public void deleteAll() {
    records.removeAllElements();
  }

/**
 * Deletes a specified binder (<code>b</code>) from environmental stack.
 * @see BinderValue
 * @param b binder which is to be deleted.
 * @exception TypeMismatchException if comparison of elements fails.
 */
  public void deleteElement(BinderValue b) throws TypeMismatchException {
    for (int i=getSize()-1; i>=0; i--) { 
      ActivationRecord ar = getElementAt(i);
      if (ar.containsBinderHavingName(b.getName()))
        ar.deleteBinder(b);
    }
  }

/**
 * Sets a specified binder (<code>b</code>) a new name (<code>name</code>).
 * @see BinderValue
 * @param b binder which name is to be changed.
 * @param name a new name for the binder
 * @exception TypeMismatchException if comparison of elements fails.
 */
  public void renameElement(BinderValue b, String name) 
                                                  throws TypeMismatchException {
    for (int i=getSize()-1; i>=0; i--) { 
      ActivationRecord ar = getElementAt(i);
      if (ar.containsBinderHavingName(b.getName()))
        ar.renameBinder(b, name);
    }
  }

/**
 * Adds a new binder to the lowest section of the environmental stack.
 * @see BinderValue
 * @see ActivationRecord
 * @param b a binder to be added.
 */
  public void addAtBottom(BinderValue b) {
    getFirstElement().addBinder(b);
  }

/**
 * Adds a new binder to the highest section of the environmental stack.
 * @see BinderValue
 * @see ActivationRecord
 * @param b a binder to be added.
 */
  public void addAtTop(BinderValue b) {
    getLastElement().addBinder(b);
  }

/**
 * Deletes a specified binder from the lowest section of environmental stack.
 * @see BinderValue
 * @see ActivationRecord
 * @param b a binder to be deleted.
 * @exception TypeMismatchException if comparison of elements fails.
 */
  public void deleteAtBottom(BinderValue b) throws TypeMismatchException {
    for (int i=getFirstElement().getSize()-1; i>=0; i--) {
      if (getFirstElement().getBinderAt(i).operatorCOMPARE(b).booleanValue() == true)
        getFirstElement().deleteBinderAt(i);
    }
  }

/**
 * Deletes a specified binder from the highest section of environmental stack.
 * @see BinderValue
 * @see ActivationRecord
 * @param b a binder to be deleted.
 * @exception TypeMismatchException if comparison of elements fails.
 */
  public void deleteAtTop(BinderValue b) throws TypeMismatchException {
    for (int i=getLastElement().getSize()-1; i>=0; i--) {
      if (getLastElement().getBinderAt(i).operatorCOMPARE(b).booleanValue() == true)
        getLastElement().deleteBinderAt(i);
    }
  }

/**
 * Deletes all binders from the highest section of environmental stack.
 * @see BinderValue
 * @see ActivationRecord
 * @param b a binder to be deleted.
 */
  public void deleteAtTop() {
    for (int i=getLastElement().getSize()-1; i>=0; i--) {
      BinderValue bv = getLastElement().getBinderAt(i);
      if (bv.getValue() instanceof IdentifierValue) {
        try { 
          YObject obj = om.getObject(((IdentifierValue)bv.getValue()).getId());
          obj.disconnect();
        } catch (NoSuchObjectException e) { }
      }
    }
  }

/**
 * Returns a set ({@link BagValue}) of object identifiers which were positively 
 * binded. If nothing was binded, empty set is returned.
 * @see BagValue
 * @see BinderValue
 * @param name a name to be binded.
 * @return a set of object identifiers.
 */
  public BagValue bind(String name) {
    for (int i=getSize()-1; i>=0; i--) { 
      ActivationRecord ar = getElementAt(i);
      if (ar.containsBinderHavingName(name))
        return ar.getBinderValuesHavingName(name);
    }
    return new BagValue();
  }
}
