package pl.edu.pjwstk.yaod.yaql.engine;

import pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes.QueryResult;
import pl.edu.pjwstk.yaod.yaql.visitors.EvalException;

/**
 * The <code>TypeMismatchException</code> class represents an exception which is
 * thrown if an error occurs during evaluation. It possess the reason of the 
 * error in <code>message</code> value.
 * @see EvalException
 * @see QueryResult
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class TypeMismatchException extends EvalException {

/**
 * Creates a new exception with specified <code>message</code>.
 * @param message the message representing the error.
 */
  public TypeMismatchException(String message) {
    super(message);
  }

/**
 * Creates a new exception with specified <code>message</code>, based on an
 * unary operator type mismatch.
 * @param message the message representing the error.
 * @param l the operand of such operator.
 */
  public TypeMismatchException(QueryResult l, String message) {
    super(l.getTypeAsString() + "." + message + "()");
  }

/**
 * Creates a new exception with specified <code>message</code>, based on a
 * binary operator type mismatch.
 * @param message the message representing the error.
 * @param l the left operand of such operator.
 * @param r the right operand of such operator.
 */
  public TypeMismatchException(QueryResult l, String message, QueryResult r) {
    super(l.getTypeAsString() + "." + message + "(" + r.getTypeAsString() + ")");
  }

/**
 * Creates a new exception with specified <code>message</code>, based on a
 * ternary operator type mismatch.
 * @param message the message representing the error.
 * @param l the left operand of such operator.
 * @param m the medium operand of such operator.
 * @param r the right operand of such operator.
 */
  public TypeMismatchException(QueryResult l, String message, QueryResult m, 
                                                                QueryResult r) {
    super(l.getTypeAsString() + "." + message + "(" + m.getTypeAsString() + ", "
                                                   + r.getTypeAsString() + ")");
  }

/**
 * Creates a new exception with message. A message consists of 
 * @param message1 the part of a message representing the error.
 * @param message2 the part of a message representing the error.
 * @param message3 the part of a message representing the error.
 * @param l the first result.
 * @param r the second result.
 */
  public TypeMismatchException(String message1, QueryResult l, String message2, 
                                               QueryResult r, String message3) {
    super(message1 + " " + l.getTypeAsString() + " " + 
          message2 + " " + r.getTypeAsString() + " " + message3);
  }

/**
 * Returns the exception name.
 * @return the message representing the exception name.
 */
  public String getExceptionName() {
    return "TYPE MISMATCH ERROR";
  }
}