package pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes;

import java.util.*;
import pl.edu.pjwstk.yaod.yaql.engine.*;

/**
 * The <code>BagValue</code> class represents the bag collections.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class BagValue extends ComplexValue {

/**
 * Creates a new empty bag collection.
 */
	public BagValue() { 
  }

/**
 * Creates a new bag collection initialized with the <code>v</code>.
 * @param v vector of elements.
 */
	public BagValue(Vector v) {
		super(v);
	}

/**
 * Creates a new bag collection initialized with the <code>r</code>.
 * @param r any result.
 * @see QueryResult
 */
	public BagValue(QueryResult r) {
    addResult(r);
  }

/**
 * Creates a new bag collection initialized with the <code>r1</code> and
 * <code>r2</code>.
 * @param r1 any result.
 * @param r2 any result.
 * @see QueryResult
 */
	public BagValue(QueryResult r1, QueryResult r2) {
    this(r1);
    addResult(r2);
  }

/**
 * Returns a name of the result ({@link QueryResult}) type.
 * @see QueryResult
 * @return a binder type name.
 */
  public String getTypeAsString() {
    return "TYPE_COMPLEX_BAG";
  }



//******************************************************************************
//******************************* UNARY OPERATORS ******************************
//******************************************************************************
/**
 * Method responsible for unary struct operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary struct operation.
 * @see QueryResult
 */
	public QueryResult operatorSTRUCT() throws TypeMismatchException {
    throw new TypeMismatchException("Not defined: "+this.getClass().getName()+".operatorSTRUCT("+getTypeAsString()+")");
	}

/**
 * Method responsible for unary bag operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary bag operation.
 * @see QueryResult
 */
	public QueryResult operatorBAG() throws TypeMismatchException {
    return this;
	}

/**
 * Method responsible for unary sequence operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary sequence operation.
 * @see QueryResult
 */
	public QueryResult operatorSEQUENCE() throws TypeMismatchException {
    return this.operatorTOSEQUENCE();
	}

/**
 * Method responsible for unary unique operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary unique operation.
 * @see QueryResult
 */
	public QueryResult operatorUNIQUE() throws TypeMismatchException {
    BagValue res = new BagValue();
    for (int i=0; i<getSize(); i++) {
      QueryResult tmp = getResultAt(i).operatorDEREFERENCE();
      boolean found = false;
      for (int j=0; j<i; j++)
        if (tmp.operatorCOMPARE(getResultAt(j).operatorDEREFERENCE()).booleanValue() == true) {
          found = true;
          break;
        }
      if (found == false) res.addResult(tmp);
    }
    return res;
	}

/**
 * Method responsible for unary descending operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary descending operation.
 * @see QueryResult
 */
	public QueryResult operatorDESCENDING() throws TypeMismatchException {
    for (int i=0; i<getSize(); i++)
      setResultAt(i,getResultAt(i).operatorDEREFERENCE().operatorDESCENDING());
    return this;
	}


//******************************************************************************
//************************* UNARY PARAMETRIC OPERATORS *************************
//******************************************************************************
/**
 * Method responsible for unary as operation.
 * @param r the right operand of the unary as operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary as operation.
 * @see QueryResult
 * @see BinderValue
 * @see StringValue
 */
	public QueryResult operatorAS(QueryResult r) throws TypeMismatchException {
    BagValue res = new BagValue();
    for (int i=0; i<getSize(); i++)
      res.addResult(new BinderValue(((StringValue)r).getValue(), getResultAt(i)));
    return res.operatorTOSINGLE();
	}


//******************************************************************************
//****************************** BINARY OPERATORS ******************************
//******************************************************************************
/**
 * Method responsible for binary comma operation.
 * @param r the right operand of the binary comma operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary comma operation.
 * @see QueryResult
 * @see SequenceValue
 * @see SingleValue
 * @see StructValue
 */
	public QueryResult operatorCOMMA(QueryResult r) throws TypeMismatchException {
    BagValue res = new BagValue();
    if (r instanceof SingleValue) {
      for (int i=0; i<getSize(); i++)
        res.addResult((new StructValue()).addStruct(getResultAt(i)).addStruct(r));
      return res;
    } else if (r instanceof StructValue) {
      for (int i=0; i<getSize(); i++)
        res.addResult((new StructValue()).addStruct(getResultAt(i)).addStruct(r));
      return res;
    } else if (r instanceof BagValue) {
      for (int i=0; i<getSize(); i++)
        for (int j=0; j<((BagValue)r).getSize(); j++)
          res.addResult((new StructValue()).addStruct(getResultAt(i)).addStruct(((BagValue)r).getResultAt(j)));
      return res;
    } else if (r instanceof SequenceValue) {
      for (int i=0; i<getSize(); i++)
        for (int j=0; j<((SequenceValue)r).getSize(); j++)
          res.addResult((new StructValue()).addStruct(getResultAt(i)).addStruct(((SequenceValue)r).getResultAt(j)));
      return res;
    }
    throw new TypeMismatchException(this, "operatorCOMMA", r);
	}

/**
 * Method responsible for binary union operation.
 * @param r the right operand of the binary union operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary union operation.
 * @see QueryResult
 * @see SequenceValue
 * @see SingleValue
 * @see StructValue
 */
	public QueryResult operatorUNION(QueryResult r) throws TypeMismatchException {
    if (r instanceof SingleValue || r instanceof StructValue) 
      return this.addResult(r);
    else if (r instanceof BagValue)
      return this.addBag((BagValue)r);
    else if (r instanceof SequenceValue)
      return this.addBag(r.operatorTOBAG());
    else
      throw new TypeMismatchException(this, "operatorUNION", r);
	}

/**
 * Method responsible for binary inclusion operation.
 * @param r the right operand of the binary inclusion operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary inclusion operation.
 * @see QueryResult
 * @see BooleanValue
 */
	public BooleanValue operatorINCLUSION(QueryResult r) throws TypeMismatchException {
    if (r instanceof BagValue) {
      for (int i=0; i<getSize(); i++) {
        boolean found = false;
        for (int j=0; j<((BagValue)r).getSize(); j++)
          if (this.getResultAt(i).operatorCOMPARE(((BagValue)r).getResultAt(j)).getValue().booleanValue() == true) {
            found = true; break;
          }
        if (found == false) return new BooleanValue(false);
      }
      return new BooleanValue(true);
    } else
      throw new TypeMismatchException(this, "operatorINCLUSION", r);
	}

/**
 * Method responsible for binary intersection operation.
 * @param r the right operand of the binary intersection operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary intersection operation.
 * @see QueryResult
 */
	public QueryResult operatorINTERSECTION(QueryResult r) throws TypeMismatchException {
    if (r instanceof BagValue) {
      BagValue res = new BagValue();
      for (int i=0; i<getSize(); i++) {
        if ( getResultAt(i).operatorDEREFERENCE().operatorINCLUSION(r).booleanValue() == true )
          res.addResult(this.getResultAt(i));
      }
      return res.operatorUNIQUE();
    } else
      throw new TypeMismatchException(this, "operatorINTERSECTION", r);
	}

/**
 * Method responsible for binary difference operation.
 * @param r the right operand of the binary difference operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary difference operation.
 * @see QueryResult
 */
	public QueryResult operatorDIFFERENCE(QueryResult r) throws TypeMismatchException {
    if (r instanceof BagValue) {
      BagValue res = new BagValue();
      for (int i=0; i<getSize(); i++) {
        if (getResultAt(i).operatorDEREFERENCE().operatorINCLUSION(r).booleanValue() != true )
          res.addResult(this.getResultAt(i));
      }
      return res.operatorUNIQUE();
    } else
      throw new TypeMismatchException(this, "operatorDIFFERENCE", r);
	}

/**
 * Method responsible for binary symmetric difference operation.
 * @param r the right operand of the binary symmetric difference operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary symmetric difference operation.
 * @see QueryResult
 */
	public QueryResult operatorSYMMETRICDIFFERENCE(QueryResult r) throws TypeMismatchException {
    if (r instanceof BagValue)
      return this.operatorDIFFERENCE(r).operatorUNION(r.operatorDIFFERENCE(this));
    else
      throw new TypeMismatchException(this, "operatorSYMMETRICDIFFERENCE", r);
  }

/**
 * Method responsible for binary compare operation.
 * @param r the right operand of the binary compare operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary compare operation.
 * @see QueryResult
 * @see BooleanValue
 */
	public BooleanValue operatorCOMPARE(QueryResult r) throws TypeMismatchException {
    if (r instanceof BagValue) {
      if (getSize() == ((BagValue)r).getSize()) {
        for (int i=0; i<getSize(); i++)
          if (getResultAt(i).operatorDEREFERENCE().operatorCOMPARE(((BagValue)r).getResultAt(i).operatorDEREFERENCE()).booleanValue() == false)
            return new BooleanValue(false);
        return new BooleanValue(true);
      } else return new BooleanValue(false);
		} else
      return new BooleanValue(false);
	}

}