package pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes;

import java.io.*;

import pl.edu.pjwstk.yaod.objects.*;
import pl.edu.pjwstk.yaod.yaql.engine.*;

/**
 * The <code>BinderValue</code> class represents a single binder, which is 
 * a pair (name, result). Such binder is a basic element which can be hold on 
 * environmental stack in its sections (activation records).
 * @see EnvironmentalStack
 * @see ActivationRecord
 * @see QueryResult
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class BinderValue extends SingleValue {
  private String name;
  private QueryResult value; 

/**
 * Creates a new binder with specified <code>name</code> and <code>value</code>.
 * @param name the name of the binder.
 * @param value the value of the binder.
 * @see QueryResult
 */
  public BinderValue(String name, QueryResult value) {
    this.name  = name;
    this.value = value;
  }

/**
 * Returns a name of the result ({@link QueryResult}) type.
 * @see QueryResult
 * @return a binder type name.
 */
  public String getTypeAsString() {
    return "TYPE_SINGLE_BINDER";
  }

/**
 * Returns the name of the binder.
 * @return the name of the binder.
 */
  public String getName() { 
  	return name;
  }

/**
 * Sets the new name of the binder.
 * @param name the new name of the binder.
 */
  public void setName(String name) { 
    this.name = name;
  }

/**
 * Returns the value of the binder.
 * @return the value of the binder.
 */
  public QueryResult getValue() {
    return value;
  }

/**
 * Sets the new value of the binder. Additionally returns the acutal binder.
 * @param value the new value of the binder.
 * @return this binder.
 */
  public QueryResult setValue(QueryResult value) {
    this.value = value;
    return this;
  }

/**
 * The result of <code>nested</code> function, as specified in Stack Based Approach
 * by K. Subieta, is a new activation record containing this binder.
 * @return this binder.
 * @see ActivationRecord
 */
  public ActivationRecord nested() {
    ActivationRecord ar = new ActivationRecord();
    ar.addBinder(this);
    return ar;
  }  

/**
 * Returns a the result of dereference operation on binder result, which is
 * this binder itself.
 * @return this binder.
 * @exception TypeMismatchException never in this case.
 */
  public QueryResult operatorDEREFERENCE() throws TypeMismatchException {
    return this;
  }

/**
 * Return a the result of full dereference operation on binder result, which is
 * this binder containing recursively dereferenced value.
 * @return this binder.
 * @exception TypeMismatchException never in this case.
 */
  public QueryResult operatorFULLDEREFERENCE() throws TypeMismatchException {
    return this.setValue(value.operatorFULLDEREFERENCE());
  }

/**
 * Returns a the result of special dereference operation on binder result, which is
 * this binder containing recursively dereferenced value.
 * @return this binder.
 * @exception TypeMismatchException - never in this case.
 */
  public QueryResult operatorSPECIALDEREFERENCE() throws TypeMismatchException {
    return this.setValue(value.operatorSPECIALDEREFERENCE());
  }

/**
 * Returns a the result of mixed dereference operation on binder result, which is
 * this binder contained recursively dereferenced value.
 * @return this binder.
 * @exception TypeMismatchException never in this case.
 */
  public QueryResult operatorMIXEDDEREFERENCE() throws TypeMismatchException {
    return this.setValue(value.operatorMIXEDDEREFERENCE());
  }

/**
 * Returns a the result of comparing this binder with the result.
 * @return this binder.
 * @param r the object to be compare with.
 * @see BooleanValue
 * @see QueryResult
 * @exception TypeMismatchException never in this case.
 */
	public BooleanValue operatorCOMPARE(QueryResult r) throws TypeMismatchException {
    if (r instanceof BinderValue)
      if (getName().equals(((BinderValue)r).getName()))
        return getValue().operatorCOMPARE(((BinderValue)r).getValue());
      else return new BooleanValue(false);
    else
      return new BooleanValue(false);
	}

/**
 * Prints the actual binder in the raw fasion. The result is printed
 * on <code>out</code> object.
 * @exception TypeMismatchException never in this case.
 * @param st the string which is an auxiliary tabulator.
 * @param out the object responsible for displaying information.
 */
  public void show(String st, PrintWriter out) {
    out.println(" [ " + getPreparedTypeAsString() + " ]\t" + st + getName() + " (");
    value.show(st + "  ", out);
    out.println(" [ " + getLongestTypeAsSpacesString() + " ]\t" + st + ")");
  }

/**
 * Prints the actual binder in the normal fashion. The result is printed
 * on <code>out</code> object.
 * @exception TypeMismatchException never in this case.
 * @param st the string which is an auxiliary tabulator.
 * @param out the object responsible for displaying information.
 */
	public void operatorPRINT(String st, PrintWriter out) throws TypeMismatchException {
    if (value instanceof AtomicValue) {
      String tmp = (value instanceof StringValue) ? "\"" : "";
      out.print(st + getName() + "(");
      out.print( tmp + ((AtomicValue)value).getValueAsString() + tmp);
      out.println(")");
    } else if (value instanceof IdentifierValue) {
      out.print(st + getName() + "(");
      out.print( ((IdentifierValue)value).getIdAsString() );
      out.println(")");
    } else {
      out.println(st + getName() + "(");
      value.operatorPRINT(st + "  ", out);
      out.println(st + ")");
    }
	}

/**
 * Prints the actual binder in the XML fasion. The result is printed
 * on <code>out</code> object.
 * @exception TypeMismatchException never in this case.
 * @param st the string which is an auxiliary tabulator.
 * @param out the object responsible for displaying information.
 */
	public void operatorXMLPRINT(String st, PrintWriter out) throws TypeMismatchException {
    if (value instanceof AtomicValue) {
      out.print(st + "<" + getName() + ">");
      out.print( ((AtomicValue)value).getValueAsString() );
      out.println("</" + getName() + ">");
    } else if (value instanceof IdentifierValue) {
      out.print(st + "<" + getName() + ">");
      out.print( ((IdentifierValue)value).getIdAsString() );
      out.println("</" + getName() + ">");
    } else {
      out.println(st + "<" + getName() + ">");
      value.operatorXMLPRINT(st + "  ", out);
      out.println(st + "</" + getName() + ">");
    }
	}

/**
 * Creates a new object in data store, which structure corresponds to 
 * the structure this binder. Additionally it returns the identifier of a newly 
 * created object. It works the same as {@link operatorCREATELOCAL}.
 * @return identifier of the created object.
 * @exception TypeMismatchException if the structure of binder is not valid.
 * @param id Identifier of the parent object in object store.
 */
	public QueryResult operatorCREATE(IdentifierValue id) throws TypeMismatchException {
    return this.operatorCREATELOCAL(id);
  }

/**
 * Creates a new local object in data store, which structure corresponds to 
 * the structure this binder. Additionally it returns the identifier of a newly 
 * created object.
 * @return identifier of the created object.
 * @exception TypeMismatchException if the structure of binder is not valid.
 * @param id Identifier of the parent object in object store.
 */
	public QueryResult operatorCREATELOCAL(IdentifierValue id) throws TypeMismatchException {
    return this.operatorCREATEPERMANENT(id);
  }

/**
 * Creates a new permanent object in data store, which structure corresponds to 
 * the structure this binder. Additionally it returns the identifier of a newly 
 * created object.
 * @return identifier of the created object.
 * @exception TypeMismatchException if the structure of binder is not valid.
 * @param id Identifier of the parent object in object store.
 */
	public QueryResult operatorCREATEPERMANENT(IdentifierValue id) throws TypeMismatchException {
    ObjectManager om = id.getObjectManager();
    ObjectID parent = id.getId();
   try {
    if (value instanceof AtomicValue) {             // AtomObject
      if (value instanceof IntegerValue) {
        YInteger yi = om.createInteger(getName(), ((IntegerValue)value).intValue());
        yi.connect(id.getId());
        return new IdentifierValue(yi.getOID(), om);
      } else if (value instanceof DoubleValue) {
        YDouble yd = om.createDouble(getName(), ((DoubleValue)value).doubleValue());
        yd.connect(id.getId());
        return new IdentifierValue(yd.getOID(), om);
      } else if (value instanceof StringValue) {
        YString ys = om.createString(getName(), ((StringValue)value).stringValue());
        ys.connect(id.getId());
        return new IdentifierValue(ys.getOID(), om);
      } else
        throw new TypeMismatchException(this, "[wrong binder value]operatorCREATEPERMANENT");
    } else if (value instanceof IdentifierValue) {  // PointerObject
      YReference yr = om.createReference(getName(), ((IdentifierValue)value).getId());
      yr.connect(id.getId());
      return new IdentifierValue(yr.getOID(), om);
    } else if (value instanceof BinderValue) {      // ComplexObject
      YSpider ysp = om.createSpider(getName());
      ysp.connect(id.getId());
      IdentifierValue iv = new IdentifierValue(ysp.getOID(), om);
      ((BinderValue)value).operatorCREATEPERMANENT(iv);
      return iv;
    } else if (value instanceof ComplexValue) {     // ComplexObject
      YSpider yc = om.createSpider(getName());
      yc.connect(id.getId());
      IdentifierValue iyc = new IdentifierValue(yc.getOID(), om);
      for (int i=0; i<((ComplexValue)value).getSize(); i++) {
        QueryResult tmp = ((ComplexValue)value).getResultAt(i);
        tmp.operatorCREATEPERMANENT(iyc);
      }
      return iyc;
    }
//   } catch (NoSuchObjectException e) { e.printStackTrace(); 
//   } catch (BlockFreeSpaceException e) { e.printStackTrace(); }
   } catch (Exception e) { e.printStackTrace(); }
    throw new TypeMismatchException(this, "operatorCREATEPERMANENT -> binder", value);
  }
}
