package pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes;

import pl.edu.pjwstk.yaod.yaql.engine.*;

/**
 * The <code>BooleanValue</code> class represents the boolean values.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class BooleanValue extends AtomicValue {
  Boolean value;

/**
 * Creates a new boolean value initialized with specified <code>value</code>.
 * @param value the initializing value.
 */
	public BooleanValue(Boolean value) { 
    this.value = value;
  }

/**
 * Creates a new boolean value initialized with specified <code>value</code>.
 * @param value the initializing value.
 */
	public BooleanValue(boolean value){
    this(new Boolean(value));
	}

/**
 * Returns a name of the result ({@link QueryResult}) type.
 * @see QueryResult
 * @return a boolean result type name.
 */
  public String getTypeAsString() {
    return "TYPE_SINGLE_ATOM_BOOLEAN";
  }

/**
 * Returns the boolean value as a string value.
 * @return the result type as string.
 */
  public String getValueAsString() {
    return ((Boolean)value).toString();
  }

/**
 * Returns the value of a boolean value.
 * @return the value of a boolean value.
 */
  public Boolean getValue() {
    return (Boolean) value;
  }

/**
 * Returns the boolean value as a boolean value.
 * @return the boolean value as a boolean value.
 */
  public boolean booleanValue() {
    return getValue().booleanValue();
  }

/**
 * Returns the boolean value as a string value.
 * @return the boolean value as a string value.
 */
  public String stringValue() {
    return getValue().toString();
  }


//******************************************************************************
//******************************* UNARY OPERATORS ******************************
//******************************************************************************
/**
 * Method responsible for unary descending operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary descending operation.
 * @see QueryResult
 */
	public QueryResult operatorDESCENDING() throws TypeMismatchException {
    return this.operatorNEGATION();
	}

/**
 * Method responsible for unary tostring operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary tostring operation.
 * @see StringValue
 */
	public StringValue operatorTOSTRING() throws TypeMismatchException {
    return new StringValue(stringValue());
  }

/**
 * Method responsible for unary negation operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary negation operation.
 * @see BooleanValue
 */
	public BooleanValue operatorNEGATION() throws TypeMismatchException {
		return new BooleanValue(!booleanValue()); 
	}


//******************************************************************************
//******************************* BINARY OPERATORS *****************************
//******************************************************************************
/**
 * Method responsible for binary plus operation.
 * @param r the right operand of the binary plus operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary plus operation.
 * @see QueryResult
 * @see StringValue
 */
 	public QueryResult operatorPLUS(QueryResult r) throws TypeMismatchException {
    if (r instanceof StringValue)
      return new StringValue(stringValue() + ((StringValue)r).stringValue());
    else
      throw new TypeMismatchException(this, "operatorPLUS", r);
	}

/**
 * Method responsible for binary equal operation.
 * @param r the right operand of the binary equal operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary equal operation.
 * @see QueryResult
 */
  public BooleanValue operatorEQUAL(QueryResult r) throws TypeMismatchException {
    if (r instanceof BooleanValue)
      return new BooleanValue(booleanValue() == ((BooleanValue)r).booleanValue());
    else
      throw new TypeMismatchException(this, "operatorEQUAL", r);
  }	

/**
 * Method responsible for binary not equal operation.
 * @param r the right operand of the binary not equal operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary not equal operation.
 * @see QueryResult
 */
  public BooleanValue operatorNOTEQUAL(QueryResult r) throws TypeMismatchException {
    if (r instanceof BooleanValue)
      return new BooleanValue(booleanValue() != ((BooleanValue)r).booleanValue());
    else
      throw new TypeMismatchException(this, "operatorNOTEQUAL", r);
  }

/**
 * Method responsible for binary disjunction operation.
 * @param r the right operand of the binary disjunction operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary disjunction operation.
 * @see QueryResult
 */
	public BooleanValue operatorDISJUNCTION(QueryResult r) throws TypeMismatchException {
    if (r instanceof BooleanValue)
			return new BooleanValue(booleanValue() || ((BooleanValue)r).booleanValue()); 
		else
      throw new TypeMismatchException(this, "operatorDISJUNCTION", r);
	}

/**
 * Method responsible for binary conjunction operation.
 * @param r the right operand of the binary conjunction operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary conjunction operation.
 * @see QueryResult
 */
	public BooleanValue operatorCONJUNCTION(QueryResult r) throws TypeMismatchException {
    if (r instanceof BooleanValue)
			return new BooleanValue(booleanValue() && ((BooleanValue)r).booleanValue());
    else
      throw new TypeMismatchException(this, "operatorCONJUNCTION", r);
  }

/**
 * Method responsible for binary compare operation.
 * @param r the right operand of the binary compare operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary compare operation.
 * @see QueryResult
 */
	public BooleanValue operatorCOMPARE(QueryResult r) throws TypeMismatchException {
    if (r instanceof BooleanValue)
      return new BooleanValue(booleanValue() == ((BooleanValue)r).booleanValue());
    else
      return new BooleanValue(false);
	}


//******************************************************************************
//****************************** TERNARY OPERATORS *****************************
//******************************************************************************
/**
 * Method responsible for ternary condition operation.
 * @param m the middle operand of the ternary condition operator.
 * @param r the right operand of the ternary condition operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the ternary condition operation.
 * @see QueryResult
 */
	public QueryResult operatorCONDITION(QueryResult l, QueryResult r) throws TypeMismatchException {
    if (this.operatorEQUAL(new BooleanValue(true)).booleanValue() == true) 
      return l;
    else return r;
	}

}