package pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes;

import java.io.*;
import java.util.*;
import pl.edu.pjwstk.yaod.yaql.engine.*;

/**
 * The <code>ComplexValue</code> class represents the main class of available
 * complex results - collections (structures, bags and sequences).
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public abstract class ComplexValue extends QueryResult {
  public Vector results;

/**
 * Creates a new empty collection.
 */
  public ComplexValue() { 
    results = new Vector();
  }

/**
 * Creates a new collection initialized with the <code>v</code>.
 * @param v vector of elements.
 */
  public ComplexValue(Vector v) {
    results = v;
  }

/**
 * Returns the i-th element from the collection.
 * @see QueryResult
 * @param i the index of element to be returned.
 * @return the i-th element of collection.
 */
  public QueryResult getResultAt(int i) {
  	return (QueryResult)results.elementAt(i);
  }

/**
 * Sets the i-th element value to <code>r</code> in the collection.
 * @see QueryResult
 * @param i the index of element to be updated.
 */
  public void setResultAt(int i, QueryResult r) { 
    results.setElementAt(r, i);
  }

/**
 * Returns the number of elements in the collection.
 * @return the number of elements in the collection.
 */
  public int getSize() {
  	return results.size();
  }

/**
 * Adds a new element to the collection, additionally returns the collection.
 * @see QueryResult
 * @param result an element to be added.
 * @return the actual collection.
 */
	public QueryResult addResult(QueryResult result) {
		results.add(result);
    return this;
	}

/**
 * Tests whether or not the collection is empty.
 * @result <code>true</code> if the collection is empty, <code>false</code>
 * otherwise.
 */
  public boolean isEmpty() {
    return results.isEmpty();
  }

/**
 * Returns the first element in the collection.
 * @see QueryResult
 */
	public QueryResult getFirstResult() {
		return (QueryResult)(results.firstElement());
	}

/**
 * Returns the elements of the collection.
 * @return the elements of the collection.
 */
	public Vector getResults() {
	  return results;
	}

/**
 * Method used during evaluation of non algebraic operators. 
 * It returns the size of all collections, except structure ({@link StructValue}).
 * @return the number of elements in the collection.
 */
  public int getNumberOfResults() {
    return getSize();
  }

/**
 * Method used during evaluation of non algebraic operators. 
 * It returns the element <code>i</code> of the collection, except structure
 * ({@link StructValue}).
 * @see QueryResult
 * @param i the i-th element to be returned.
 * @return the i-th element in the collection.
 */
  public QueryResult getNextResultAt(int i) {
    return getResultAt(i);
  }

/**
 * Adds a new element to the collection. If this element is a bag,
 * all its elements are added to the actual collection, otherwise the 
 * entire result is added as an element. Additionally returns the actual 
 * collection.
 * @see QueryResult
 * @see ComplexValue
 * @see BagValue
 * @param bag an element to be added.
 * @return the actual collection.
 */
  public ComplexValue addBag(QueryResult bag) {
    if (bag instanceof BagValue)
      for (int i=0; i<((BagValue)bag).getSize(); i++)
        addResult(((BagValue)bag).getResultAt(i));
    else addResult(bag);
    return this;
  }

/**
 * Adds a new element to the collection. If this element is a sequence,
 * all its elements are added to the actual collection, otherwise the 
 * entire result is added as an element. Additionally returns the actual 
 * collection.
 * @see QueryResult
 * @see ComplexValue
 * @see SequenceValue
 * @param seq an element to be added.
 * @return the actual collection.
 */
  public ComplexValue addSequence(QueryResult seq) {
    if (seq instanceof SequenceValue)
      for (int i=0; i<((SequenceValue)seq).getSize(); i++)
        addResult(((SequenceValue)seq).getResultAt(i));
    else addResult(seq);
    return this;
  }

/**
 * Adds a new element to the collection. If this element is a structure,
 * all its elements are added to the actual collection, otherwise the 
 * entire result is added as an element. Additionally returns the actual 
 * collection.
 * @see QueryResult
 * @see ComplexValue
 * @see StructValue
 * @param str an element to be added.
 * @return the actual collection.
 */
  public ComplexValue addStruct(QueryResult str) {
    if (str instanceof StructValue)
      for (int i=0; i<((StructValue)str).getSize(); i++)
        addResult(((StructValue)str).getResultAt(i));
    else addResult(str);
    return this;
  }



//******************************************************************************
//******************************* UNARY OPERATORS ******************************
//******************************************************************************
/**
 * Returns the result of dereference operation on complex result, which is
 * this complex result itself.
 * @return this complex value.
 * @exception TypeMismatchException never in this case.
 * @see QueryResult
 */
  public QueryResult operatorDEREFERENCE() throws TypeMismatchException {
    return this;
  }

/**
 * Returns the result of full dereference operation on complex result, which is
 * this complex result with recursively fully dereferenced elements.
 * @return this complex value.
 * @exception TypeMismatchException never in this case.
 * @see QueryResult
 */
  public QueryResult operatorFULLDEREFERENCE() throws TypeMismatchException {
    for (int i=0; i<getSize(); i++)
      this.setResultAt(i, getResultAt(i).operatorFULLDEREFERENCE());
    return this;
  }

/**
 * Returns the result of special dereference operation on complex result, which is
 * this complex result with recursively specially dereferenced elements.
 * @return this complex value.
 * @exception TypeMismatchException never in this case.
 * @see QueryResult
 */
  public QueryResult operatorSPECIALDEREFERENCE() throws TypeMismatchException {
    for (int i=0; i<getSize(); i++)
      this.setResultAt(i, getResultAt(i).operatorSPECIALDEREFERENCE());
    return this;
  }

/**
 * Returns the result of mixed dereference operation on complex result, which is
 * this complex result with recursively mixed dereferenced elements.
 * @return this complex value.
 * @exception TypeMismatchException never in this case.
 * @see QueryResult
 */
  public QueryResult operatorMIXEDDEREFERENCE() throws TypeMismatchException {
    for (int i=0; i<getSize(); i++)
      this.setResultAt(i, getResultAt(i).operatorMIXEDDEREFERENCE());
    return this;
  }

/**
 * Prints the actual atomic result in the raw fashion. The result is printed
 * on <code>out</code> object.
 * @exception TypeMismatchException never in this case.
 * @param st the string which is a auxiliary tabulator.
 * @param out the object responsible for displaying information.
 */
  public void show(String st, PrintWriter out) {
    out.println(" [ " + getPreparedTypeAsString() + " ]\t" + st + "{");
    for (int i=0; i<getSize(); i++)
      getResultAt(i).show(st + "  ", out);
    out.println(" [ " + getLongestTypeAsSpacesString() + " ]\t" + st + "}");
  }

/**
 * Prints the actual atomic result in the normal fashion. The result is printed
 * on <code>out</code> object.
 * @exception TypeMismatchException never in this case.
 * @param st the string which is a auxiliary tabulator.
 * @param out the object responsible for displaying information.
 */
	public void operatorPRINT(String st, PrintWriter out) throws TypeMismatchException {
    for (int i=0; i<getSize(); i++) {
      QueryResult tmp = getResultAt(i);
      tmp.operatorPRINT(st, out);
      if (tmp instanceof StructValue) out.println("");
    }
  }

/**
 * Prints the actual atomic result in the xml fashion. The result is printed
 * on <code>out</code> object.
 * @exception TypeMismatchException never in this case.
 * @param st the string which is a auxiliary tabulator.
 * @param out the object responsible for displaying information.
 */
	public void operatorXMLPRINT(String st, PrintWriter out) throws TypeMismatchException {
    for (int i=0; i<getSize(); i++) {
      QueryResult tmp = getResultAt(i);
      tmp.operatorXMLPRINT(st, out);
      if (tmp instanceof StructValue) out.println("");
    }
	}

/**
 * Method responsible for unary count operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary count operation.
 * @see IntegerValue
 */
	public IntegerValue operatorCOUNT() throws TypeMismatchException {
    return new IntegerValue(getSize());
	}

/**
 * Method responsible for unary exists operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary exists operation.
 * @see BooleanValue
 */
	public BooleanValue operatorEXISTS() throws TypeMismatchException {
    return new BooleanValue((getSize() == 0) ? false : true);
	}

/**
 * Method responsible for unary sum operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary sum operation.
 * @see QueryResult
 */
	public QueryResult operatorSUM() throws TypeMismatchException {
    QueryResult res = getResultAt(0).operatorDEREFERENCE();
    for (int i=1; i<getSize(); i++)
      res = res.operatorPLUS(getResultAt(i).operatorDEREFERENCE());
    return res;
	}

/**
 * Method responsible for unary average operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary average operation.
 * @see QueryResult
 */
	public QueryResult operatorAVERAGE() throws TypeMismatchException {
    return this.operatorSUM().operatorDIVIDE(this.operatorCOUNT());
	}

/**
 * Method responsible for unary minimum operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary minimum operation.
 * @see QueryResult
 */
	public QueryResult operatorMINIMUM() throws TypeMismatchException {
    QueryResult res = getResultAt(0).operatorDEREFERENCE();
    for (int i=1; i<getSize(); i++) {
      QueryResult tmp = getResultAt(i).operatorDEREFERENCE();
      if (res.operatorGREATER(tmp).booleanValue() == true)
        res = tmp;
    }
    return res;
	}

/**
 * Method responsible for unary maximum operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary maximum operation.
 * @see QueryResult
 */
	public QueryResult operatorMAXIMUM() throws TypeMismatchException {
    QueryResult res = getResultAt(0).operatorDEREFERENCE();
    for (int i=1; i<getSize(); i++) {
      QueryResult tmp = getResultAt(i).operatorDEREFERENCE();
      if (res.operatorLOWER(tmp).booleanValue() == true)
        res = tmp;
    }
    return res;
	}

/**
 * Method responsible for unary tosingle operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary tosingle operation.
 * @see QueryResult
 */
	public QueryResult operatorTOSINGLE() throws TypeMismatchException {
		return (getSize() == 1) ? getFirstResult() : this;
  }

/**
 * Method responsible for unary tostruct operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary tostruct operation.
 * @see StructValue
 */
	public StructValue operatorTOSTRUCT() throws TypeMismatchException {
    return new StructValue(getResults());
  }

/**
 * Method responsible for unary tobag operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary tobag operation.
 * @see BagValue
 */
	public BagValue operatorTOBAG() throws TypeMismatchException {
    return new BagValue(getResults());
  }

/**
 * Method responsible for unary tosequence operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary tosequence operation.
 * @see SequenceValue
 */
	public SequenceValue operatorTOSEQUENCE() throws TypeMismatchException {
    return new SequenceValue(getResults());
  }

/**
 * Method responsible for unary ascending operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary ascending operation.
 * @see QueryResult
 */
	public QueryResult operatorASCENDING() throws TypeMismatchException {
    return this;
	}

/**
 * Method responsible for unary delete operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @param ENVS environmental stack which has to be updated.
 * @see EnvironmentalStack
 */
	public void operatorDELETE(EnvironmentalStack ENVS) throws TypeMismatchException {
    for (int i=0; i<this.getSize(); i++)
      this.getResultAt(i).operatorDELETE(ENVS);
  }

/**
 * Method responsible for unary create operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @param id Identifier of the parent object in object store.
 * @see IdentifierValue
 * @see BagValue
 * @see QueryResult
 */
	public QueryResult operatorCREATE(IdentifierValue iv) throws TypeMismatchException {
    BagValue bag = new BagValue();
    for (int i=0; i<this.getSize(); i++)
      bag.addResult(getResultAt(i).operatorCREATE(iv)); 
    return bag;
  }

/**
 * Method responsible for unary create operation on environmental stack.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @param ENVS environmental stack which has to be updated.
 * @see EnvironmentalStack
 */
	public void operatorCREATE(EnvironmentalStack ENVS) throws TypeMismatchException {
    for (int i=0; i<this.getSize(); i++)
      getResultAt(i).operatorCREATE(ENVS);
  }

/**
 * Method responsible for unary create local operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @param id Identifier of the parent object in object store.
 * @see IdentifierValue
 * @see BagValue
 * @see QueryResult
 */
	public QueryResult operatorCREATELOCAL(IdentifierValue iv) throws TypeMismatchException {
    BagValue bag = new BagValue();
    for (int i=0; i<this.getSize(); i++)
      bag.addResult(getResultAt(i).operatorCREATELOCAL(iv)); 
    return bag;
  }

/**
 * Method responsible for unary create local operation on environmental stack.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @param ENVS environmental stack which has to be updated.
 * @see EnvironmentalStack
 */
	public void operatorCREATELOCAL(EnvironmentalStack ENVS) throws TypeMismatchException {
    for (int i=0; i<this.getSize(); i++)
      getResultAt(i).operatorCREATELOCAL(ENVS);
  }

/**
 * Method responsible for unary create permanent operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @param id Identifier of the parent object in object store.
 * @see IdentifierValue
 * @see BagValue
 * @see QueryResult
 */
	public QueryResult operatorCREATEPERMANENT(IdentifierValue iv) throws TypeMismatchException {
    BagValue bag = new BagValue();
    for (int i=0; i<this.getSize(); i++)
      bag.addResult(getResultAt(i).operatorCREATEPERMANENT(iv)); 
    return bag;
  }

/**
 * Method responsible for unary create permanent operation on environmental stack.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @param ENVS environmental stack which has to be updated.
 * @see EnvironmentalStack
 */
	public void operatorCREATEPERMANENT(EnvironmentalStack ENVS) throws TypeMismatchException {
    for (int i=0; i<this.getSize(); i++)
      getResultAt(i).operatorCREATEPERMANENT(ENVS);
  }


//******************************************************************************
//************************* UNARY PARAMETRIC OPERATORS *************************
//******************************************************************************
/**
 * Method responsible for unary group as operation.
 * @param r the right operand of the unary as operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary as operation.
 * @see QueryResult
 * @see BinderValue
 * @see StringValue
 */
	public BinderValue operatorGROUPAS(QueryResult r) throws TypeMismatchException {
    return new BinderValue(((StringValue)r).getValue(), this);
	}


//******************************************************************************
//******************************* BINARY OPERATORS *****************************
//******************************************************************************
/**
 * Method responsible for binary rename to operation.
 * @param r the right operand of the binary rename to operator.
 * @param ENVS environmental stack which has to be updated.
 * @exception TypeMismatchException if type mismatch occurs.
 * @see QueryResult
 * @see EnvironmentalStack
 */
	public void operatorRENAMETO(QueryResult r, EnvironmentalStack ENVS) throws TypeMismatchException {
    for (int i=0; i<this.getSize(); i++)
      this.getResultAt(i).operatorRENAMETO(r, ENVS);
  }

/**
 * Method responsible for binary insert into operation.
 * @param r the right operand of the binary insert into operator.
 * @param ENVS environmental stack which has to be updated.
 * @exception TypeMismatchException if type mismatch occurs.
 * @see QueryResult
 * @see EnvironmentalStack
 */
	public void operatorINSERTINTO(QueryResult r, EnvironmentalStack ENVS) throws TypeMismatchException {
    for (int i=0; i<this.getSize(); i++)
      this.getResultAt(i).operatorINSERTINTO(r, ENVS);
  }

/**
 * Method responsible for binary update to operation.
 * @param r the right operand of the binary update to operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @see QueryResult
 */
	public void operatorUPDATETO(QueryResult r) throws TypeMismatchException {
    for (int i=0; i<this.getSize(); i++)
      this.getResultAt(i).operatorUPDATETO(r);
  }
}
