package pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes;

import java.io.*;
import java.util.*;
import pl.edu.pjwstk.yaod.objects.*;
import pl.edu.pjwstk.yaod.system.*;
import pl.edu.pjwstk.yaod.yaql.engine.*;

/**
 * The <code>IdentifierValue</code> class represents the object identifier
 * from the object store. Additionally each identifier possess the link
 * to the object store.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class IdentifierValue extends SingleValue {
  private ObjectID id;
  private ObjectManager om;

/**
 * Creates a new identifier initialized with specified <code>id</code>.
 * @param id the initializing identifier.
 * @param om the object manager of the object store.
 * @see ObjectID
 * @see ObjectManager
 */
  public IdentifierValue(ObjectID id, ObjectManager om) {
    this.id = id;
    this.om = om;
  }

/**
 * Returns a name of the result ({@link QueryResult}) type.
 * @see QueryResult
 * @return a binder type name.
 */
  public String getTypeAsString() {
    return "TYPE_SINGLE_IDENTIFIER";
  }

/**
 * Returns the value of a identifier.
 * @return the value of a identifier.
 * @see ObjectID
 */
  public ObjectID getId() {
    return id;
  }

/**
 * Returns the identifier value as a string value.
 * @return the identifier value as a string value.
 */
  public String getIdAsString() {
    return id.toString();
  }

/**
 * Returns the object manager of the object store.
 * @see ObjectManager
 * @see SVRP
 * @return object manager of the object store.
 */
  public ObjectManager getObjectManager() {
    return om;
  }

/**
 * Returns the value(s) of the object from the object store.
 * @return the value(s) of the object from the object store.
 * @see ActivationRecord
 */
  public ActivationRecord nested() {
    ActivationRecord ar = new ActivationRecord();
    try {
      YObject mbr = om.getObject(getId());
      if (mbr instanceof YReference) {
        ObjectID tmpid = ((YReference) mbr).getValue();
        YObject tmpobj = om.getObject(tmpid);
        ar.addBinder(tmpobj.getName(), new IdentifierValue(tmpid, om));
      } else if (mbr instanceof YSimpleObject) {
        // nothing 
      } else if (mbr instanceof YComplexObject) {
        for (Enumeration e=((YComplexObject) mbr).getSubordinates(); e.hasMoreElements(); ) {
          ObjectID tmpid = (ObjectID) e.nextElement();
          YObject tmpobj = om.getObject(tmpid);
          ar.addBinder(tmpobj.getName(), new IdentifierValue(tmpid, om));
        }
        ar.addBinder("this", this); // experiment !!! :)
      }
    } catch (NoSuchObjectException e) { e.printStackTrace(); }
    return ar;
  }



//******************************************************************************
//******************************* UNARY OPERATORS ******************************
//******************************************************************************
/**
 * Method responsible for unary dereference operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary dereference operation.
 * @see QueryResult
 * @see YObject
 */
  public QueryResult operatorDEREFERENCE() throws TypeMismatchException {
    try {
      YObject mbr = om.getObject(getId());
      if (mbr instanceof YSimpleObject) {
        if (mbr instanceof YInteger) {
          return new IntegerValue(((YInteger)mbr).getValue()); 
        } else if (mbr instanceof YDouble) {
          return new DoubleValue(((YDouble)mbr).getValue()); 
        } else if (mbr instanceof YString) {
          return new StringValue(((YString)mbr).getValue());           
        }
      } else if (mbr instanceof YReference) {
        return new IdentifierValue(((YReference)mbr).getValue(), om); 
      } else if (mbr instanceof YComplexObject) { 
        StructValue tmp = new StructValue();
        for (Enumeration e = ((YComplexObject)mbr).getSubordinates(); e.hasMoreElements(); ) {
          ObjectID elID = (ObjectID) e.nextElement();
          YObject el = om.getObject(elID);
          IdentifierValue iv = new IdentifierValue(elID, om);
          ((StructValue)tmp).addResult(new BinderValue(el.getName(), iv.operatorDEREFERENCE()));
        }
        return tmp;
      }
    } catch (NoSuchObjectException e) { e.printStackTrace(); }
    throw new TypeMismatchException(this, "[unknown Object]operatorDEREFERENCE");        
  }

/**
 * Method responsible for unary full dereference operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary full dereference operation.
 * @see QueryResult
 */
  public QueryResult operatorFULLDEREFERENCE() throws TypeMismatchException {
    return this.operatorDEREFERENCE();
  }

/**
 * Method responsible for unary special dereference operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary special dereference operation.
 * @see QueryResult
 * @see BinderValue
 * @see StructValue
 * @see YObject
 */
  public QueryResult operatorSPECIALDEREFERENCE() throws TypeMismatchException {
    try {
      YObject mbr = om.getObject(getId());
      if (mbr instanceof YReference) {
        return new BinderValue(mbr.getName(), this.operatorDEREFERENCE());
      } else if (mbr instanceof YSimpleObject) {
        return new BinderValue(mbr.getName(), this.operatorDEREFERENCE());
      } else if (mbr instanceof YComplexObject) {
        StructValue str = new StructValue();
        BinderValue tmp = new BinderValue(mbr.getName(), str);
        for (Enumeration e = ((YComplexObject)mbr).getSubordinates(); e.hasMoreElements(); ) {
          ObjectID elID = (ObjectID) e.nextElement();
          IdentifierValue iv = new IdentifierValue(elID, om);
          str.addResult(iv.operatorSPECIALDEREFERENCE());
        }
        return tmp;
      }
    } catch (NoSuchObjectException e) { e.printStackTrace(); }
    throw new TypeMismatchException(this, "[unknown Object]operatorSPECIALDEREFERENCE");
  }

/**
 * Method responsible for unary mixed dereference operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary mixed dereference operation.
 * @see QueryResult
 * @see YObject
 */
  public QueryResult operatorMIXEDDEREFERENCE() throws TypeMismatchException {
    try {
      YObject mbr = om.getObject(getId());
      if (mbr instanceof YReference) {
        return this.operatorDEREFERENCE();
      } else if (mbr instanceof YSimpleObject) {
        return this.operatorDEREFERENCE();
      } else if (mbr instanceof YComplexObject) {
        return this.operatorSPECIALDEREFERENCE();
      }
    } catch (NoSuchObjectException e) { e.printStackTrace(); }
    throw new TypeMismatchException(this, "[unknown Object]operatorMIXEDDEREFERENCE");
  }

/**
 * Method responsible for unary preincrement operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary preincrement operation.
 * @see QueryResult
 * @see IntegerValue
 */
	public QueryResult operatorPREINCREMENT() throws TypeMismatchException {
    return this.operatorPLUSASSIGN(new IntegerValue(1));
	}

/**
 * Method responsible for unary predecrement operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary predecrement operation.
 * @see QueryResult
 * @see IntegerValue
 */
	public QueryResult operatorPREDECREMENT() throws TypeMismatchException {
    return this.operatorMINUSASSIGN(new IntegerValue(1));
	}

/**
 * Method responsible for unary delete operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary delete operation.
 * @see EnvironmentalStack
 * @see BinderValue
 * @see YObject
 */
	public void operatorDELETE(EnvironmentalStack ENVS) throws TypeMismatchException {
    try {
      YObject obj = om.getObject(getId());
      ENVS.deleteElement(new BinderValue(obj.getName(),  this));
      obj.disconnect();
    } catch (NoSuchObjectException e) {
      // everything is all right
      // deleting non existing object is acceptable
    }
	}

/**
 * Method responsible for unary craete permanent operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary create permanent operation.
 * @see EnvironmentalStack
 * @see BinderValue
 * @see YObject
 */
	public void operatorCREATEPERMANENT(EnvironmentalStack ENVS) throws TypeMismatchException {
    try {
      YObject obj = om.getObject(this.getId());
      ENVS.addAtBottom(new BinderValue(obj.getName(), this));
    } catch (NoSuchObjectException e) { e.printStackTrace(); }
  }

/**
 * Method responsible for unary create local operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary create local operation.
 * @see EnvironmentalStack
 * @see BinderValue
 * @see YObject
 */
	public void operatorCREATELOCAL(EnvironmentalStack ENVS) throws TypeMismatchException {
    try {
      YObject obj = om.getObject(this.getId());
      ENVS.addAtTop(new BinderValue(obj.getName(), this));
    } catch (NoSuchObjectException e) { e.printStackTrace(); }
  }

/**
 * Method responsible for unary create operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary create operation.
 * @see EnvironmentalStack
 */
	public void operatorCREATE(EnvironmentalStack ENVS) throws TypeMismatchException {
    this.operatorCREATELOCAL(ENVS);
  }

/**
 * Method responsible for unary show operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary show operation.
 */
  public void show(String st, PrintWriter out) {
    out.print(" [ " + getPreparedTypeAsString() + " ]\t");
    out.println(st + getIdAsString());
  }

/**
 * Method responsible for unary print operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary print operation.
 */
	public void operatorPRINT(String st, PrintWriter out) throws TypeMismatchException {
    out.println(st + getIdAsString());
	}

/**
 * Method responsible for unary xmlprint operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary xmlprint operation.
 */
	public void operatorXMLPRINT(String st, PrintWriter out) throws TypeMismatchException {
    out.println(st + getIdAsString());
	}

//******************************************************************************
//****************************** BINARY OPERATORS ******************************
//******************************************************************************
/**
 * Method responsible for binary compare operation.
 * @param r the right operand of the binary compare operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary compare operation.
 * @see QueryResult
 * @see BooleanValue
 */
	public BooleanValue operatorCOMPARE(QueryResult r) throws TypeMismatchException {
    if (r instanceof IdentifierValue) {
      if (getId().equals(((IdentifierValue)r).getId()))
        return new BooleanValue(true);
    }
    return new BooleanValue(false);
	}

/**
 * Method responsible for binary update to operation.
 * @param r the right operand of the binary update to operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @see QueryResult
 */
	public void operatorUPDATETO(QueryResult r) throws TypeMismatchException {
    this.operatorASSIGN(r);
	}

/**
 * Method responsible for binary assign operation.
 * @param r the right operand of the binary assign operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary assign operation.
 * @see QueryResult
 */
	public QueryResult operatorASSIGN(QueryResult r) throws TypeMismatchException {
    try {
      YObject lobj = om.getObject(getId());
      if (lobj instanceof YReference) {
        if (r instanceof IdentifierValue)
          ((YReference)lobj).setValue(((IdentifierValue)r).getId());
        else
          throw new TypeMismatchException(this, "[ReferenceObject].operatorASSIGN", r);
      } else if (lobj instanceof YSimpleObject) {
        if (lobj instanceof YInteger && (r instanceof IntegerValue || r instanceof DoubleValue)) {
          ((YInteger)lobj).setValue(r.operatorTOINTEGER().intValue());
        } else if (lobj instanceof YDouble && (r instanceof IntegerValue || r instanceof DoubleValue)) {
          ((YDouble)lobj).setValue(r.operatorTODOUBLE().doubleValue());
        } else if (lobj instanceof YString && r instanceof StringValue) {
          ((YString)lobj).setValue(((StringValue)r).stringValue());
        } else {
          throw new TypeMismatchException(this, "[SimpleObject].operatorASSIGN", r);
        }
      } else if (lobj instanceof YComplexObject) {
        if (r instanceof StructValue || r instanceof BinderValue) {
          Vector aaaBezSensuBufor = new Vector();
          for (Enumeration e = ((YComplexObject)lobj).getSubordinates(); e.hasMoreElements(); ) {
            ObjectID elID = (ObjectID) e.nextElement();
            YObject el = om.getObject(elID);
            aaaBezSensuBufor.add(el);
//            el.disconnect();
          }
          for (int i=0; i<aaaBezSensuBufor.size(); i++)
            ((YObject)aaaBezSensuBufor.elementAt(i)).disconnect();

          r.operatorCREATEPERMANENT(this);
        } else
          throw new TypeMismatchException(this, "[ComplexObject].operatorASSIGN", r);
      }
    } catch (NoSuchObjectException e) { e.printStackTrace(); }
    return this;
	}

/**
 * Method responsible for binary plus assign operation.
 * @param r the right operand of the plus modulo assign operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary plus assign operation.
 * @see QueryResult
 */
	public QueryResult operatorPLUSASSIGN(QueryResult r) throws TypeMismatchException {
    return this.operatorASSIGN(this.operatorDEREFERENCE().operatorPLUS(r));
  }

/**
 * Method responsible for binary minus assign operation.
 * @param r the right operand of the binary minus assign operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary minus assign operation.
 * @see QueryResult
 */
	public QueryResult operatorMINUSASSIGN(QueryResult r) throws TypeMismatchException {
    return this.operatorASSIGN(this.operatorDEREFERENCE().operatorMINUS(r));
  }

/**
 * Method responsible for binary times assign operation.
 * @param r the right operand of the binary times assign operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary times assign operation.
 * @see QueryResult
 */
	public QueryResult operatorTIMESASSIGN(QueryResult r) throws TypeMismatchException {
    return this.operatorASSIGN(this.operatorDEREFERENCE().operatorTIMES(r));
  }

/**
 * Method responsible for binary divide assign operation.
 * @param r the right operand of the binary divide assign operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary divide assign operation.
 * @see QueryResult
 */
	public QueryResult operatorDIVIDEASSIGN(QueryResult r) throws TypeMismatchException {
    return this.operatorASSIGN(this.operatorDEREFERENCE().operatorDIVIDE(r));
  }

/**
 * Method responsible for binary modulo assign operation.
 * @param r the right operand of the binary modulo assign operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary modulo assign operation.
 * @see QueryResult
 */
	public QueryResult operatorMODULOASSIGN(QueryResult r) throws TypeMismatchException {
    return this.operatorASSIGN(this.operatorDEREFERENCE().operatorMODULO(r));
  }

/**
 * Method responsible for binary rename to operation.
 * @param r the right operand of the binary rename to operator.
 * @param ENVS environmental stack which has to be updated.
 * @exception TypeMismatchException if type mismatch occurs.
 * @see QueryResult
 * @see EnvironmentalStack
 * @see StringValue
 */
	public void operatorRENAMETO(QueryResult r, EnvironmentalStack ENVS) throws TypeMismatchException {
    try {
      YObject obj = om.getObject(getId());
      if (r instanceof StringValue) {
        ENVS.renameElement(new BinderValue(obj.getName(), this), ((StringValue)r).getValue());
        obj.setName(((StringValue)r).getValue());
      } else
        throw new TypeMismatchException(this, "operatorRENAMETO", r);
    } catch (NoSuchObjectException e) { e.printStackTrace(); }
	}

/**
 * Method responsible for binary insert operation.
 * @param r the right operand of the binary insert operator.
 * @param ENVS environmental stack which has to be updated.
 * @exception TypeMismatchException if type mismatch occurs.
 * @see QueryResult
 * @see EnvironmentalStack
 */
	public QueryResult operatorINSERT(QueryResult r, EnvironmentalStack ENVS) throws TypeMismatchException {
    if (r instanceof IdentifierValue) {
      if (this.operatorCOMPARE(r).booleanValue() == true)
        throw new TypeMismatchException(this, "operatorINSERT[lid = rid]", r);
      try {
        YObject lobj = om.getObject(this.getId());
        if (lobj instanceof YComplexObject) {
          YObject robj = om.getObject(((IdentifierValue)r).getId());
          robj.reconnect(lobj.getOID());
          ENVS.deleteAtBottom(new BinderValue(robj.getName(), r));
        } else
          throw new TypeMismatchException(this, "[not YComplexObject]operatorINSERT", r);
      } catch (NoSuchObjectException e) { e.printStackTrace(); }
    } else
      throw new TypeMismatchException(this, "operatorINSERT", r);
    return this;
  }

/**
 * Method responsible for binary insert into operation.
 * @param r the right operand of the binary insert into operator.
 * @param ENVS environmental stack which has to be updated.
 * @exception TypeMismatchException if type mismatch occurs.
 * @see QueryResult
 * @see EnvironmentalStack
 */
	public void operatorINSERTINTO(QueryResult r, EnvironmentalStack ENVS) throws TypeMismatchException {
    r.operatorINSERT(this, ENVS);
	}
}