package pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes;

import pl.edu.pjwstk.yaod.yaql.engine.*;

/**
 * The <code>IntegerValue</code> class represents the integer numbers.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class IntegerValue extends AtomicValue {
  Integer value;

/**
 * Creates a new integer value initialized with specified <code>value</code>.
 * @param value the initializing value.
 */
	public IntegerValue(Integer value){
		this.value = value;
	}

/**
 * Creates a new integer value initialized with specified <code>value</code>.
 * @param value the initializing value.
 */
	public IntegerValue(int value){
		this(new Integer(value));
	}

/**
 * Creates a new integer value initialized with specified <code>value</code>.
 * @param value the initializing value.
 */
  public IntegerValue(double value){
    this((int)value);
  }

/**
 * Returns a name of the result ({@link QueryResult}) type.
 * @see QueryResult
 * @return a integer result type name.
 */
  public String getTypeAsString() {
    return "TYPE_SINGLE_ATOM_INTEGER";
  }

/**
 * Returns the integer value as a string value.
 * @return the integer value as a string value.
 */
  public String getValueAsString() {
    return ((Integer)value).toString();
  }

/**
 * Returns the value of a integer value.
 * @return the value of a integer value.
 */
  public Integer getValue() {
    return (Integer) value;
  }

/**
 * Returns the integer value as a integer value.
 * @return the integer value as a integer value.
 */
  public int intValue() {
    return getValue().intValue();
  }

/**
 * Returns the integer value as a double value.
 * @return the integer value as a double value.
 */
  public double doubleValue() {
    return getValue().doubleValue();
  }

/**
 * Returns the integer value as a string value.
 * @return the integer value as a string value.
 */
  public String stringValue() {
    return getValue().toString();
  }




//******************************************************************************
//******************************* UNARY OPERATORS ******************************
//******************************************************************************
/**
 * Method responsible for unary plus operation.
 * @param r the right operand of the unary plus operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary plus operation.
 * @see QueryResult
 */
	public QueryResult operatorPLUS() throws TypeMismatchException {
    return new IntegerValue( + intValue()); 
	}

/**
 * Method responsible for unary minus operation.
 * @param r the right operand of the unary  minus operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary minus operation.
 * @see QueryResult
 */
	public QueryResult operatorMINUS() throws TypeMismatchException {
    return new IntegerValue( - intValue()); 
	}

/**
 * Method responsible for unary square root operation.
 * @param r the right operand of the unary square root operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary square root operation.
 * @see QueryResult
 * @see DoubleValue
 */
	public QueryResult operatorSQUAREROOT() throws TypeMismatchException {
    return new DoubleValue(Math.sqrt(doubleValue()));			
	}

/**
 * Method responsible for unary absolute value operation.
 * @param r the right operand of the unary absolute value operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary absolute value operation.
 * @see QueryResult
 */
	public QueryResult operatorABSOLUTEVALUE() throws TypeMismatchException {
    return new IntegerValue(Math.abs(intValue()));			
	}

/**
 * Method responsible for unary signum operation.
 * @param r the right operand of the unary signum operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary signum operation.
 * @see QueryResult
 */
	public QueryResult operatorSIGNUM() throws TypeMismatchException {
    return new IntegerValue( (intValue() >= 0) ? 1 : -1 );			
	}

/**
 * Method responsible for unary sine operation.
 * @param r the right operand of the unary sine operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary sine operation.
 * @see QueryResult
 * @see DoubleValue
 */
	public QueryResult operatorSINE() throws TypeMismatchException {
    return new DoubleValue(Math.sin(doubleValue()));			
	}

/**
 * Method responsible for unary cosine operation.
 * @param r the right operand of the unary cosine operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary cosine operation.
 * @see QueryResult
 * @see DoubleValue
 */
	public QueryResult operatorCOSINE() throws TypeMismatchException {
    return new DoubleValue(Math.cos(doubleValue()));			
	}

/**
 * Method responsible for unary exponent operation.
 * @param r the right operand of the unary exponent operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary exponent operation.
 * @see QueryResult
 * @see DoubleValue
 */
	public QueryResult operatorEXPONENT() throws TypeMismatchException {
    return new DoubleValue((float)Math.exp(doubleValue()));			
	}

/**
 * Method responsible for unary logarithm operation.
 * @param r the right operand of the unary logarithm operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary logarithm operation.
 * @see QueryResult
 * @see DoubleValue
 */
	public QueryResult operatorLOGARITHM() throws TypeMismatchException {
    return new DoubleValue((float)Math.log(doubleValue()));			
	}

/**
 * Method responsible for unary sum operation.
 * @param r the right operand of the unary sum operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary sum operation.
 * @see QueryResult
 */
	public QueryResult operatorSUM() throws TypeMismatchException {
    return this;
	}

/**
 * Method responsible for unary average operation.
 * @param r the right operand of the unary average operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary average operation.
 * @see QueryResult
 */
	public QueryResult operatorAVERAGE() throws TypeMismatchException {
    return this;
	}

/**
 * Method responsible for unary minimum operation.
 * @param r the right operand of the unary minimum operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary minimum operation.
 * @see QueryResult
 */
	public QueryResult operatorMINIMUM() throws TypeMismatchException {
    return this;
	}

/**
 * Method responsible for unary maximum operation.
 * @param r the right operand of the unary maximum operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary maximum operation.
 * @see QueryResult
 */
	public QueryResult operatorMAXIMUM() throws TypeMismatchException {
    return this;
	}

/**
 * Method responsible for unary descending operation.
 * @param r the right operand of the unary descending operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary descending operation.
 * @see QueryResult
 */
  public QueryResult operatorDESCENDING() throws TypeMismatchException {
    return this.operatorTIMES(new IntegerValue(-1));
	}

/**
 * Method responsible for unary tostring operation.
 * @param r the right operand of the unary tostring operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary tostring operation.
 * @see StringValue
 */
	public StringValue operatorTOSTRING() throws TypeMismatchException {
    return new StringValue(stringValue());
  }

/**
 * Method responsible for unary tointeger operation.
 * @param r the right operand of the unary tointeger operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary tointeger operation.
 */
	public IntegerValue operatorTOINTEGER() throws TypeMismatchException {
    return this;
  }

/**
 * Method responsible for unary todouble operation.
 * @param r the right operand of the unary todouble operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary todouble operation.
 * @see DoubleValue
 */
	public DoubleValue operatorTODOUBLE() throws TypeMismatchException {
    return new DoubleValue(doubleValue());
  }


//******************************************************************************
//******************************* BINARY OPERATORS *****************************
//******************************************************************************
/**
 * Method responsible for binary plus operation.
 * @param r the right operand of the binary plus operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary plus operation.
 * @see QueryResult
 * @see DoubleValue
 * @see StringValue
 */
 	public QueryResult operatorPLUS(QueryResult r) throws TypeMismatchException {
    if (r instanceof IntegerValue)
      return new IntegerValue(intValue() + ((IntegerValue)r).intValue()); 
    else if (r instanceof DoubleValue)
      return new DoubleValue(doubleValue() + ((DoubleValue)r).doubleValue()); 
    else if (r instanceof StringValue)
      return new StringValue(stringValue() + ((StringValue)r).stringValue());
    else
      throw new TypeMismatchException(this, "operatorPLUS", r);
	}

/**
 * Method responsible for binary minus operation.
 * @param r the right operand of the binary minus operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary minus operation.
 * @see QueryResult
 * @see DoubleValue
 */
	public QueryResult operatorMINUS(QueryResult r) throws TypeMismatchException {
    if (r instanceof IntegerValue)
      return new IntegerValue(intValue() - ((IntegerValue)r).intValue()); 
    else if (r instanceof DoubleValue)
      return new DoubleValue(doubleValue() - ((DoubleValue)r).doubleValue()); 
    else
      throw new TypeMismatchException(this, "operatorMINUS", r);
	}

/**
 * Method responsible for binary times operation.
 * @param r the right operand of the binary times operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary times operation.
 * @see QueryResult
 * @see DoubleValue
 */
	public QueryResult operatorTIMES(QueryResult r) throws TypeMismatchException {
    if (r instanceof IntegerValue)
      return new IntegerValue(intValue() * ((IntegerValue)r).intValue()); 
    else if (r instanceof DoubleValue)
      return new DoubleValue(doubleValue() * ((DoubleValue)r).doubleValue()); 
    else
      throw new TypeMismatchException(this, "operatorTIMES", r);
  }

/**
 * Method responsible for binary divide operation.
 * @param r the right operand of the binary divide operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary divide operation.
 * @see QueryResult
 * @see DoubleValue
 */
  public QueryResult operatorDIVIDE(QueryResult r) throws TypeMismatchException {
    if (r instanceof IntegerValue)
      return new DoubleValue(doubleValue() / ((IntegerValue)r).doubleValue()); 
    else if (r instanceof DoubleValue)
      return new DoubleValue(doubleValue() / ((DoubleValue)r).doubleValue()); 
    else
      throw new TypeMismatchException(this, "operatorDIVIDE", r);
	}

/**
 * Method responsible for binary equal operation.
 * @param r the right operand of the binary equal operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary equal operation.
 * @see QueryResult
 * @see BooleanValue
 * @see DoubleValue
 */
  public BooleanValue operatorEQUAL(QueryResult r) throws TypeMismatchException {
    if (r instanceof IntegerValue)
      return new BooleanValue(intValue() == ((IntegerValue)r).intValue());
    else if (r instanceof DoubleValue)
      return new BooleanValue(doubleValue() == ((DoubleValue)r).doubleValue());
    else
      throw new TypeMismatchException(this, "operatorEQUAL", r);
	}

/**
 * Method responsible for binary not equal operation.
 * @param r the right operand of the binary not equal operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary not equal operation.
 * @see QueryResult
 * @see BooleanValue
 * @see DoubleValue
 */
	public BooleanValue operatorNOTEQUAL(QueryResult r) throws TypeMismatchException {
    if (r instanceof IntegerValue)
      return new BooleanValue(intValue() != ((IntegerValue)r).intValue()); 
    else if (r instanceof DoubleValue)
      return new BooleanValue(doubleValue() != ((DoubleValue)r).doubleValue()); 
    else
      throw new TypeMismatchException(this, "operatorNOTEQUAL", r);
	}

/**
 * Method responsible for binary greater operation.
 * @param r the right operand of the binary greater operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary greater operation.
 * @see QueryResult
 * @see BooleanValue
 * @see DoubleValue
 */
	public BooleanValue operatorGREATER(QueryResult r) throws TypeMismatchException {
    if (r instanceof IntegerValue)
      return new BooleanValue(intValue() > ((IntegerValue)r).intValue()); 
    else if (r instanceof DoubleValue)
      return new BooleanValue(doubleValue() > ((DoubleValue)r).doubleValue()); 
    else
      throw new TypeMismatchException(this, "operatorGREATER", r);
	}

/**
 * Method responsible for binary lower operation.
 * @param r the right operand of the binary lower operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary lower operation.
 * @see QueryResult
 * @see BooleanValue
 * @see DoubleValue
 */
	public BooleanValue operatorLOWER(QueryResult r) throws TypeMismatchException {
    if (r instanceof IntegerValue)
      return new BooleanValue(intValue() < ((IntegerValue)r).intValue()); 
    else if (r instanceof DoubleValue)
      return new BooleanValue(doubleValue() < ((DoubleValue)r).doubleValue()); 
    else
      throw new TypeMismatchException(this, "operatorLOWER", r);
	}

/**
 * Method responsible for binary lower equal operation.
 * @param r the right operand of the binary lower equal operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary lower equal operation.
 * @see QueryResult
 * @see BooleanValue
 * @see DoubleValue
 */
	public BooleanValue operatorLOWEREQUAL(QueryResult r) throws TypeMismatchException {
    if (r instanceof IntegerValue)
      return new BooleanValue(intValue() <= ((IntegerValue)r).intValue()); 
    else if (r instanceof DoubleValue)
      return new BooleanValue(doubleValue() <= ((DoubleValue)r).doubleValue()); 
    else
      throw new TypeMismatchException(this, "operatorLOWEREQUAL", r);
	}

/**
 * Method responsible for binary greater equal operation.
 * @param r the right operand of the binary greater equal operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary greater equal operation.
 * @see QueryResult
 * @see BooleanValue
 * @see DoubleValue
 */
	public BooleanValue operatorGREATEREQUAL(QueryResult r) throws TypeMismatchException {
    if (r instanceof IntegerValue)
      return new BooleanValue(intValue() >= ((IntegerValue)r).intValue()); 
    else if (r instanceof DoubleValue)
      return new BooleanValue(doubleValue() >= ((DoubleValue)r).doubleValue()); 
    else
      throw new TypeMismatchException(this, "operatorGREATEREQUAL", r);
	}	

/**
 * Method responsible for binary modulo operation.
 * @param r the right operand of the binary modulo operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary modulo operation.
 * @see QueryResult
 * @see DoubleValue
 */
	public QueryResult operatorMODULO(QueryResult r) throws TypeMismatchException {
    if (r instanceof IntegerValue)
      return new IntegerValue(intValue() % ((IntegerValue)r).intValue()); 
    else if (r instanceof DoubleValue)
      return new IntegerValue(doubleValue() % ((DoubleValue)r).doubleValue()); 
    else
      throw new TypeMismatchException(this, "operatorMODULO", r);
	}

/**
 * Method responsible for binary power operation.
 * @param r the right operand of the binary power operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary power operation.
 * @see QueryResult
 * @see DoubleValue
 */
	public QueryResult operatorPOWER(QueryResult r) throws TypeMismatchException {
    if (r instanceof IntegerValue)
      return new DoubleValue(Math.pow(doubleValue(), ((IntegerValue)r).doubleValue())); 
    else if (r instanceof DoubleValue)
      return new DoubleValue(Math.pow(doubleValue(), ((DoubleValue)r).doubleValue())); 
    else
      throw new TypeMismatchException(this, "operatorPOWER", r);
	}

/**
 * Method responsible for binary compare operation.
 * @param r the right operand of the binary compare operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary compare operation.
 * @see QueryResult
 * @see BooleanValue
 */
	public BooleanValue operatorCOMPARE(QueryResult r) throws TypeMismatchException {
    if (r instanceof IntegerValue)
      return new BooleanValue(intValue() == ((IntegerValue)r).intValue());
    else
      return new BooleanValue(false);
	}
}