package pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes;

import java.io.*;
import pl.edu.pjwstk.yaod.yaql.engine.*;

/**
 * The <code>QueryResult</code> class represents the main class of available
 * results.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public abstract class QueryResult {

/**
 * Returns the query result type as a string value.
 * @return the result type as string.
 */
  public abstract String getTypeAsString();

/**
 * Returns the space string, which size is equal to the name of the longest
 * type name.
 * @return the name of the longest type, replaced by spaces.
 */
  public String getLongestTypeAsSpacesString() {
    String longestTypeName = "TYPE_SINGLE_ATOM_BOOLEAN";
    String res = "";
    for (int i=0; i<longestTypeName.length(); i++) 
      res += " ";
    return res;
  }

/**
 * Returns the prepared type name for display.
 * @return the prepared type name.
 */
  public String getPreparedTypeAsString() {
    String st = getTypeAsString();
    String longestTypeName = getLongestTypeAsSpacesString();
    int spaces = longestTypeName.length() - st.length();
    for (int i=0; i<spaces; i++)
      st += " ";
    return st;
  }

/**
 * Returns the new empty activation record. This is a default action for all 
 * query results.
 * @return the new empty activation record.
 * @see ActivationRecord
 */
  public ActivationRecord nested() {
    return new ActivationRecord();
  }

/**
 * Method used during evaluation of non algebraic operators. 
 * If the result is a complex value (collection) it should return the size
 * of such collection. Otherwise it should return <code>1</code>.
 * @return the number of elements in the collection.
 */
  public abstract int getNumberOfResults();

/**
 * Method used during evaluation of non algebraic operators. 
 * If the result is a complex value (collection) it should return the element 
 * <code>i</code>. Otherwise it should return <code>this</code>, despite of the 
 * value of <code>i</code>.
 * @see QueryResult
 * @param i the i-th element to be returned.
 * @return the i-th element in the collection.
 */
  public abstract QueryResult getNextResultAt(int i);


//******************************************************************************
//******************************* UNARY OPERATORS ******************************
//******************************************************************************
/**
 * Method responsible for dereference operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
  public QueryResult operatorDEREFERENCE() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorDEREFERENCE");
  }

/**
 * Method responsible for full dereference operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
  public QueryResult operatorFULLDEREFERENCE() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorFULLDEREFERENCE");
  }

/**
 * Method responsible for special dereference operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
  public QueryResult operatorSPECIALDEREFERENCE() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorSPECIALDEREFERENCE");
  }

/**
 * Method responsible for mixed dereference operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
  public QueryResult operatorMIXEDDEREFERENCE() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorMIXEDDEREFERENCE");
  }

/**
 * Method responsible for unary plus operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorPLUS() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorPLUS");
	}

/**
 * Method responsible for unary plus operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorMINUS() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorMINUS");
	}

/**
 * Method responsible for unary negation operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public BooleanValue operatorNEGATION() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorNEGATION");
	}

/**
 * Method responsible for unary exists operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public BooleanValue operatorEXISTS() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorEXISTS");
	}

/**
 * Method responsible for unary square root operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorSQUAREROOT() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorSQUAREROOT");
	}

/**
 * Method responsible for unary absolute value operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorABSOLUTEVALUE() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorABSOLUTEVALUE");
	}

/**
 * Method responsible for unary sine operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorSINE() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorSINE");
	}

/**
 * Method responsible for unary cosine operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorCOSINE() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorCOSINE");
	}

/**
 * Method responsible for unary exponent operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorEXPONENT() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorEXPONENT");
	}

/**
 * Method responsible for unary logarithm operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorLOGARITHM() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorLOGARITHM");
	}

/**
 * Method responsible for unary count operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public IntegerValue operatorCOUNT() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorCOUNT");
	}

/**
 * Method responsible for unary sum operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorSUM() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorSUM");
	}

/**
 * Method responsible for unary average operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorAVERAGE() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorAVERAGE");
	}

/**
 * Method responsible for unary minimum operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorMINIMUM() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorMINIMUM");
	}

/**
 * Method responsible for unary maximum operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorMAXIMUM() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorMAXIMUM");
	}

/**
 * Method responsible for unary signum operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorSIGNUM() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorSIGNUM");
	}

/**
 * Method responsible for unary struct operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorSTRUCT() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorSTRUCT");
	}

/**
 * Method responsible for unary bag operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorBAG() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorBAG");
	}

/**
 * Method responsible for unary sequence operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorSEQUENCE() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorSEQUENCE");
	}

/**
 * Method responsible for unary unique operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorUNIQUE() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorUNIQUE");
	}

/**
 * Method responsible for unary tostruct operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public StructValue operatorTOSTRUCT() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorTOSTRUCT");
  }

/**
 * Method responsible for unary tobag operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public BagValue operatorTOBAG() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorTOBAG");
  }

/**
 * Method responsible for unary tosequence operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public SequenceValue operatorTOSEQUENCE() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorTOSEQUENCE");
  }

/**
 * Method responsible for unary tosingle operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorTOSINGLE() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorTOSINGLE");
  }

/**
 * Method responsible for unary tostring operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public StringValue operatorTOSTRING() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorTOSTRING");
  }

/**
 * Method responsible for unary tointeger operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public IntegerValue operatorTOINTEGER() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorTOINTEGER");
  }

/**
 * Method responsible for unary todouble operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public DoubleValue operatorTODOUBLE() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorTODOUBLE");
  }

/**
 * Method responsible for unary ascending operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorASCENDING() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorASCENDING");
	}

/**
 * Method responsible for unary descending operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorDESCENDING() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorDESCENDING");
	}

/**
 * Method responsible for unary preincrement operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorPREINCREMENT() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorPREINCREMENT");
	}

/**
 * Method responsible for unary predecrement operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorPREDECREMENT() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorPREDECREMENT");
	}

/**
 * Method responsible for unary create local operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @param id Identifier of the parent object in object store.
 * @see IdentifierValue
 */
	public QueryResult operatorCREATELOCAL(IdentifierValue iv) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorCREATELOCAL");
  }

/**
 * Method responsible for unary create local operation on environmental stack.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @param ENVS environmental stack which has to be updated.
 * @see EnvironmentalStack
 */
	public void operatorCREATELOCAL(EnvironmentalStack ENVS) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorCREATELOCAL[ENVS]");
  }

/**
 * Method responsible for unary create permanent operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @param id Identifier of the parent object in object store.
 * @see IdentifierValue
 */
	public QueryResult operatorCREATEPERMANENT(IdentifierValue iv) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorCREATEPERMANENT");
  }

/**
 * Method responsible for unary create permanent operation on environmental stack.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @param ENVS environmental stack which has to be updated.
 * @see EnvironmentalStack
 */
	public void operatorCREATEPERMANENT(EnvironmentalStack ENVS) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorCREATEPERMANENT[ENVS]");
  }

/**
 * Method responsible for unary create operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @param id Identifier of the parent object in object store.
 * @see IdentifierValue
 */
	public QueryResult operatorCREATE(IdentifierValue iv) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorCREATE");
  }

/**
 * Method responsible for unary create operation on environmental stack.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @param ENVS environmental stack which has to be updated.
 * @see EnvironmentalStack
 */
	public void operatorCREATE(EnvironmentalStack ENVS) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorCREATE[ENVS]");
  }

/**
 * Method responsible for unary create operation on environmental stack.
 * By default, it always throws {@link TypeMismatchException}.
 * @exception TypeMismatchException if type mismatch occurs.
 * @param ENVS environmental stack which has to be updated.
 * @see EnvironmentalStack
 */
	public void operatorDELETE(EnvironmentalStack ENVS) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorDELETE");
	}

/**
 * Method responsible for unary print operation.
 * @param out the object on which the result will be displayed.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public void operatorPRINT(PrintWriter out) throws TypeMismatchException {
    operatorPRINT("", out);
  }

/**
 * Method responsible for unary print operation.
 * @param st the auxiliary string, used as a tabulator.
 * @param out the object on which the result will be displayed.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public void operatorPRINT(String st, PrintWriter out) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorPRINT");
	}

/**
 * Method responsible for unary xmlprint operation.
 * @param out the object on which the result will be displayed.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public void operatorXMLPRINT(PrintWriter out) throws TypeMismatchException {
    operatorXMLPRINT("", out);
	}

/**
 * Method responsible for unary xmlprint operation.
 * @param st the auxiliary string, used as a tabulator.
 * @param out the object on which the result will be displayed.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public void operatorXMLPRINT(String st, PrintWriter out) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorXMLPRINT");
	}

/**
 * Method responsible for unary show operation.
 * @param st the auxiliary string, used as a tabulator.
 * @param out the object on which the result will be displayed.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
  public abstract void show(String st, PrintWriter out);

/**
 * Method responsible for unary show operation.
 * @param out the object on which the result will be displayed.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
  public void show(PrintWriter out) {
    show("", out);
  }


//******************************************************************************
//************************* UNARY PARAMETRIC OPERATORS *************************
//******************************************************************************
/**
 * Method responsible for binary as operation.
 * @param r the right operand of the binary as operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorAS(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorAS", r);
	}

/**
 * Method responsible for binary group as operation.
 * @param r the right operand of the binary group as operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @see BinderValue
 */
	public BinderValue operatorGROUPAS(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorGROUPAS", r);
	}


//******************************************************************************
//******************************* BINARY OPERATORS *****************************
//******************************************************************************
/**
 * Method responsible for binary plus operation.
 * @param r the right operand of the binary plus operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorPLUS(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorPLUS", r);
	}
/**
 * Method responsible for binary minus operation.
 * @param r the right operand of the binary minus operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorMINUS(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorMINUS", r);
	}

/**
 * Method responsible for binary times operation.
 * @param r the right operand of the binary times operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorTIMES(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorTIMES", r);
	}

/**
 * Method responsible for binary divide operation.
 * @param r the right operand of the binary divide operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorDIVIDE(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorDIVIDE", r);
	}

/**
 * Method responsible for binary modulo operation.
 * @param r the right operand of the binary modulo operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorMODULO(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorMODULO", r);
	}

/**
 * Method responsible for binary power operation.
 * @param r the right operand of the binary power operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorPOWER(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorPOWER", r);
	}

/**
 * Method responsible for binary equal operation.
 * @param r the right operand of the binary equal operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @see BooleanValue
 */
	public BooleanValue operatorEQUAL(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorEQUAL", r);
	}

/**
 * Method responsible for binary not equal operation.
 * @param r the right operand of the binary not equal operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @see BooleanValue
 */
	public BooleanValue operatorNOTEQUAL(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorNOTEQUAL", r);
	}

/**
 * Method responsible for binary greater operation.
 * @param r the right operand of the binary greater operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @see BooleanValue
 */
	public BooleanValue operatorGREATER(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorGREATER", r);
	}

/**
 * Method responsible for binary lower operation.
 * @param r the right operand of the binary lower operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @see BooleanValue
 */
	public BooleanValue operatorLOWER(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorLOWER", r);
	}

/**
 * Method responsible for binary greater equal operation.
 * @param r the right operand of the binary greater equal operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @see BooleanValue
 */
	public BooleanValue operatorGREATEREQUAL(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorGREATEREQUAL", r);
	}

/**
 * Method responsible for binary lower equal operation.
 * @param r the right operand of the binary lower equal operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @see BooleanValue
 */
	public BooleanValue operatorLOWEREQUAL(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorLOWEREQUAL", r);
	}

/**
 * Method responsible for binary conjunction operation.
 * @param r the right operand of the binary conjunction operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @see BooleanValue
 */
	public BooleanValue operatorCONJUNCTION(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorCONJUNCTION", r);
	}

/**
 * Method responsible for binary disjunction operation.
 * @param r the right operand of the binary disjunction operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @see BooleanValue
 */
	public BooleanValue operatorDISJUNCTION(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorDISJUNCTION", r);
	}

/**
 * Method responsible for binary comma operation.
 * @param r the right operand of the binary comma operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorCOMMA(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorCOMMA", r);
	}

/**
 * Method responsible for binary inclusion operation.
 * @param r the right operand of the binary inclusion operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @see BooleanValue
 */
	public BooleanValue operatorINCLUSION(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorINCLUSION", r);
	}

/**
 * Method responsible for binary union operation.
 * @param r the right operand of the binary union operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorUNION(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorUNION", r);
	}

/**
 * Method responsible for binary intersection operation.
 * @param r the right operand of the binary intersection operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorINTERSECTION(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorINTERSECTION", r);
	}

/**
 * Method responsible for binary difference operation.
 * @param r the right operand of the binary difference operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorDIFFERENCE(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorDIFFERENCE", r);
	}

/**
 * Method responsible for binary symmetric difference operation.
 * @param r the right operand of the binary symmetric difference operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorSYMMETRICDIFFERENCE(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorSYMMETRICDIFFERENCE", r);
	}

/**
 * Method responsible for binary assign operation.
 * @param r the right operand of the binary assign operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorASSIGN(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorASSIGN", r);
	}

/**
 * Method responsible for binary plus assign operation.
 * @param r the right operand of the binary plus assign operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorPLUSASSIGN(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorPLUSASSIGN", r);
	}

/**
 * Method responsible for binary minus assign operation.
 * @param r the right operand of the binary minus assign operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorMINUSASSIGN(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorMINUSASSIGN", r);
	}

/**
 * Method responsible for binary times assign operation.
 * @param r the right operand of the binary times assign operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorTIMESASSIGN(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorTIMESSSIGN", r);
	}

/**
 * Method responsible for binary divide assign operation.
 * @param r the right operand of the binary divide assign operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorDIVIDEASSIGN(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorDIVIDEASSIGN", r);
	}

/**
 * Method responsible for binary modulo assign operation.
 * @param r the right operand of the binary modulo assign operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorMODULOASSIGN(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorMODULOASSIGN", r);
	}

/**
 * Method responsible for binary range operation.
 * @param r the right operand of the binary range operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorRANGE(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorRANGE", r);
	}

/**
 * Method responsible for binary compare operation.
 * @param r the right operand of the binary compare operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @see BooleanValue
 */
	public BooleanValue operatorCOMPARE(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorCOMPARE", r);
	}

/**
 * Method responsible for binary rename to operation.
 * By default, it always throws {@link TypeMismatchException}.
 * @param r the right operand of the binary rename to operator.
 * @param ENVS environmental stack which has to be updated.
 * @exception TypeMismatchException if type mismatch occurs.
 * @see EnvironmentalStack
 */
	public void operatorRENAMETO(QueryResult r, EnvironmentalStack ENVS) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorRENAMETO", r);
	}

/**
 * Method responsible for binary insert into operation.
 * By default, it always throws {@link TypeMismatchException}.
 * @param r the right operand of the binary insert into operator.
 * @param ENVS environmental stack which has to be updated.
 * @exception TypeMismatchException if type mismatch occurs.
 * @see EnvironmentalStack
 */
	public void operatorINSERTINTO(QueryResult r, EnvironmentalStack ENVS) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorINSERTINTO", r);
	}

/**
 * Method responsible for binary update to operation.
 * By default, it always throws {@link TypeMismatchException}.
 * @param r the right operand of the binary update to operator.
 * @exception TypeMismatchException if type mismatch occurs.
 */
	public void operatorUPDATETO(QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorUPDATETO", r);
	}

/**
 * Method responsible for binary insert operation.
 * @param r the right operand of the binary insert operator.
 * @param ENVS environmental stack which has to be updated.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 * @see EnvironmentalStack
 */
	public QueryResult operatorINSERT(QueryResult r, EnvironmentalStack ENVS) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorINSERT", r);
  }


//******************************************************************************
//***************************** TERNARY OPERATORS ******************************
//******************************************************************************
/**
 * Method responsible for ternary condition operation.
 * @param m the middle operand of the ternary condition operator.
 * @param r the right operand of the ternary condition operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorCONDITION(QueryResult m, QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorCONDITION", m, r);
	}

/**
 * Method responsible for ternary range operation.
 * @param m the middle operand of the ternary range operator.
 * @param r the right operand of the ternary range operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return by default, it always throws {@link TypeMismatchException}.
 */
	public QueryResult operatorRANGE(QueryResult m, QueryResult r) throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorRANGE", m, r);
	}

}
