package pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes;

import java.util.*;
import pl.edu.pjwstk.yaod.yaql.engine.*;

/**
 * The <code>SequenceValue</code> class represents the sequence collections.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class SequenceValue extends ComplexValue {

/**
 * Creates a new empty sequence collection.
 */
	public SequenceValue() { 
  }

/**
 * Creates a new sequence collection initialized with the <code>v</code>.
 * @param v vector of elements.
 */
	public SequenceValue(Vector v){
		super(v);
	}

/**
 * Creates a new sequence collection initialized with the <code>r1</code>.
 * @param r1 any result.
 * @param r2 any result.
 * @see QueryResult
 */
	public SequenceValue(QueryResult r) {
    addResult(r);
  }

/**
 * Creates a new sequence collection initialized with the <code>r1</code> and
 * <code>r2</code>.
 * @param r1 any result.
 * @param r2 any result.
 * @see QueryResult
 */
	public SequenceValue(QueryResult r1, QueryResult r2) {
    addResult(r2);
  }

/**
 * Returns a name of the result ({@link QueryResult}) type.
 * @see QueryResult
 * @return a binder type name.
 */
  public String getTypeAsString() {
    return "TYPE_COMPLEX_SEQUENCE";
  }



//******************************************************************************
//****************************** UNARY OPERATORS *******************************
//******************************************************************************
/**
 * Method responsible for unary bag operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary bag operation.
 * @see QueryResult
 */
	public QueryResult operatorBAG() throws TypeMismatchException {
    return this.operatorTOBAG();
	}

/**
 * Method responsible for unary struct operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary struct operation.
 * @see QueryResult
 */
	public QueryResult operatorSTRUCT() throws TypeMismatchException {
    throw new TypeMismatchException(this, "operatorSTRUCT");
	}

/**
 * Method responsible for unary sequence operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary sequence operation.
 * @see QueryResult
 */
	public QueryResult operatorSEQUENCE() throws TypeMismatchException {
    return this;
	}

/**
 * Method responsible for unary unique operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary unique operation.
 * @see QueryResult
 * @see BagValue
 */
	public QueryResult operatorUNIQUE() throws TypeMismatchException {
    BagValue res = new BagValue();
    for (int i=0; i<getSize(); i++) {
      QueryResult tmp = getResultAt(i).operatorDEREFERENCE();
      boolean found = false;
      for (int j=0; j<i; j++) {
        if (tmp.operatorCOMPARE(getResultAt(j).operatorDEREFERENCE()).booleanValue() == true) {
          found = true; break;
        }
      }
      if (found == false) res.addResult(tmp);
    }
    return res;
	}


//******************************************************************************
//************************ UNARY PARAMETRIC OPERATORS **************************
//******************************************************************************
/**
 * Method responsible for unary as operation.
 * @param r the right operand of the unary as operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary as operation.
 * @see QueryResult
 * @see BinderValue
 * @see StringValue
 */
	public QueryResult operatorAS(QueryResult r) throws TypeMismatchException {
    SequenceValue res = new SequenceValue();
    for (int i=0; i<getSize(); i++)
      res.addResult(new BinderValue(((StringValue)r).getValue(), getResultAt(i)));
    return res.operatorTOSINGLE();
	}


//******************************************************************************
//****************************** BINARY OPERATORS ******************************
//******************************************************************************
/**
 * Method responsible for binary union operation.
 * @param r the right operand of the binary union operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary union operation.
 * @see QueryResult
 * @see BagValue
 * @see SingleValue
 * @see StructValue
 */
	public QueryResult operatorUNION(QueryResult r) throws TypeMismatchException {
    if (r instanceof SingleValue || r instanceof StructValue)
      return this.operatorTOBAG().addResult(r);
    else if (r instanceof BagValue)
      return this.operatorTOBAG().addBag(r);
    else if (r instanceof SequenceValue)
      return this.operatorTOBAG().addBag(r.operatorTOBAG());
    else
      throw new TypeMismatchException(this, "operatorUNION", r);
	}

/**
 * Method responsible for binary comma operation.
 * @param r the right operand of the binary comma operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary comma operation.
 * @see QueryResult
 * @see BagValue
 * @see SingleValue
 * @see StructValue
 */
	public QueryResult operatorCOMMA(QueryResult r) throws TypeMismatchException {
    BagValue res = new BagValue();
    if (r instanceof SingleValue) {
      for (int i=0; i<getSize(); i++)
        res.addResult((new StructValue()).addStruct(getResultAt(i)).addResult(r));
      return res;
    } else if (r instanceof StructValue) {
      for (int i=0; i<getSize(); i++)
        res.addResult((new StructValue()).addStruct(getResultAt(i)).addStruct(r));
      return res;
    } else if (r instanceof BagValue) {
      for (int i=0; i<getSize(); i++)
        for (int j=0; j<((BagValue)r).getSize(); j++)
          res.addResult((new StructValue()).addStruct(getResultAt(i)).addStruct(((BagValue)r).getResultAt(j)));
      return res;
    } else if (r instanceof SequenceValue) {
      for (int i=0; i<getSize(); i++)
        for (int j=0; j<((SequenceValue)r).getSize(); j++)
          res.addResult((new StructValue()).addStruct(getResultAt(i)).addStruct(((SequenceValue)r).getResultAt(j)));
      return res;
    } else
      throw new TypeMismatchException(this, "operatorCOMMA", r);
	}

/**
 * Method responsible for binary inclusion operation.
 * @param r the right operand of the binary inclusion operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary inclusion operation.
 * @see QueryResult
 * @see BooleanValue
 */
	public BooleanValue operatorINCLUSION(QueryResult r) throws TypeMismatchException {
    if (r instanceof SequenceValue) {
      for (int j=0, i=0; i<getSize(); i++) {
        boolean found = false;
        for (; j<((SequenceValue)r).getSize(); j++)
          if (getResultAt(i).operatorDEREFERENCE().operatorCOMPARE(((SequenceValue)r).getResultAt(j).operatorDEREFERENCE()).booleanValue() == true) {
            found = true; ++j; break;
          }
        if (found == false) return new BooleanValue(false);
      }
      return new BooleanValue(true);
    } else
      throw new TypeMismatchException(this, "operatorINCLUSION", r);
	}

/**
 * Method responsible for binary compare operation.
 * @param r the right operand of the binary compare operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary compare operation.
 * @see QueryResult
 * @see BooleanValue
 */
	public BooleanValue operatorCOMPARE(QueryResult r) throws TypeMismatchException {
    if (r instanceof SequenceValue) {
      if (getSize() == ((SequenceValue)r).getSize()) {
        for (int i=0; i<getSize(); i++)
          if (getResultAt(i).operatorDEREFERENCE().operatorCOMPARE(((SequenceValue)r).getResultAt(i).operatorDEREFERENCE()).getValue().booleanValue() == false)
            return new BooleanValue(false);
        return new BooleanValue(true);
      } else return new BooleanValue(false);
    } else
      return new BooleanValue(false);
	}

/**
 * Method responsible for binary range operation.
 * @param r the right operand of the binary range operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary range operation.
 * @see QueryResult
 * @see IntegerValue
 */
	public QueryResult operatorRANGE(QueryResult r) throws TypeMismatchException {
    if (!(r instanceof IntegerValue)) 
      throw new TypeMismatchException(this, "operatorRANGE", r);
    if (r.operatorLOWEREQUAL(new IntegerValue(0)).booleanValue() == true)
      throw new TypeMismatchException("Wrong range parameter (r <= 0): "+this.getClass().getName()+".operatorRANGE("+r.getTypeAsString()+")");
    if (r.operatorGREATER(new IntegerValue(getSize()+1)).booleanValue() == true)
      throw new TypeMismatchException("Wrong range parameter (r > seq.size): "+this.getClass().getName()+".operatorRANGE("+r.getTypeAsString()+")");

    for (int i=0; i<getSize(); i++) {
      if ((i+1) == ((IntegerValue)r).intValue() )
        return getResultAt(i);
    }
    throw new TypeMismatchException("operatorRANGE() failed!!!");
	}


//******************************************************************************
//****************************** BINARY OPERATORS ******************************
//******************************************************************************
/**
 * Method responsible for ternary range operation.
 * @param m the middle operand of the ternary range operator.
 * @param r the right operand of the ternary range operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the ternary range operation.
 * @see QueryResult
 * @see IntegerValue
 */
  public QueryResult operatorRANGE(QueryResult l, QueryResult r) throws TypeMismatchException {
    if (!(l instanceof IntegerValue)) 
      throw new TypeMismatchException(this, "operatorRANGE", l, r);
    if (!(r instanceof IntegerValue)) 
      throw new TypeMismatchException(this, "operatorRANGE", l, r);
    if (l.operatorGREATER(r).booleanValue() == true)
      throw new TypeMismatchException("Wrong range parameters (l > r): "+this.getClass().getName()+".operatorRANGE("+l.getTypeAsString()+r.getTypeAsString()+")");
    if (l.operatorLOWEREQUAL(new IntegerValue(0)).booleanValue() == true)
      throw new TypeMismatchException("Wrong range parameters (l <= 0): "+this.getClass().getName()+".operatorRANGE("+l.getTypeAsString()+r.getTypeAsString()+")");
    if (r.operatorGREATER(new IntegerValue(getSize()+1)).booleanValue() == true)
      throw new TypeMismatchException("Wrong range parameters (r > seq.size): "+this.getClass().getName()+".operatorRANGE("+l.getTypeAsString()+r.getTypeAsString()+")");

    SequenceValue seq = new SequenceValue();
    for (int i=0; i<getSize(); i++) {
      if ((i+1) >= ((IntegerValue)l).intValue() && (i+1) <= ((IntegerValue)r).intValue())
        seq.addResult(getResultAt(i));
    }
    return seq;
	}

}