package pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes;

import pl.edu.pjwstk.yaod.yaql.engine.*;

/**
 * The <code>SingleValue</code> class represents the main class of available
 * single results (binders, identifiers and atomic values).
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public abstract class SingleValue extends QueryResult {

/**
 * Method used during evaluation of non algebraic operators. 
 * It returns <code>1</code> for all single results (identifiers, binders
 * and atomic values).
 * @return <code>1</code> always.
 */
  public int getNumberOfResults() {
    return 1;
  }

/**
 * Method used during evaluation of non algebraic operators. 
 * It returns <code>this</code> for all single results, despite of the value 
 * of argument <code>i</code>.
 * @see QueryResult
 * @param i in single values it is ignored.
 * @return <code>this</code> always.
 */
  public QueryResult getNextResultAt(int i) {
    return this;
  }



//******************************************************************************
//******************************* UNARY OPERATORS ******************************
//******************************************************************************
/**
 * Method responsible for unary tosingle operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary as operation.
 * @see QueryResult
 */
	public QueryResult operatorTOSINGLE() throws TypeMismatchException {
		return this;
  }

/**
 * Method responsible for unary exists operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary as operation.
 * @see BooleanValue
 */
	public BooleanValue operatorEXISTS() throws TypeMismatchException {
    return new BooleanValue(true);
	}

/**
 * Method responsible for unary count operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary count operation.
 * @see IntegerValue
 */
	public IntegerValue operatorCOUNT() throws TypeMismatchException {
    return new IntegerValue(1);
	}

/**
 * Method responsible for unary bag operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary as operation.
 * @see QueryResult
 * @see BagValue
 */
	public QueryResult operatorBAG() throws TypeMismatchException {
    return new BagValue(this);
	}

/**
 * Method responsible for unary struct operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary as operation.
 * @see QueryResult
 * @see StructValue
 */
	public QueryResult operatorSTRUCT() throws TypeMismatchException {
    return new StructValue(this);
	}

/**
 * Method responsible for unary sequence operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary as operation.
 * @see QueryResult
 * @see SequenceValue
 */
	public QueryResult operatorSEQUENCE() throws TypeMismatchException {
    return new SequenceValue(this);
	}

/**
 * Method responsible for unary unique operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary as operation.
 * @see QueryResult
 */
	public QueryResult operatorUNIQUE() throws TypeMismatchException {
    return this;
	}

/**
 * Method responsible for unary ascending operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary as operation.
 * @see QueryResult
 */
	public QueryResult operatorASCENDING() throws TypeMismatchException {
    return this;
	}


//******************************************************************************
//************************* UNARY PARAMETRIC OPERATORS *************************
//******************************************************************************
/**
 * Method responsible for unary as operation.
 * @param r the right operand of the unary as operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary as operation.
 * @see QueryResult
 * @see BinderValue
 * @see StringValue
 */
  public BinderValue operatorGROUPAS(QueryResult r) throws TypeMismatchException {
    return new BinderValue(((StringValue)r).getValue(), this);
	}

/**
 * Method responsible for unary group as operation.
 * @param r the right operand of the unary group as operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary group as operation.
 * @see QueryResult
 * @see BinderValue
 * @see StringValue
 */
	public QueryResult operatorAS(QueryResult r) throws TypeMismatchException {
    return new BinderValue(((StringValue)r).getValue(), this);
	}


//******************************************************************************
//******************************* BINARY OPERATORS *****************************
//******************************************************************************
/**
 * Method responsible for binary union operation.
 * @param r the right operand of the binary union operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary union operation.
 * @see QueryResult
 * @see IntegerValue
 * @see StructValue
 * @see BagValue
 */
	public QueryResult operatorUNION(QueryResult r) throws TypeMismatchException {
    if (r instanceof SingleValue || r instanceof StructValue)
      return new BagValue(this, r);
    else if (r instanceof BagValue)
      return (new BagValue(this)).addBag(r);
    else if (r instanceof SequenceValue)
      return (new BagValue(this)).addBag(r.operatorTOBAG());
    else 
      throw new TypeMismatchException(this, "operatorUNION", r);
	}

/**
 * Method responsible for binary comma operation.
 * @param r the right operand of the binary comma operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary comma operation.
 * @see QueryResult
 * @see SequenceValue
 * @see StructValue
 * @see BagValue
 */
	public QueryResult operatorCOMMA(QueryResult r) throws TypeMismatchException {
    if (r instanceof SingleValue)
      return new StructValue(this, r);
    else if (r instanceof StructValue)
      return (new StructValue(this)).addStruct(r);
    else if (r instanceof BagValue) { 
      BagValue res = new BagValue();
      for (int i=0; i<((BagValue)r).getSize(); i++)
        res.addResult(new StructValue(this).addStruct(((BagValue)r).getResultAt(i)));
      return res;
    } else if (r instanceof SequenceValue) {
      BagValue res = new BagValue();
      for (int i=0; i<((SequenceValue)r).getSize(); i++)
        res.addResult(new StructValue(this).addStruct(((SequenceValue)r).getResultAt(i)));
      return res;
    } else
      throw new TypeMismatchException(this, "operatorCOMMA", r);
	}

/**
 * Method responsible for binary inclusion operation.
 * @param r the right operand of the binary inclusion operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary inclusion operation.
 * @see QueryResult
 * @see BooleanValue
 * @see SequenceValue
 * @see BagValue
 */
	public BooleanValue operatorINCLUSION(QueryResult r) throws TypeMismatchException {
    if (r instanceof BagValue || r instanceof SequenceValue) {
      for (int i=0; i<((ComplexValue)r).getSize(); i++) {
        if (this.operatorCOMPARE(((ComplexValue)r).getResultAt(i).operatorDEREFERENCE()).booleanValue() == true)
          return new BooleanValue(true);
      }
      return new BooleanValue(false);
    } else
      throw new TypeMismatchException(this, "operatorINCLUSION", r);
	}
}
