package pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes;

import pl.edu.pjwstk.yaod.yaql.engine.*;

/**
 * The <code>StringValue</code> class represents the string values.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class StringValue extends AtomicValue {
  String value;

/**
 * Creates a new string initialized with specified <code>value</code>.
 * @param value the initializing value.
 */
	public StringValue(String value) {
		this.value = value;
	}

/**
 * Returns a name of the result ({@link QueryResult}) type.
 * @see QueryResult
 * @return a string result type name.
 */
  public String getTypeAsString() {
    return "TYPE_SINGLE_ATOM_STRING";
  }

/**
 * Returns the string value as a string value.
 * @return the result type as string.
 */
  public String getValueAsString() {
    return ((String)value).toString();
  }

/**
 * Returns the value of the string value.
 * @return the value of the string value.
 */
  public String getValue() {
    return (String) value;
  }

/**
 * Returns the string value as a string value.
 * @return the string value as a string value.
 */
  public String stringValue() {
    return getValue().toString();
  }


//******************************************************************************
//******************************* UNARY OPERATORS ******************************
//******************************************************************************
/**
 * Method responsible for unary descending operation.
 * @param r the right operand of the unary descending operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary descending operation.
 * @see QueryResult
 */
	public QueryResult operatorDESCENDING() throws TypeMismatchException {
    char data[] = new char[getValue().length()];
    for (int i=0; i<getValue().length(); i++)
      data[i] = (char)(-getValue().charAt(i));
    value = new String(data);  
    return this;
	}

/**
 * Method responsible for unary tostring operation.
 * @param r the right operand of the unary tostring operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary tostring operation.
 */
	public StringValue operatorTOSTRING() throws TypeMismatchException {
    return this;
  }


//******************************************************************************
//******************************* BINARY OPERATORS ******************************
//******************************************************************************
/**
 * Method responsible for binary plus operation, in this case - string 
 * concatenation.
 * @param r the right operand of the binary plus operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return concatenated string.
 * @see QueryResult
 * @see BooleanValue
 * @see IntegerValue
 * @see DoubleValue
 */
 	public QueryResult operatorPLUS(QueryResult r) throws TypeMismatchException {
    if (r instanceof BooleanValue)
      return new StringValue(stringValue() + ((BooleanValue)r).stringValue());
    else if (r instanceof IntegerValue)
      return new StringValue(stringValue() + ((IntegerValue)r).stringValue()); 
    else if (r instanceof DoubleValue)
      return new StringValue(stringValue() + ((DoubleValue)r).stringValue()); 
    else if (r instanceof StringValue)
      return new StringValue(stringValue() + ((StringValue)r).stringValue());
    else
      throw new TypeMismatchException(this, "operatorPLUS", r);
	}

/**
 * Method responsible for binary equal operation.
 * @param r the right operand of the binary equal operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary equal operation.
 * @see QueryResult
 * @see BooleanValue
 */
 	public BooleanValue operatorEQUAL(QueryResult r) throws TypeMismatchException {
    if (r instanceof StringValue)
      return new BooleanValue(stringValue().equals(((StringValue)r).stringValue()));
    else
      throw new TypeMismatchException(this, "operatorEQUAL", r);
	}

/**
 * Method responsible for binary not equal operation.
 * @param r the right operand of the binary not equal operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary not equal operation.
 * @see QueryResult
 * @see BooleanValue
 */
 	public BooleanValue operatorNOTEQUAL(QueryResult r) throws TypeMismatchException {
    if (r instanceof StringValue)
      return new BooleanValue(! stringValue().equals(((StringValue)r).stringValue()));
    else
      throw new TypeMismatchException(this, "operatorNOTEQUAL", r);
	}

/**
 * Method responsible for binary compare operation.
 * @param r the right operand of the binary compare operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary compare operation.
 * @see QueryResult
 * @see BooleanValue
 */
	public BooleanValue operatorCOMPARE(QueryResult r) throws TypeMismatchException {
    if (r instanceof StringValue)
      return new BooleanValue(stringValue().equals(((StringValue)r).stringValue()));
    else
      return new BooleanValue(false);
	}
}