package pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes;

import java.util.*;
import pl.edu.pjwstk.yaod.yaql.engine.*;

/**
 * The <code>StructValue</code> class represents the structure collections.
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class StructValue extends ComplexValue {

/**
 * Creates a new empty struct collection.
 */
	public StructValue() { 
  }

/**
 * Creates a new struct collection initialized with the <code>v</code>.
 * @param v vector of elements.
 */
	public StructValue(Vector v) {
		super(v);
	}

/**
 * Creates a new struct collection initialized with the <code>r</code>.
 * @param r any result.
 * @see QueryResult
 */
	public StructValue(QueryResult r) {
    addResult(r);
  }

/**
 * Creates a new bag collection initialized with the <code>r1</code> and
 * <code>r2</code>.
 * @param r1 any result.
 * @param r2 any result.
 * @see QueryResult
 */
	public StructValue(QueryResult r1, QueryResult r2) {
    this(r1);
    addResult(r2);
  }

/**
 * Returns a name of the result ({@link QueryResult}) type.
 * @see QueryResult
 * @return a binder type name.
 */
  public String getTypeAsString() {
    return "TYPE_COMPLEX_STRUCT";
  }
  
/**
 * Returns the sum of recursively called <code>nested</code> method for all
 * structure elements.
 * @return the sum of recursively called <code>nested</code> method for all
 * structure elements.
 * @see ActivationRecord
 */
  public ActivationRecord nested() {
    ActivationRecord ar = new ActivationRecord();
    for (int i=0; i<getSize(); i++)
      ar.addBinders(getResultAt(i).nested());
    return ar;
  }

/**
 * Method used during evaluation of non algebraic operators. 
 * It returns <code>1</code> for all single results (identifiers, binders
 * and atomic values). The structure is treated as a single result!
 * @return <code>1</code> always.
 */
  public int getNumberOfResults() {
    return 1;
  }

/**
 * Method used during evaluation of non algebraic operators. 
 * It returns <code>this</code> for all single results, despite of the value 
 * of argument <code>i</code>. The structure is treated as a single result!
 * @see QueryResult
 * @param i in single values it is ignored.
 * @return <code>this</code> always.
 */
  public QueryResult getNextResultAt(int i) {
    return this;
  }


//******************************************************************************
//******************************* UNARY OPERATORS ******************************
//******************************************************************************
/**
 * Method responsible for unary count operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary count operation.
 * @see IntegerValue
 */
  public IntegerValue operatorCOUNT() throws TypeMismatchException {
    return new IntegerValue(1);
	}

/**
 * Method responsible for unary struct operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary struct operation.
 * @see QueryResult
 */
	public QueryResult operatorSTRUCT() throws TypeMismatchException {
    return this;
	}

/**
 * Method responsible for unary bag operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary bag operation.
 * @see QueryResult
 * @see BagValue
 */
	public QueryResult operatorBAG() throws TypeMismatchException {
    return new BagValue(this);
	}

/**
 * Method responsible for unary sequence operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary sequence operation.
 * @see QueryResult
 * @see SequenceValue
 */
	public QueryResult operatorSEQUENCE() throws TypeMismatchException {
    return new SequenceValue(this);
	}

/**
 * Method responsible for unary unique operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary unique operation.
 * @see QueryResult
 */
	public QueryResult operatorUNIQUE() throws TypeMismatchException {
    return this;
	}

/**
 * Method responsible for unary descending operation.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary descending operation.
 * @see QueryResult
 */
	public QueryResult operatorDESCENDING() throws TypeMismatchException {
    for (int i=0; i<getSize(); i++)
      setResultAt(i,getResultAt(i).operatorDEREFERENCE().operatorDESCENDING());
    return this;
	}


//******************************************************************************
//************************** UNARY PARAMETRIC OPERATORS ************************
//******************************************************************************
/**
 * Method responsible for unary as operation.
 * @param r the right operand of the unary as operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the unary as operation.
 * @see QueryResult
 * @see BinderValue
 * @see StringValue
 */
	public QueryResult operatorAS(QueryResult r) throws TypeMismatchException {
    return new BinderValue(((StringValue)r).getValue(), this);
	}


//******************************************************************************
//****************************** BINARY OPERATORS ******************************
//******************************************************************************
/**
 * Method responsible for binary comma operation.
 * @param r the right operand of the binary comma operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary comma operation.
 * @see QueryResult
 * @see SequenceValue
 * @see SingleValue
 * @see BagValue
 */
	public QueryResult operatorCOMMA(QueryResult r) throws TypeMismatchException {
    if (r instanceof SingleValue)
      return this.addResult(r);
    else if (r instanceof StructValue)
      return this.addStruct(r);
    else if (r instanceof BagValue) {
      BagValue res = new BagValue();
      for (int i=0; i<((BagValue)r).getSize(); i++)
        res.addResult(((new StructValue()).addStruct(this)).addStruct(((BagValue)r).getResultAt(i)));
      return res;
    } else if (r instanceof SequenceValue) { // TODO: ma nie dzialac?
      BagValue res = new BagValue();
      for (int i=0; i<((SequenceValue)r).getSize(); i++)
        res.addResult(((new StructValue()).addStruct(this)).addStruct(((SequenceValue)r).getResultAt(i)));
      return res;
    } else
      throw new TypeMismatchException(this, "operatorCOMMA", r);
	}

/**
 * Method responsible for binary union operation.
 * @param r the right operand of the binary union operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary union operation.
 * @see QueryResult
 * @see SequenceValue
 * @see SingleValue
 * @see BagValue
 */
	public QueryResult operatorUNION(QueryResult r) throws TypeMismatchException {
    if (r instanceof SingleValue || r instanceof StructValue)
      return new BagValue(this, r);
    else if (r instanceof BagValue)
      return (new BagValue(this)).addBag(r);
    else if (r instanceof SequenceValue)
      return (new BagValue(this)).addBag(r.operatorTOBAG());
    else
      throw new TypeMismatchException(this, "operatorUNION", r);
	}

/**
 * Method responsible for binary compare operation.
 * @param r the right operand of the binary compare operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary compare operation.
 * @see QueryResult
 * @see BooleanValue
 */
	public BooleanValue operatorCOMPARE(QueryResult r) throws TypeMismatchException {
    if (r instanceof StructValue) {
      if (getSize() == ((StructValue)r).getSize()) {
        for (int i=0; i<getSize(); i++)
          if (getResultAt(i).operatorDEREFERENCE().operatorCOMPARE(((StructValue)r).getResultAt(i).operatorDEREFERENCE()).booleanValue() == false)
            return new BooleanValue(false);
        return new BooleanValue(true);
      } else return new BooleanValue(false);
    } else 
      return new BooleanValue(false);
	}

/**
 * Method responsible for binary inclusion operation.
 * @param r the right operand of the binary inclusion operator.
 * @exception TypeMismatchException if type mismatch occurs.
 * @return the result of the binary inclusion operation.
 * @see QueryResult
 * @see SequenceValue
 * @see BooleanValue
 * @see BagValue
 * @see ComplexValue
 */
	public BooleanValue operatorINCLUSION(QueryResult r) throws TypeMismatchException {
    if (r instanceof BagValue || r instanceof SequenceValue) {
      for (int i=0; i<((ComplexValue)r).getSize(); i++) {
        if (this.operatorCOMPARE(((ComplexValue)r).getResultAt(i).operatorDEREFERENCE()).booleanValue() == true)
          return new BooleanValue(true);
      }
      return new BooleanValue(false);
    } else
      throw new TypeMismatchException(this, "operatorINCLUSION", r);
	}

}