package pl.edu.pjwstk.yaod.yaql.visitors;

import java.io.*;
import java.text.*;
import java.util.*;
import pl.edu.pjwstk.yaod.objects.*;
import pl.edu.pjwstk.yaod.system.*;
import pl.edu.pjwstk.yaod.system.processes.*;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.*;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.*;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.binary.*;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.single.*;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.ternary.*;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.unary.*;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.statements.*;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.terminals.*;
import pl.edu.pjwstk.yaod.yaql.engine.*;
import pl.edu.pjwstk.yaod.yaql.engine.queryresulttypes.*;

/**
 * The <code>EvalVisitor</code> class represents one of the interpratations 
 * of the program - in this case evaluataion. It consists of 
 * many <code>visit</code> methods - each for one main node in abstract syntax 
 * tree. 
 * @see AbstractSyntaxTree
 * @see Visitor
 * @see EvalException
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public class EvalVisitor implements Visitor {
  QueryResultStack QRES;
  EnvironmentalStack ENVS;
  PrintWriter out;
  SVRP SP;
  Vector results;

/**
 * Creates a new evaluator with specified environmental stack, query result stack,
 * abstract syntax tree, server process, writer and vector of results.
 * @param wr an object responsible for displaying information.
 * @param QRES a query result stack.
 * @param ENVS an environmental stack.
 * @param SP a link to data store (server process).
 * @param results vector of results of evaluation.
 * @see QueryResultStack
 * @see EnvironmentalStack
 * @see SVRP
 */
  public EvalVisitor(PrintWriter wr, QueryResultStack QRES, 
                             EnvironmentalStack ENVS, SVRP SP, Vector results) {
    this.QRES = QRES;
    this.ENVS = ENVS;
    this.SP = SP;
    this.out = wr;
    this.results = results;
  }

/**
 * Returns the entry of the object store.
 * @see ObjectID
 * @see SVRP
 * @return entry of the object store.
 */
  public ObjectID getEntry() {
    return SP.getStoreByID(0).getEntry();
  }

/**
 * Returns the object manager of the object store.
 * @see ObjectManager
 * @see SVRP
 * @return object manager of the object store.
 */
  public ObjectManager getObjectManager() {
    return SP.getObjectManager();
  }


// EXPRESSIONS
/**
 * Method responsible for traversing certain single node in abstract syntax tree.
 * @param n is a certain single node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see SingleExpression
 */
  public void visit(SingleExpression n) throws EvalException {
    try {
      QRES.push(  compute(n)  );
    } catch (TypeMismatchException e) {
      throw new TypeMismatchException(n.getSpellingAndPosition()+" "+e.getFullMessage());
    }
  }

/**
 * Method responsible for traversing certain unary node in abstract syntax tree.
 * @param n is a certain unary node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see UnaryExpression
 * @see ParametricExpression
 */
  public void visit(ParametricExpression n) throws EvalException { // UNARY
    QueryResult right_res;
    n.r.accept(this);
    right_res = QRES.pop();
    try {
      QRES.push(  compute(n, right_res, new StringValue(n.i.getSpelling())  ));
    } catch (TypeMismatchException e) {
      throw new TypeMismatchException(n.getSpellingAndPosition()+" "+e.getFullMessage());
    }
  }

/**
 * Method responsible for traversing certain unary node in abstract syntax tree.
 * @param n is a certain unary node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see UnaryExpression
 * @see NonParametricExpression
 */
  public void visit(NonParametricExpression n) throws EvalException { // UNARY
   if (n instanceof ProcedureCallExpression) {
     QRES.push( compute(n, null) );
   } else {
    QueryResult right_res;
    n.r.accept(this);
    right_res = QRES.pop();
    try {
      QRES.push(  compute(n, right_res)  );
    } catch (TypeMismatchException e) {
      throw new TypeMismatchException(n.getSpellingAndPosition()+" "+e.getFullMessage());
    }
   }
  }

/**
 * Method responsible for traversing certain binary node in abstract syntax tree.
 * @param n is a certain binary node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see BinaryExpression
 * @see AlgebraicExpression
 */
  public void visit(AlgebraicExpression n) throws EvalException { // BINARY
    QueryResult left_res, right_res;
    n.l.accept(this);
    n.r.accept(this);
    right_res = QRES.pop();
    left_res = QRES.pop();
    try {
      QRES.push(  compute(n, left_res, right_res)  );
    } catch (TypeMismatchException e) {
      throw new TypeMismatchException(n.getSpellingAndPosition()+" "+e.getFullMessage());
    }
  }

/**
 * Method responsible for traversing certain binary node in abstract syntax tree.
 * @param n is a certain binary node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see BinaryExpression
 * @see NonAlgebraicExpression
 */
  public void visit(NonAlgebraicExpression n) throws EvalException { // BINARY
    BagValue partial_results = new BagValue();
    n.l.accept(this);
    for (int i=0; i<QRES.top().operatorTOSINGLE().getNumberOfResults(); i++) {
      QueryResult e = QRES.top().operatorTOSINGLE().getNextResultAt(i);
      ENVS.push(e.nested());
      n.r.accept(this);
      try {
        partial_results.addResult( combine(n, e, QRES.top().operatorTOSINGLE()) );
      } catch (TypeMismatchException e1) {
        throw new TypeMismatchException(n.getSpellingAndPosition()+" "+e1.getFullMessage());
      }
      QRES.pop();
      ENVS.pop();
    }
    QueryResult final_result = null;
    try {
      final_result = merge(n, partial_results);
    } catch (TypeMismatchException e2) {
      throw new TypeMismatchException(n.getSpellingAndPosition()+" "+e2.getFullMessage());
    }
    QRES.pop();
    QRES.push(final_result);  
  }

/**
 * Method responsible for traversing certain ternary node in abstract syntax tree.
 * @param n is a certain ternary node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see TernaryExpression
 */
   public void visit(TernaryExpression n) throws EvalException {
    QueryResult left_res, middle_res, right_res;
    n.l.accept(this);
    n.m.accept(this);
    n.r.accept(this);
    right_res = QRES.pop();
    middle_res = QRES.pop();
    left_res = QRES.pop();
    try {
       QRES.push(  compute(n, left_res, middle_res, right_res)  );
    } catch (TypeMismatchException e) {
      throw new TypeMismatchException(n.getSpellingAndPosition()+" "+e.getFullMessage());
    }
  }



// STATEMENTS
/**
 * Method responsible for traversing node representing empty statement 
 * (<code> ;</code>) in abstract syntax tree.
 * @param n is a empty statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see EmptyStatement
 */
  public void visit(EmptyStatement n) throws EvalException {
    // nothing
  }

/**
 * Method responsible for traversing node representing expression statement 
 * (<code>... ;</code>)in abstract syntax tree.
 * @param n is a expression statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see ExpressionStatement
 */
  public void visit(ExpressionStatement n) throws EvalException {
    n.e.accept(this);
// zakomentowac w celu ogladniecia QRES po kazdym prostym wyrazeniu (zapytaniu)
// TODO: powinien dzialac jak PRINT
    QueryResult res = QRES.pop().operatorMIXEDDEREFERENCE();
    res.operatorPRINT(out);
    results.add(res);
  }

/**
 * Method responsible for traversing node representing print statement 
 * (<code>print (...) ;</code>) in abstract syntax tree.
 * @param n is a print statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see PrintStatement
 */
  public void visit(PrintStatement n) throws EvalException {
    n.e.accept(this);
    QRES.pop().operatorTOSINGLE().operatorMIXEDDEREFERENCE().operatorPRINT(out);
  }

/**
 * Method responsible for traversing node representing xmlprint statement 
 * (<code>xmlprint (...) ;</code>) in abstract syntax tree.
 * @param n is a xmlprint statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see XmlPrintStatement
 */
  public void visit(XmlPrintStatement n) throws EvalException {
    n.e.accept(this);
    QRES.pop().operatorTOSINGLE().operatorSPECIALDEREFERENCE().operatorXMLPRINT(out);
  }

/**
 * Method responsible for traversing node representing show statement 
 * (<code>show (...) ;</code>) in abstract syntax tree.
 * @param n is a show statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see ShowStatement
 */
  public void visit(ShowStatement n) throws EvalException {
    n.e.accept(this);
    QRES.pop().operatorTOSINGLE().show(out);
  }

/**
 * Method responsible for traversing node representing block statement 
 * (<code>{ ... }</code>) in abstract syntax tree.
 * @param n is a block statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see BlockStatement
 */
  public void visit(BlockStatement n) throws EvalException {
    // open section on ENVS
    ENVS.push(new ActivationRecord());
    // execute instructions
    n.s.accept(this);
    // delete all local object
    ENVS.deleteAtTop();
    // close section on ENVS
    ENVS.pop();
  }

/**
 * Method responsible for traversing node representing empty block statement 
 * (<code>{   }</code>) in abstract syntax tree.
 * @param n is a empty block statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see EmptyBlockStatement
 */
  public void visit(EmptyBlockStatement n) throws EvalException {
    // nothing
  }

/**
 * Method responsible for traversing node representing if statement 
 * (<code>if (...) ...</code>) in abstract syntax tree.
 * @param n is a if statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see IfStatement
 */
  public void visit(IfStatement n) throws EvalException {
    boolean res = true;
    n.e.accept(this);
    try {
      res = QRES.pop().operatorEQUAL(new BooleanValue(true)).booleanValue();
    } catch (TypeMismatchException e) {
      throw new TypeMismatchException(n.getSpellingAndPosition()+" "+e.getFullMessage());
    }
    if (res == true)
      n.s.accept(this);
  }

/**
 * Method responsible for traversing node representing if else statement 
 * (<code>if (...) ... else ...</code>) in abstract syntax tree.
 * @param n is a if else statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see IfElseStatement
 */
  public void visit(IfElseStatement n) throws EvalException {
    boolean res = true;
    n.e.accept(this);
    try {
      res = QRES.pop().operatorEQUAL(new BooleanValue(true)).booleanValue();
    } catch (TypeMismatchException e) {
      throw new TypeMismatchException(n.getSpellingAndPosition()+" "+e.getFullMessage());
    }
    if (res == true)
      n.s1.accept(this);
    else
      n.s2.accept(this);
  }

/**
 * Method responsible for traversing node representing while statement 
 * (<code>while (...) ...</code>) in abstract syntax tree.
 * @param n is a while statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see WhileStatement
 */
  public void visit(WhileStatement n) throws EvalException {
    n.e.accept(this);
    boolean res = true;
    try {
      res = QRES.pop().operatorEQUAL(new BooleanValue(true)).booleanValue();
    } catch (TypeMismatchException e) {
      throw new TypeMismatchException(n.getSpellingAndPosition()+" "+e.getFullMessage());
    }
    while (res == true) {
      try {
        n.s.accept(this);
      } catch (BreakException e) { break; }
      n.e.accept(this);
      try {
        res = QRES.pop().operatorEQUAL(new BooleanValue(true)).booleanValue();
      } catch (TypeMismatchException e) {
        throw new TypeMismatchException(n.getSpellingAndPosition()+" "+e.getFullMessage());
      }
    }
  }

/**
 * Method responsible for traversing node representing for statement 
 * (<code>for (...; ...; ...) ...</code>) in abstract syntax tree.
 * @param n is a for statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see ForStatement
 */
  public void visit(ForStatement n) throws EvalException {
    n.e1.accept(this);
    QRES.pop();
    n.e2.accept(this);
    boolean res = true;
    try {
      res = QRES.pop().operatorEQUAL(new BooleanValue(true)).booleanValue();
    } catch (TypeMismatchException e) {
      throw new TypeMismatchException(n.getSpellingAndPosition()+" "+e.getFullMessage());
    }
    while (res == true) {
      try {
        n.s.accept(this);
      } catch (BreakException e) { break; }
      n.e3.accept(this);
      QRES.pop();
      n.e2.accept(this);
      try {
        res = QRES.pop().operatorEQUAL(new BooleanValue(true)).booleanValue();
      } catch (TypeMismatchException e) {
        throw new TypeMismatchException(n.getSpellingAndPosition()+" "+e.getFullMessage());
      }
    }
  }

/**
 * Method responsible for traversing node representing do while statement 
 * (<code>do ... while (...) ;</code>) in abstract syntax tree.
 * @param n is a do while statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see DoWhileStatement
 */
  public void visit(DoWhileStatement n) throws EvalException {
    boolean res = true;
    do {
      try {
        n.s.accept(this);
      } catch (BreakException e) { break; }
      n.e.accept(this);
      try {
        res = QRES.pop().operatorEQUAL(new BooleanValue(true)).booleanValue();
      } catch (TypeMismatchException e) {
        throw new TypeMismatchException(n.getSpellingAndPosition()+" "+e.getFullMessage());
      }
    } while (res == true);
  }

/**
 * Method responsible for traversing node representing for each statement 
 * (<code>for each ... do ...</code>) in abstract syntax tree.
 * @param n is a for each statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see ForEachStatement
 */
  public void visit(ForEachStatement n) throws EvalException {
    n.e.accept(this);
    for (int i=0; i<QRES.top().getNumberOfResults(); i++) {
      QueryResult e = QRES.top().getNextResultAt(i);
      ENVS.push(e.nested());
      try {
        n.s.accept(this);
      } catch (BreakException etmp) { break; }
      ENVS.pop();
    }
    QRES.pop();
  }

/**
 * Method responsible for traversing node representing delete statement 
 * (<code>delet ... ;</code>) in abstract syntax tree.
 * @param n is a delete statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see DeleteStatement
 */
  public void visit(DeleteStatement n) throws EvalException {
    n.e.accept(this);
    QRES.pop().operatorTOSINGLE().operatorDELETE(ENVS);
  }

/**
 * Method responsible for traversing node representing rename to statement 
 * (<code>rename ... to ... ;</code>) in abstract syntax tree.
 * @param n is a rename to statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see RenameToStatement
 */
  public void visit(RenameToStatement n) throws EvalException {
    n.e.accept(this);
    QueryResult l = QRES.pop().operatorTOSINGLE();
    QueryResult r = new StringValue(n.i.getSpelling());
    try {
      l.operatorRENAMETO(r, ENVS);
    } catch (TypeMismatchException e) {
      throw new TypeMismatchException(n.getSpellingAndPosition()+" "+e.getFullMessage());
    }
  }

/**
 * Method responsible for traversing node representing insert into statement 
 * (<code>insert ... into ... ;</code>) in abstract syntax tree.
 * @param n is a insert into statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see InsertIntoStatement
 */
  public void visit(InsertIntoStatement n) throws EvalException {
    n.e1.accept(this);
    n.e2.accept(this);
    QueryResult r = QRES.pop().operatorTOSINGLE();
    QueryResult l = QRES.pop().operatorTOSINGLE();
    try {
      l.operatorINSERTINTO(r, ENVS);
    } catch (TypeMismatchException e) {
      throw new TypeMismatchException(n.getSpellingAndPosition()+" "+e.getFullMessage());
    }
  }

/**
 * Method responsible for traversing node representing update to statement 
 * (<code>update ... to ... ;</code>)in abstract syntax tree.
 * @param n is a update to statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see UpdateToStatement
 */
  public void visit(UpdateToStatement n) throws EvalException {
    n.e1.accept(this);
    n.e2.accept(this);
    QueryResult r = QRES.pop().operatorTOSINGLE().operatorDEREFERENCE();
    QueryResult l = QRES.pop().operatorTOSINGLE();
    try {
      l.operatorUPDATETO(r);
    } catch (TypeMismatchException e) {
      throw new TypeMismatchException(n.getSpellingAndPosition()+" "+e.getFullMessage());
    }
  }

/**
 * Method responsible for traversing node representing empty procedure
 * declaration statement (procedure without parameters) 
 * (<code>procedure/function id() { ... }</code>) in abstract syntax tree.
 * @param n is a empty procedure declaration statement node in abstract 
 * syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see EmptyProcedureDeclarationStatement
 */
  public void visit(EmptyProcedureDeclarationStatement n) throws EvalException {
   try {
    YProcedure proc = SP.getObjectManager().createProcedure(n.i.getSpelling());
    proc.connect(this.getEntry());
    proc.setBody(n.s);
//    proc.setSource(n.getSource());
    ENVS.addAtBottom(new BinderValue(proc.getName(), new IdentifierValue(proc.getOID(), getObjectManager())));
   } catch (NoSuchObjectException e) { e.printStackTrace(); 
//   } catch (BlockFreeSpaceException e) { e.printStackTrace(); }
   } catch (Exception e) { e.printStackTrace(); }
//    proc.setSource(n.getSource());
  }

/**
 * Method responsible for traversing node representing procedure declaration
 * statement (<code>procedure/function id(...) { ... }</code>) in abstract
 * syntax tree.
 * @param n is a procedure declaration statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see ProcedureDeclarationStatement
 */
  public void visit(ProcedureDeclarationStatement n) throws EvalException {
   try {
    YProcedure proc = SP.getObjectManager().createProcedure(n.i.getSpelling());
    proc.connect(this.getEntry());
    proc.setBody(n.s);
    proc.setParameters(n.p);
//    proc.setSource(n.getSource());
    ENVS.addAtBottom(new BinderValue(proc.getName(), new IdentifierValue(proc.getOID(), getObjectManager())));
   } catch (NoSuchObjectException e) { e.printStackTrace(); 
//   } catch (BlockFreeSpaceException e) { e.printStackTrace(); }
   } catch (Exception e) { e.printStackTrace(); }
  }

/**
 * Method responsible for traversing node representing break statement 
 * (<code>break ;</code>)in abstract syntax tree.
 * @param n is a break statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see BreakStatement
 */
  public void visit(BreakStatement n) throws EvalException {
    throw new BreakException();
  }

/**
 * Method responsible for traversing node representing return statement 
 * (<code>return ...;</code>) in abstract syntax tree.
 * @param n is a return statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see ReturnStatement
 */
  public void visit(ReturnStatement n) throws EvalException {
    n.e.accept(this);
    QueryResult tmp = QRES.pop();
    throw new ReturnException(tmp);
  }

/**
 * Method responsible for traversing node representing empty return statement 
 * (<code>return ;</code>) in abstract syntax tree.
 * @param n is a empty return statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see EmptyReturnStatement
 */
  public void visit(EmptyReturnStatement n) throws EvalException {
    throw new EmptyReturnException();    
  }

/**
 * Method responsible for traversing node representing  pair statement list
 * (<code>... ; ... ;</code>) in abstract syntax tree.
 * @param n is a pair statement list node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see PairStatementList
 */
  public void visit(PairStatementList n) throws EvalException { 
    n.head.accept(this); 
    n.tail.accept(this); 
  }

/**
 * Method responsible for traversing node representing last statement list
 * (<code>... ;</code>) in abstract syntax tree.
 * @param n is a last statement list node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see LastStatementList
 */
  public void visit(LastStatementList n) throws EvalException { 
    n.tail.accept(this); 
  }





// COMPUTE
/**
 * Auxilary method responsible for evaluating single expression nodes.
 * @param n is a certain single expression node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see SingleExpression
 */
  public QueryResult compute(SingleExpression n) throws EvalException {
    if (n instanceof IdentifierExpression) {
      return ENVS.bind(((IdentifierExpression)n).id.getSpelling());
    } else if (n instanceof BooleanExpression) {
      return new BooleanValue((Boolean)((BooleanExpression)n).lit.getValue());
    } else if (n instanceof IntegerExpression) {
      return new IntegerValue((Integer)((IntegerExpression)n).lit.getValue());
    } else if (n instanceof DoubleExpression) {
      return new DoubleValue((Double)((DoubleExpression)n).lit.getValue());
    } else if (n instanceof StringExpression) {
      return new StringValue((String)((StringExpression)n).lit.getValue());
    } else if (n instanceof EmptyStructExpression) {
      return new StructValue();
    } else if (n instanceof EmptyBagExpression) {
      return new BagValue();
    } else if (n instanceof EmptySequenceExpression) {
      return new SequenceValue();
    } else if (n instanceof EmptyProcedureCallExpression) {
     try {
      // bind zwraca bag identyfikatorow
      BagValue bag = ENVS.bind(((EmptyProcedureCallExpression)n).i.getSpelling());
      for (int i=0; i<bag.getSize(); i++) {
        IdentifierValue iv = (IdentifierValue)bag.getResultAt(i);
        YObject obj = SP.getObjectManager().getObject(iv.getId());
        if (obj instanceof YProcedure) {
          ENVS.push(new ActivationRecord());
          AbstractSyntaxTree body = ((YProcedure)obj).getBody();
          QueryResult result;
          try {
            body.accept(this);
            result = new BagValue();
          } catch (EmptyReturnException e) { result = new BagValue();
          } catch (ReturnException e) {      result = e.getReturnValue(); }
          ENVS.deleteAtTop();
          ENVS.pop();
          return result;
        } else
          throw new TypeMismatchException(iv, "ProcedureCALL - id of non Procedure !!!");
      }
     } catch (NoSuchObjectException e) { e.printStackTrace(); }
    } else if (n instanceof AsteriskExpression) {
      BagValue bag = new BagValue();
      ActivationRecord ar = ENVS.getLastElement();
      for (int i=0; i<ar.getSize(); i++)
        if (!ar.getBinderAt(i).getName().equals("this"))
          bag.addResult(ar.getBinderAt(i).getValue());
      return bag;
    }
    throw new EvalException("EvalVisitor.compute("+n.getTypeAsString()+")");
  }

/**
 * Auxilary method responsible for evaluating unary expression nodes.
 * @param n is a certain unary expression node in abstract syntax tree.
 * @param l is a left operand.
 * @param r is a right operand.
 * @exception EvalException if evaluation in this node fails.
 * @see UnaryExpression
 * @see ParametricExpression
 * @see QueryResult
 */
  public QueryResult compute(ParametricExpression n, QueryResult l, QueryResult r) throws EvalException {
    l = l.operatorTOSINGLE();
    r = r.operatorTOSINGLE();
    if (n instanceof AsExpression) {                   // E as ID
      return l.operatorAS(r);
    } else if (n instanceof GroupAsExpression) {       // E group as ID
      return l.operatorGROUPAS(r);
    } else if (n instanceof EmptyAsExpression) {       // ID ( E ) 
      return l.operatorAS(r);
    } else if (n instanceof EmptyGroupAsExpression) {  // ID { E } 
      return l.operatorGROUPAS(r);
    }
    throw new EvalException("EvalVisitor.compute("+n.getTypeAsString()+")");
  }

/**
 * Auxilary method responsible for evaluating unary expression nodes.
 * @param n is a certain unary expression node in abstract syntax tree.
 * @param r is a right operand.
 * @exception EvalException if evaluation in this node fails.
 * @see UnaryExpression
 * @see NonParametricExpression
 * @see QueryResult
 */
  public QueryResult compute(NonParametricExpression n, QueryResult r) throws EvalException {
    if (n instanceof UMinusExpression) {                 // -E
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return r.operatorMINUS();
    } else if (n instanceof UPlusExpression) {           // +E
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return r.operatorPLUS();
    } else if (n instanceof PreIncrementExpression) {    // ++E
      r = r.operatorTOSINGLE();
      return r.operatorPREINCREMENT();
    } else if (n instanceof PreDecrementExpression) {    // --E
      r = r.operatorTOSINGLE();
      return r.operatorPREDECREMENT();
    } else if (n instanceof DereferenceExpression) {     // deref(E)
      return r.operatorTOSINGLE().operatorDEREFERENCE();
    } else if (n instanceof NegationExpression) {        // !E
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return r.operatorNEGATION();
    } else if (n instanceof SquareRootExpression) {      // sqrt(E)
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return r.operatorSQUAREROOT();
    } else if (n instanceof AbsoluteValueExpression) {   // abs(E)
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return r.operatorABSOLUTEVALUE();
    } else if (n instanceof SineExpression) {            // sin(E)
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return r.operatorSINE();
    } else if (n instanceof CosineExpression) {          // cos(E)
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return r.operatorCOSINE();
    } else if (n instanceof ExponentExpression) {        // exp(E)
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return r.operatorEXPONENT();
    } else if (n instanceof CountExpression) {           // count(E)
      return r.operatorCOUNT();
    } else if (n instanceof LogarithmExpression) {       // log(E)
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return r.operatorLOGARITHM();
    } else if (n instanceof ExistsExpression) {          // exists E
      r = (r.operatorTOSINGLE());
      return r.operatorEXISTS();
    } else if (n instanceof SumExpression) {             // sum(E)
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return r.operatorSUM();
    } else if (n instanceof MinimumExpression) {         // min(E)
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return r.operatorMINIMUM();
    } else if (n instanceof MaximumExpression) {         // max(E)
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return r.operatorMAXIMUM();
    } else if (n instanceof AverageExpression) {         // avg(E)
      return (r.operatorDEREFERENCE()).operatorAVERAGE();
    } else if (n instanceof SignumExpression) {          // sign(E)
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return r.operatorSIGNUM();
    } else if (n instanceof StructExpression) {          // struct(E)
      r = r.operatorTOSINGLE(); // TODO: struct z bagiem i sequence
      return r.operatorSTRUCT();
    } else if (n instanceof SequenceExpression) {        // sequence(E)
      return r.operatorSEQUENCE();
    } else if (n instanceof BagExpression) {             // bag(E)
      return r.operatorBAG();
    } else if (n instanceof ProcedureCallExpression) {   // id(E)
     try {
      // bind zwraca bag identyfikatorow
      BagValue bag = ENVS.bind(((ProcedureCallExpression)n).i.getSpelling());
      for (int j=0; j<bag.getSize(); j++) {
        IdentifierValue iv = (IdentifierValue)bag.getResultAt(j);
        YObject obj = SP.getObjectManager().getObject(iv.getId());
        if (obj instanceof YProcedure) {
          FormalParametersList params = (FormalParametersList)((YProcedure)obj).getParameters();
          AbstractSyntaxTree body = ((YProcedure)obj).getBody();
          Vector formalParameters = new Vector();
          Vector actualParameters = new Vector();
          int fp = params.getNumberOfParameters(formalParameters);
          int ac = ((ProcedureCallExpression)n).getNumberOfParameters(actualParameters);
          if (fp == ac) {
            ENVS.push(new ActivationRecord());
            for (int i=0; i<fp; i++) {
              ((Expression)actualParameters.get(i)).accept(this);
              BinderValue bv = new BinderValue(
                      ((IdentifierTerminal)formalParameters.get(i)).getSpelling(),
                      QRES.pop().operatorTOSINGLE());
              ENVS.addAtTop(bv);
            }
            QueryResult result;
            try {
              body.accept(this);
              result = new BagValue();
            } catch (EmptyReturnException e) { result = new BagValue();
            } catch (ReturnException e) {      result = e.getReturnValue(); }
            ENVS.deleteAtTop();
            ENVS.pop();
            return result;
          }
        } else
          throw new TypeMismatchException(iv, "ProcedureCALL - id of non YProcedure !!! ");
      }
     } catch (NoSuchObjectException e) { e.printStackTrace(); }
    } else if (n instanceof UniqueExpression) {          // unique(E)
      r = (r.operatorTOSINGLE());
      return r.operatorUNIQUE();
    } else if (n instanceof AscendingExpression) {       // asc(E)
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return r.operatorASCENDING();
    } else if (n instanceof DescendingExpression) {      // desc(E)
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return r.operatorDESCENDING();
    } else if (n instanceof ToIntegerExpression) {       // toInteger(E)
      r = r.operatorTOSINGLE().operatorDEREFERENCE();
      return r.operatorTOINTEGER();
    } else if (n instanceof ToDoubleExpression) {        // toDouble(E)
      r = r.operatorTOSINGLE().operatorDEREFERENCE();
      return r.operatorTODOUBLE();
    } else if (n instanceof ToStringExpression) {        // toString(E)
      r = r.operatorTOSINGLE().operatorDEREFERENCE();
      return r.operatorTOSTRING();
    } else if (n instanceof ToStructExpression) {        // toStruct(E)
      return r.operatorTOSTRUCT();
    } else if (n instanceof ToSequenceExpression) {      // toSequence(E)
      return r.operatorTOSEQUENCE();
    } else if (n instanceof ToBagExpression) {           // toBag(E)
      return r.operatorTOBAG();
    } else if (n instanceof ToSingleExpression) {        // toSingle(E)
      return r.operatorTOSINGLE();
    } else if (n instanceof CreateLocalExpression) {     // create local E 
      QueryResult res = r.operatorCREATELOCAL(new IdentifierValue(getEntry(), getObjectManager()));  
      res.operatorCREATELOCAL(ENVS);
      return res;
    } else if (n instanceof CreatePermanentExpression) { // create permanent E 
      QueryResult res = r.operatorCREATEPERMANENT(new IdentifierValue(getEntry(), getObjectManager()));  
      res.operatorCREATEPERMANENT(ENVS);
      return res;
    } else if (n instanceof CreateExpression) {          // create E 
      QueryResult res = r.operatorCREATE(new IdentifierValue(getEntry(), getObjectManager()));  
      res.operatorCREATE(ENVS);
      return res;
    }
    throw new EvalException("EvalVisitor.compute("+n.getTypeAsString()+")");
  }

/**
 * Auxilary method responsible for evaluating binary expression nodes.
 * @param n is a certain binary expression node in abstract syntax tree.
 * @param l is a left operand.
 * @param r is a right operand.
 * @exception EvalException if evaluation in this node fails.
 * @see BinaryExpression
 * @see AlgebraicExpression
 * @see QueryResult
 */
  public QueryResult compute(AlgebraicExpression n, QueryResult l, QueryResult r) throws EvalException {
    if (n instanceof PlusExpression) {                       // E + E
      l = (l.operatorTOSINGLE()).operatorDEREFERENCE();
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return l.operatorPLUS(r);
    } else if (n instanceof MinusExpression) {               // E - E
      l = (l.operatorTOSINGLE()).operatorDEREFERENCE();
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return l.operatorMINUS(r);
    } else if (n instanceof TimesExpression) {               // E * E
      l = (l.operatorTOSINGLE()).operatorDEREFERENCE();
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return l.operatorTIMES(r);
    } else if (n instanceof DivideExpression) {              // E / E
      l = (l.operatorTOSINGLE()).operatorDEREFERENCE();
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return l.operatorDIVIDE(r);
    } else if (n instanceof EqualExpression) {               // E = E
      l = (l.operatorTOSINGLE()).operatorDEREFERENCE();
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return l.operatorEQUAL(r);
    } else if (n instanceof NotEqualExpression) {            // E != E
      l = (l.operatorTOSINGLE()).operatorDEREFERENCE();
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return l.operatorNOTEQUAL(r);
    } else if (n instanceof GreaterExpression) {             // E > E
      l = (l.operatorTOSINGLE()).operatorDEREFERENCE();
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return l.operatorGREATER(r);
    } else if (n instanceof GreaterEqualExpression) {        // E >= E
      l = (l.operatorTOSINGLE()).operatorDEREFERENCE();
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return l.operatorGREATEREQUAL(r);
    } else if (n instanceof LowerExpression) {               // E < E
      l = (l.operatorTOSINGLE()).operatorDEREFERENCE();
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return l.operatorLOWER(r);
    } else if (n instanceof LowerEqualExpression) {          // E <= E
      l = (l.operatorTOSINGLE()).operatorDEREFERENCE();
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return l.operatorLOWEREQUAL(r);
    } else if (n instanceof ConjunctionExpression) {         // E && E
      l = (l.operatorTOSINGLE()).operatorDEREFERENCE();
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return l.operatorCONJUNCTION(r);
    } else if (n instanceof DisjunctionExpression) {         // E || E
      l = (l.operatorTOSINGLE()).operatorDEREFERENCE();
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return l.operatorDISJUNCTION(r);
    } else if (n instanceof PowerExpression) {               // E ^ E
      l = (l.operatorTOSINGLE()).operatorDEREFERENCE();
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return l.operatorPOWER(r);
    } else if (n instanceof ModuloExpression) {              // E % E
      l = (l.operatorTOSINGLE()).operatorDEREFERENCE();
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return l.operatorMODULO(r);
    } else if (n instanceof CommaExpression) {               // E, E
      return l.operatorCOMMA(r);
    } else if (n instanceof UnionExpression) {               // E union E
      return l.operatorUNION(r);
    } else if (n instanceof IntersectionExpression) {        // E intersect E
      return l.operatorINTERSECTION(r);
    } else if (n instanceof DifferenceExpression) {          // E minus E
      return l.operatorDIFFERENCE(r);
    } else if (n instanceof SymmetricDifferenceExpression) { // E symmetric minus E
      return l.operatorSYMMETRICDIFFERENCE(r);
    } else if (n instanceof InclusionExpression) {           // E in E
      l = l.operatorTOSINGLE().operatorDEREFERENCE();
      return l.operatorINCLUSION(r);
    } else if (n instanceof CompareExpression) {             // E compare E
      return l.operatorCOMPARE(r);
    } else if (n instanceof BRangeExpression) {              // E[E]
      l = l.operatorDEREFERENCE();
      r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
      return l.operatorRANGE(r);
    } else if (n instanceof AssignExpression) {              // E = E
      l = l.operatorTOSINGLE();
      r = r.operatorTOSINGLE().operatorDEREFERENCE();
      return l.operatorASSIGN(r);
    } else if (n instanceof InsertExpression) {              // E <- E
      l = l.operatorTOSINGLE();
      r = r.operatorTOSINGLE();
      return l.operatorINSERT(r, ENVS); 
    } else if (n instanceof PlusAssignExpression) {          // E += E
      l = l.operatorTOSINGLE();
      r = r.operatorTOSINGLE().operatorDEREFERENCE();
      return l.operatorPLUSASSIGN(r);
    } else if (n instanceof MinusAssignExpression) {         // E -= E
      l = l.operatorTOSINGLE();
      r = r.operatorTOSINGLE().operatorDEREFERENCE();
      return l.operatorMINUSASSIGN(r);
    } else if (n instanceof TimesAssignExpression) {         // E *= E
      l = l.operatorTOSINGLE();
      r = r.operatorTOSINGLE().operatorDEREFERENCE();
      return l.operatorTIMESASSIGN(r);
    } else if (n instanceof DivideAssignExpression) {        // E /= E
      l = l.operatorTOSINGLE();
      r = r.operatorTOSINGLE().operatorDEREFERENCE();
      return l.operatorDIVIDEASSIGN(r);
    } else if (n instanceof ModuloAssignExpression) {        // E %= E
      l = l.operatorTOSINGLE();
      r = r.operatorTOSINGLE().operatorDEREFERENCE();
      return l.operatorMODULOASSIGN(r);
    } 
    throw new EvalException("EvalVisitor.compute("+n.getTypeAsString()+")");
  }

/**
 * Auxilary method responsible for evaluating ternary expression nodes.
 * @param n is a certain ternary expression node in abstract syntax tree.
 * @param l is a left operand.
 * @param m is a middle operand.
 * @param r is a right operand.
 * @exception EvalException if evaluation in this node fails.
 * @see TernaryExpression
 * @see QueryResult
 */
  public QueryResult compute(TernaryExpression n, QueryResult l, QueryResult m, QueryResult r) throws EvalException {
    m = (m.operatorTOSINGLE()).operatorDEREFERENCE();
    r = (r.operatorTOSINGLE()).operatorDEREFERENCE();
    if (n instanceof ConditionExpression) {            // E ? E : E
      l = (l.operatorTOSINGLE()).operatorDEREFERENCE();
      return l.operatorCONDITION(m, r);
    } else if (n instanceof RangeExpression) {         // E[E .. E]
      l = l.operatorDEREFERENCE();
      return l.operatorRANGE(m, r);
    }
    throw new EvalException("EvalVisitor.compute("+n.getTypeAsString()+")");
  }

// COMBINE (for non algebraic BINARY)
/**
 * Auxilary method responsible for evaluating binary non algebraic expression nodes.
 * @param n is a certain binary expression node in abstract syntax tree.
 * @param e is one of the results of the left operand evaluation.
 * @param tmp is a the result of <code>nested(e)</code>.
 * @exception EvalException if evaluation in this node fails.
 * @see BinaryExpression
 * @see NonAlgebraicExpression
 * @see QueryResult
 */
  public QueryResult combine(NonAlgebraicExpression n, QueryResult e, QueryResult tmp) throws EvalException {
    QueryResult temporary = QRES.pop();
    QueryResult left = QRES.top();
    QRES.push(temporary);
    if (n instanceof DependentJoinExpression) {
      // zwraca bag(struct, struct, struct, ...) lub jeden struct
      // albo sequence(struct, struct, ...) ... splaszcza struktury !!
      if (e instanceof BagValue || e instanceof SequenceValue)
        throw new TypeMismatchException("DependentJoin left operand has bag or seq in one of its elements");

      if (tmp instanceof SingleValue)
        return (new StructValue()).addStruct(e).addResult(tmp);
      else if (tmp instanceof StructValue)
        return (new StructValue()).addStruct(e).addStruct(tmp);
      else if (tmp instanceof BagValue) {
//        if (left instanceof SequenceValue) throw new TypeMismatchException("Type mismatch: SEQUENCE JOIN BAG");
        BagValue res = new BagValue();
        for (int i=0; i<((BagValue)tmp).getSize(); i++)
          res.addResult((new StructValue()).addStruct(e).addStruct(((BagValue)tmp).getResultAt(i)));
        return res;
      } else if (tmp instanceof SequenceValue) {
//        if (left instanceof BagValue)  throw new TypeMismatchException("Type mismatch: BAG JOIN SEQUENCE");
        SequenceValue res = new SequenceValue();
        for (int i=0; i<((SequenceValue)tmp).getSize(); i++)
          res.addResult((new StructValue()).addStruct(e).addStruct(((SequenceValue)tmp).getResultAt(i)));
        return res;
      }
      throw new EvalException("EvalVisitor.combine("+n.getTypeAsString()+") [end of combine]");

    } else if (n instanceof ForAllExpression) {
      return tmp;

    } else if (n instanceof ForAnyExpression) {
      return tmp;

    } else if (n instanceof NavigationExpression) {
      return tmp;

    } else if (n instanceof SelectionExpression) {
      BagValue bag = new BagValue();
      boolean res = true;
      res = tmp.operatorEQUAL(new BooleanValue(true)).booleanValue();
      if (res == true)
        bag.addResult(e);
      return bag;

    // podobnie jak join
    } else if (n instanceof SortingExpression) {
      // zwraca jeden struct
      // nie splaszcza struktury lewego podzapytania!!
      if (e instanceof BagValue || e instanceof SequenceValue)
        throw new TypeMismatchException("Sorting left operand has bag or seq in one of its elements");
      if (tmp instanceof SingleValue) {
        return (new StructValue(e)).addResult(tmp.operatorDEREFERENCE());
      } else if (tmp instanceof StructValue) {
        ComplexValue res = (new StructValue(e)).addStruct(tmp);
        // plus dereferencja na kluczach (bez pierwszego elementu - wyniku)
        for (int i=1; i<res.getSize(); i++)
          res.setResultAt(i, res.getResultAt(i).operatorDEREFERENCE());
        return res;
      } else if (tmp instanceof BagValue) {
        BagValue res = new BagValue();
//        if (left instanceof SequenceValue) throw new RuntimeException("Type mismatch: SEQUENCE ORDER BY BAG");
        for (int i=0; i<((BagValue)tmp).getSize(); i++) {
          res.addResult( (new StructValue(e)).addStruct(((BagValue)tmp).getResultAt(i)));
          // plus dereferencja na kluczach (bez pierwszego elementu - wyniku)
          for (int j=1; j<((ComplexValue)res.getResultAt(i)).getSize(); j++)
            ((ComplexValue)res.getResultAt(i)).setResultAt(j, ((ComplexValue)res.getResultAt(i)).getResultAt(j).operatorDEREFERENCE());
        }
        return res;
      } else if (tmp instanceof SequenceValue) {
        SequenceValue res = new SequenceValue();
//        if (left instanceof BagValue) throw new RuntimeException("Type mismatch: BAG ORDER BY SEQUENCE");
        for (int i=0; i<((SequenceValue)tmp).getSize(); i++) {
          res.addResult( (new StructValue(e)).addStruct(((SequenceValue)tmp).getResultAt(i)));
          // plus dereferencja na kluczach (bez pierwszego elementu - wyniku)
          for (int j=1; j<((ComplexValue)res.getResultAt(i)).getSize(); j++)
            ((ComplexValue)res.getResultAt(i)).setResultAt(j, ((ComplexValue)res.getResultAt(i)).getResultAt(j).operatorDEREFERENCE());
        }
        return res;
      }
      throw new EvalException("EvalVisitor.combine("+n.getTypeAsString()+") [end of combine]");

    } else if (n instanceof TransitiveClosureExpression) {
      QueryResult tmp1 = QRES.pop();
      QueryResult T    = QRES.pop();
      T = T.operatorUNION(tmp);
      QRES.push(T);
      QRES.push(tmp1);
      return tmp;
    }
    throw new EvalException("EvalVisitor.combine("+n.getTypeAsString()+")");
  }




// MERGE (for non algebraic BINARY)
/**
 * Auxilary method responsible for evaluating binary non algebraic expression nodes.
 * @param n is a certain binary expression node in abstract syntax tree.
 * @param bag is the set of all partial results yielded by method <code>combine</code>.
 * @param tmp is a the result of <code>nested(e)</code>.
 * @exception EvalException if evaluation in this node fails.
 * @see BinaryExpression
 * @see NonAlgebraicExpression
 * @see BagValue
 */
   public QueryResult merge(NonAlgebraicExpression n, BagValue bag) throws EvalException {
    QueryResult left = QRES.top();
    if (n instanceof DependentJoinExpression) {
      QueryResult res = null;
//      if (left instanceof SequenceValue)   res = new SequenceValue();
//      else if (left instanceof BagValue)   
        res = new BagValue();
//      else                                 return bag.operatorTOSINGLE();

      for (int i=0; i<bag.getSize(); i++) {
        QueryResult tmp = bag.getResultAt(i);
        if (tmp instanceof StructValue) // tylko complex (BAG, SEQUENCE lub STRUCT)
          ((ComplexValue)res).addResult(tmp);
        else if (tmp instanceof BagValue)
          ((ComplexValue)res).addBag(tmp);
        else if (tmp instanceof SequenceValue)
          ((ComplexValue)res).addSequence(tmp);
        else
          throw new EvalException("EvalVisitor.merge("+n.getTypeAsString()+") [end of merge]");
      }
      return res;

    } else if (n instanceof ForAllExpression) {
      for (int i=0; i<bag.getSize(); i++) {
        QueryResult tmp = bag.getResultAt(i);
        if (tmp.operatorEQUAL(new BooleanValue(false)).booleanValue() == true)
          return new BooleanValue(false);
      }
      return new BooleanValue(true);

    } else if (n instanceof ForAnyExpression) {
      for (int i=0; i<bag.getSize(); i++) {
        QueryResult tmp = bag.getResultAt(i);
        if (tmp.operatorEQUAL(new BooleanValue(true)).booleanValue() == true)
          return new BooleanValue(true);
      }
      return new BooleanValue(false);

    } else if (n instanceof NavigationExpression) {
      QueryResult res = null;
      if (left instanceof SequenceValue)   res = new SequenceValue();
      else if (left instanceof BagValue)   res = new BagValue();
      else                                 return bag.operatorTOSINGLE();

      for (int i=0; i<bag.getSize(); i++) {
        QueryResult tmp = bag.getResultAt(i);
        if (tmp instanceof SingleValue || tmp instanceof StructValue)
          ((ComplexValue)res).addResult(tmp);
        else if (tmp instanceof BagValue) // TODO: co w przypadku sekwencji?
          ((ComplexValue)res).addBag(tmp);
        else if (tmp instanceof SequenceValue) // TODO: co w przypadku bagow?
          ((ComplexValue)res).addSequence(tmp);
        else
          throw new EvalException("EvalVisitor.merge("+n.getTypeAsString()+") [end of merge]");
      }
      return res;

    } else if (n instanceof SelectionExpression) {
      QueryResult res = null;
      if (left instanceof SequenceValue)   res = new SequenceValue();
      else if (left instanceof BagValue)   res = new BagValue();
      else                                 return bag.operatorTOSINGLE();
      // bag ma w sobie albo bag{} albo bag{i} - patrz wyzej - combine()
      for (int i=0; i<bag.getSize(); i++) {
        QueryResult tmp = bag.getResultAt(i);
        if (tmp instanceof BagValue)
          ((ComplexValue)res).addBag(tmp);
        else
          throw new EvalException("EvalVisitor.merge("+n.getTypeAsString()+") [wrong value]");
      }
      return res;

    } else if (n instanceof SortingExpression) {
      SequenceValue res = new SequenceValue();
      for (int i=0; i<bag.getSize(); i++) {
        QueryResult tmp = bag.getResultAt(i);
        if (tmp instanceof StructValue) // tylko complex (BAG, SEQUENCE lub STRUCT)
          res.addResult(tmp);
        else if (tmp instanceof BagValue)
          res.addBag(tmp);
        else if (tmp instanceof SequenceValue)
          res.addSequence(tmp);
        else
          throw new EvalException("EvalVisitor.merge("+n.getTypeAsString()+") [wrong value]");
      }
      for (int z=1; z<((ComplexValue)res.getResultAt(0)).getSize(); z++)
        for (int i=0; i<res.getSize()-1; i++)
          for (int j=i+1; j<res.getSize(); j++)
            compareAndReplace(res, i, j, z);
      for (int i=0; i<res.getSize(); i++)
        res.setResultAt(i, ((ComplexValue)res.getResultAt(i)).getResultAt(0));
      return res;

    } else if (n instanceof TransitiveClosureExpression) {
      BagValue res = new BagValue();
      for (int i=0; i<bag.getSize(); i++)
        res.addBag(bag.getResultAt(i));
      return res;
    }
    throw new EvalException("EvalVisitor.merge("+n.getTypeAsString()+") [end of merge]");
  }


// for SORTING purposes (order by)
/**
 * Auxilary method responsible for evaluating binary non algebraic expression 
 * node - precisely {@link SortingExpression}. Its goal is to compare two
 * elemens with indexes <code>i</code> and <code>j</code> and replace them 
 * if there is such need.
 * @param bag is the set of all partial results yielded by method <code>combine</code>.
 * @param i the index of the first element to compare.
 * @param j the index of the second element to compare.
 * @param z the number of the sorting key. 
 * @exception TypeMismatchException if evaluation in this node fails.
 * @see BinaryExpression
 * @see NonAlgebraicExpression
 * @see SequenceValue
 */
  public void compareAndReplace(SequenceValue seq, int i, int j, int z) throws TypeMismatchException {
    ComplexValue l = (ComplexValue)seq.getResultAt(i);
    ComplexValue r = (ComplexValue)seq.getResultAt(j);
    if (z == 1) {
      if (compare(l.getResultAt(z), r.getResultAt(z))) {
        seq.setResultAt(i, r);
        seq.setResultAt(j, l);
      }
    } else if (z > 1) {
      if (l.getResultAt(z-1).operatorEQUAL(r.getResultAt(z-1)).booleanValue() == true)
        if (compare(l.getResultAt(z), r.getResultAt(z))) {
          seq.setResultAt(i, r);
          seq.setResultAt(j, l);
        }
    }
  }

/**
 * Auxilary method responsible for evaluating binary non algebraic expression 
 * node - precisely {@link SortingExpression}. Its goal is to compare two
 * elements and if they are comparable and the first element is greater than
 * second the result is <code>true</code>, otherwise the result 
 * is <code>false</code>.
 * @param l the first element to compare.
 * @param r the second element to compare.
 * @exception TypeMismatchException if evaluation in this node fails.
 * @return the result of comparison.
 * @see BinaryExpression
 * @see NonAlgebraicExpression
 * @see QueryResult
 */
  public boolean compare(QueryResult l, QueryResult r) throws TypeMismatchException {
    if ( l.getClass() != r.getClass() )
      throw new TypeMismatchException("[SortingExpression] EvalVisitor.compare()", l, "cant be compared with", r, "[different types]");

    if (l instanceof BooleanValue)
      return (l.operatorEQUAL(new BooleanValue(true)).operatorCONJUNCTION(r.operatorEQUAL(new BooleanValue(false))).getValue().booleanValue()) ? true : false;
    else if (l instanceof IntegerValue || l instanceof DoubleValue)
      return l.operatorGREATER(r).booleanValue();
    else if (l instanceof StringValue) {
      Collator myCollator = Collator.getInstance(new Locale("pl","PL"));
      return (myCollator.compare(((StringValue)l).getValue(), ((StringValue)r).getValue()) > 0) ? true : false;
//      return (((StringValue)l).getValue().compareTo(((StringValue)r).getValue()) > 0) ? true : false;
    } else
      throw new TypeMismatchException("[SortingExpression] EvalVisitor.compare()", l, "cant be compared with", r, "[not comparable types]");
  }
}
