package pl.edu.pjwstk.yaod.yaql.visitors;

import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.*;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.expressions.*;
import pl.edu.pjwstk.yaod.yaql.abstractsyntaxtree.statements.*;

/**
 * The <code>Visitor</code> interface represents the visitor pattern used for
 * traversing the abstract syntax tree. It consists of many <code>visit</code>
 * methods - each for one main node in abstract syntax tree. The class which
 * implements this interface, ought to implement all those methods, in order to
 * define some other (than evaluation) interpretation.
 * @see AbstractSyntaxTree
 * @see EvalVisitor
 * @see EvalException
 * @author Jakub Trzetrzelewski, jakub_trzetrzelewski@yahoo.com
 * @version	1.0
 */
public interface Visitor {

/**
 * Method responsible for traversing certain single node in abstract syntax tree.
 * @param n is a certain single expression node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see SingleExpression
 */
  public void visit(SingleExpression n) throws EvalException;

/**
 * Method responsible for traversing certain unary node in abstract syntax tree.
 * @param n is a certain unary expression node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see UnaryExpression
 * @see ParametricExpression
 */
  public void visit(ParametricExpression n) throws EvalException;

/**
 * Method responsible for traversing certain unary node in abstract syntax tree.
 * @param n is a certain unary expression node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see UnaryExpression
 * @see NonParametricExpression
 */
  public void visit(NonParametricExpression n) throws EvalException;

/**
 * Method responsible for traversing certain binary node in abstract syntax tree.
 * @param n is a certain binary expression node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see BinaryExpression
 * @see AlgebraicExpression
 */
  public void visit(AlgebraicExpression n) throws EvalException;

/**
 * Method responsible for traversing certain binary node in abstract syntax tree.
 * @param n is a certain binary expression node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see BinaryExpression
 * @see NonAlgebraicExpression
 */
  public void visit(NonAlgebraicExpression n) throws EvalException;

/**
 * Method responsible for traversing certain ternary node in abstract syntax tree.
 * @param n is a certain ternary expression node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see TernaryExpression
 */
  public void visit(TernaryExpression n) throws EvalException;

/**
 * Method responsible for traversing node representing empty statement 
 * (<code> ;</code>) in abstract syntax tree.
 * @param n is a empty statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see EmptyStatement
 */
  public void visit(EmptyStatement n) throws EvalException;   

/**
 * Method responsible for traversing node representing expression statement 
 * (<code>... ;</code>)in abstract syntax tree.
 * @param n is a expression statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see ExpressionStatement
 */
  public void visit(ExpressionStatement n) throws EvalException;

/**
 * Method responsible for traversing node representing print statement 
 * (<code>print (...) ;</code>) in abstract syntax tree.
 * @param n is a print statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see PrintStatement
 */
  public void visit(PrintStatement n) throws EvalException;     

/**
 * Method responsible for traversing node representing xmlprint statement 
 * (<code>xmlprint (...) ;</code>) in abstract syntax tree.
 * @param n is a xmlprint statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see XmlPrintStatement
 */
  public void visit(XmlPrintStatement n) throws EvalException;     

/**
 * Method responsible for traversing node representing show statement 
 * (<code>show (...) ;</code>) in abstract syntax tree.
 * @param n is a show statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see ShowStatement
 */
  public void visit(ShowStatement n) throws EvalException;      

/**
 * Method responsible for traversing node representing block statement 
 * (<code>{ ... }</code>) in abstract syntax tree.
 * @param n is a block statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see BlockStatement
 */
  public void visit(BlockStatement n) throws EvalException;     

/**
 * Method responsible for traversing node representing empty block statement 
 * (<code>{   }</code>) in abstract syntax tree.
 * @param n is a empty block statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see EmptyBlockStatement
 */
  public void visit(EmptyBlockStatement n) throws EvalException;

/**
 * Method responsible for traversing node representing if statement 
 * (<code>if (...) ...</code>) in abstract syntax tree.
 * @param n is a if statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see IfStatement
 */
  public void visit(IfStatement n) throws EvalException;        

/**
 * Method responsible for traversing node representing if else statement 
 * (<code>if (...) ... else ...</code>) in abstract syntax tree.
 * @param n is a if else statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see IfElseStatement
 */
  public void visit(IfElseStatement n) throws EvalException;    

/**
 * Method responsible for traversing node representing while statement 
 * (<code>while (...) ...</code>) in abstract syntax tree.
 * @param n is a while statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see WhileStatement
 */
  public void visit(WhileStatement n) throws EvalException;     

/**
 * Method responsible for traversing node representing for statement 
 * (<code>for (...; ...; ...) ...</code>) in abstract syntax tree.
 * @param n is a for statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see ForStatement
 */
  public void visit(ForStatement n) throws EvalException;     

/**
 * Method responsible for traversing node representing do while statement 
 * (<code>do ... while (...) ;</code>) in abstract syntax tree.
 * @param n is a do while statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see DoWhileStatement
 */
  public void visit(DoWhileStatement n) throws EvalException;   

/**
 * Method responsible for traversing node representing for each statement 
 * (<code>for each ... do ...</code>) in abstract syntax tree.
 * @param n is a for each statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see ForEachStatement
 */
  public void visit(ForEachStatement n) throws EvalException;   

/**
 * Method responsible for traversing node representing delete statement 
 * (<code>delet ... ;</code>) in abstract syntax tree.
 * @param n is a delete statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see DeleteStatement
 */
  public void visit(DeleteStatement n) throws EvalException;    

/**
 * Method responsible for traversing node representing rename to statement 
 * (<code>rename ... to ... ;</code>) in abstract syntax tree.
 * @param n is a rename to statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see RenameToStatement
 */
  public void visit(RenameToStatement n) throws EvalException;    

/**
 * Method responsible for traversing node representing insert into statement 
 * (<code>insert ... into ... ;</code>) in abstract syntax tree.
 * @param n is a insert into statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see InsertIntoStatement
 */
  public void visit(InsertIntoStatement n) throws EvalException;    

/**
 * Method responsible for traversing node representing update to statement 
 * (<code>update ... to ... ;</code>)in abstract syntax tree.
 * @param n is a update to statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see UpdateToStatement
 */
  public void visit(UpdateToStatement n) throws EvalException;    

/**
 * Method responsible for traversing node representing empty procedure
 * declaration statement (procedure without parameters) 
 * (<code>procedure/function id() { ... }</code>) in abstract syntax tree.
 * @param n is a empty procedure declaration statement node in abstract 
 * syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see EmptyProcedureDeclarationStatement
 */
  public void visit(EmptyProcedureDeclarationStatement n) throws EvalException; 

/**
 * Method responsible for traversing node representing procedure declaration
 * statement (<code>procedure/function id(...) { ... }</code>) in abstract
 * syntax tree.
 * @param n is a procedure declaration statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see ProcedureDeclarationStatement
 */
  public void visit(ProcedureDeclarationStatement n) throws EvalException;    

/**
 * Method responsible for traversing node representing break statement 
 * (<code>break ;</code>)in abstract syntax tree.
 * @param n is a break statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see BreakStatement
 */
  public void visit(BreakStatement n) throws EvalException;    

/**
 * Method responsible for traversing node representing return statement 
 * (<code>return ...;</code>) in abstract syntax tree.
 * @param n is a return statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see ReturnStatement
 */
  public void visit(ReturnStatement n) throws EvalException;    

/**
 * Method responsible for traversing node representing empty return statement 
 * (<code>return ;</code>) in abstract syntax tree.
 * @param n is a empty return statement node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see EmptyReturnStatement
 */
  public void visit(EmptyReturnStatement n) throws EvalException;    

/**
 * Method responsible for traversing node representing  pair statement list
 * (<code>... ; ... ;</code>) in abstract syntax tree.
 * @param n is a pair statement list node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see PairStatementList
 */
  public void visit(PairStatementList n) throws EvalException;   

/**
 * Method responsible for traversing node representing last statement list
 * (<code>... ;</code>) in abstract syntax tree.
 * @param n is a last statement list node in abstract syntax tree.
 * @exception EvalException if evaluation in this node fails.
 * @see LastStatementList
 */
  public void visit(LastStatementList n) throws EvalException;

}